/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 997 $
 * $Date: 2020-03-19 13:49:34 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.general.dtypes;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Locale;
import java.util.Random;
import java.util.TreeSet;

import javax.swing.JFrame;
import javax.swing.WindowConstants;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

import org.apache.commons.math3.complex.Complex;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.BlockRealMatrix;
import org.apache.commons.math3.linear.QRDecomposition;
import org.apache.commons.math3.transform.DftNormalization;
import org.apache.commons.math3.transform.FastFourierTransformer;
import org.apache.commons.math3.transform.TransformType;
import org.apache.commons.math3.transform.TransformUtils;

import at.lcm.bu21.general.dtypes.preview.TPeriodicPreview;
import at.lcm.bu21.general.plot.Plot;
import at.lcm.bu21.general.plot.SubPlot;
import at.lcm.bu21.general.utils.Clone;
import at.lcm.bu21.general.utils.CloneException;

/**
 * Data reprensentation of periodic signals
 * 
 * @author Guenther Weidenholzer
 */
public class TPeriodic extends TData {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/**  */
	private static final int SIG_REGULAR = 0;

	/**  */
	private static final int SIG_EVEN = 1;

	/**	 */
	private static final int SIG_ODD = 2;

	/**  */
	private static final double POWER_TRESHOLD = 1e-6;

	/**  */
	private static final double AMPLITUDE_TRESHOLD = 1e-4;

	/**  */
	private static final double resolution = (5 * Math.PI) / 180;

	/** Enable compress data (coeffs) */
	private boolean compress = true;

	/** DC offset of signal */
	private double DCOffset = 0.0;

	/** List of fourier-coefficients */
	private ArrayList<Coeff> coeffs = new ArrayList<Coeff>();

	/**
	 * Class Coeff represents one fourier-coefficient. It includes order of the
	 * harmonic, type (SIN or COS) and magnitude.
	 * 
	 * @author Guenther Weidenholzer
	 */
	private class Coeff implements Serializable, Cloneable {

		/**
		 * Serialization version number
		 */
		private static final long serialVersionUID = 1L;

		/**
		 * Coefficient is real
		 */
		static final int COS = 0;

		/**
		 * Coefficient is imaginary
		 */
		static final int SIN = 1;

		/**
		 * Order of harmonic
		 */
		int order;

		/**
		 * Type: SIN or COS
		 */
		int type;

		/**
		 * Magnitude of harmonic
		 */
		double value;

		/**
		 * @param order
		 *            Order of harmonics
		 * @param type
		 *            SIN or COS
		 * @param value
		 *            Magnitude of harmonic
		 */
		Coeff(int order, int type, double value) {
			this.order = order;
			this.type = type;
			this.value = value;
		}

		// /**
		// * Used in method equals
		// * @return
		// */
		// private TPeriodic getOuterType() {
		// return TPeriodic.this;
		// }

		/**
		 * Equality test
		 * 
		 * @param obj
		 *            object to test against
		 * @return true if equal
		 */
		@Override
		public boolean equals(Object obj) {
			if (this == obj) {
				return true;
			}
			if (obj == null) {
				return false;
			}
			if (this.getClass() != obj.getClass()) {
				return false;
			}
			Coeff other = (Coeff) obj;
			// if (!getOuterType().equals(other.getOuterType()))
			// return false;
			if (this.order != other.order) {
				return false;
			}
			if (this.type != other.type) {
				return false;
			}
			if (Double.doubleToLongBits(this.value) != Double.doubleToLongBits(other.value)) {
				return false;
			}
			return true;
		}
	}

	/**
	 * create 0-signal
	 */
	public TPeriodic() {
	}

	/**
	 * Calculates fourier representation from sample points
	 * 
	 * @param angle
	 *            array of angles
	 * @param vec
	 *            array of sampled values
	 */
	public TPeriodic(double angle[], double vec[]) {

		int N = angle.length; // length of data vector

		if (vec.length != N) {
			throw new IllegalArgumentException("Vectors are of different length!");
		}

		this.periodicFit(angle.clone(), vec, TPeriodic.SIG_REGULAR, -1);
	}

	/**
	 * Calculates fourier representation from sample points
	 * 
	 * @param angle
	 *            array of angles
	 * @param vec
	 *            array of sampled values
	 * @param maxOrder
	 *            maximum fitting order
	 */
	public TPeriodic(double angle[], double vec[], int maxOrder) {

		int N = angle.length; // length of data vector

		if (vec.length != N) {
			throw new IllegalArgumentException("Vectors have different length!");
		}

		this.periodicFit(angle.clone(), vec, TPeriodic.SIG_REGULAR, maxOrder);
	}

	/**
	 * Create single wave signal
	 * 
	 * @param order
	 *            order of signal
	 * @param magnitude
	 *            magnitude of signal
	 * @param phase
	 *            phase of signal
	 */
	public TPeriodic(int order, double magnitude, double phase) {

		double real, imag;

		if (order != 0) {

			real = magnitude * Math.cos(phase);
			imag = magnitude * Math.sin(-phase);

			this.coeffs.add(new Coeff(order, Coeff.COS, real));
			this.coeffs.add(new Coeff(order, Coeff.SIN, imag));

		}

	}

	/**
	 * Creates random signal for testing purposes
	 * 
	 * @return random signal
	 * @throws CloneException
	 *             Unable to clone (serialize)
	 */
	public static TPeriodic getRandom() throws CloneException {

		TPeriodic tmp = new TPeriodic();
		Random r = new Random();

		tmp = tmp.offset(r.nextGaussian());
		for (int k = 1; k <= 10; k++) {
			tmp = tmp.add(new TPeriodic(k, r.nextFloat() / k / k, r.nextFloat() * 2 * Math.PI));
		}

		return tmp;

	}

	/**
	 * Returns list of used orders
	 * 
	 * @return list of used orders
	 */
	public int[] usedOrders() {

		TreeSet<Integer> orders = new TreeSet<Integer>();

		for (Coeff c : this.coeffs) {
			orders.add(c.order);
		}

		int n = orders.size();

		Iterator<Integer> iter = orders.iterator();

		int[] tmp = new int[n];

		for (int k = 0; k < n; k++) {
			tmp[k] = iter.next();
		}

		return tmp;

	}

	/**
	 * Setup the TPeriodic object according the given measurement vectors time
	 * and data. Equidistant steps are not required. The first harmonic will be
	 * matched to the highest peak of the power density spectrum!
	 * 
	 * @param time
	 *            Time vector
	 * @param data
	 *            Data vector
	 * @return identified signal frequency
	 * @throws TDataException
	 *             Wrong dimension
	 */
	public double setMeasurement(double[] time, double[] data) throws TDataException {

		int len1 = time.length;

		// check length
		if (len1 != data.length) {
			throw new TDataException("different length of time and data vectors!");
		}

		// get frequency (clone time and data array so that they are not
		// changed)
		double freq = TPeriodic.getFreq(time.clone(), data.clone());

		double[] phi = new double[len1];
		for (int k = 0; k < len1; k++) {
			phi[k] = 2 * Math.PI * freq * time[k];
		}

		this.periodicFit(phi, data, TPeriodic.SIG_REGULAR, -1);

		return freq;
	}

	/**
	 * Use fast fourier transform to calculate coefficients Requirements: Number
	 * of points N=2^k Phase phi = (0:N)/(2*pi)
	 * 
	 * @param data
	 *            Vector of sample points
	 * @throws TDataException
	 *             Wrong number of elements
	 */
	public void setFFT(double[] data) throws TDataException {

		int len = data.length;

		try {
			TransformUtils.exactLog2(len); // throws exception if not 2^x
		} catch (Exception e) {
			throw new TDataException(e.getMessage());
		}

		FastFourierTransformer fft = new FastFourierTransformer(DftNormalization.STANDARD);
		Complex[] spectrum = fft.transform(data, TransformType.FORWARD);

		double a0 = spectrum[0].getReal() / len;

		double[] coeffsA = new double[(len / 2) - 1];
		double[] coeffsB = new double[(len / 2) - 1];

		for (int k = 1; k < (len / 2); k++) {

			coeffsA[k - 1] = (2.0 * spectrum[k].getReal()) / len;
			coeffsB[k - 1] = (-2.0 * spectrum[k].getImaginary()) / len;

		}

		this.setCoeffs(a0, coeffsA, coeffsB);
		this.compress();
	}

	/**
	 * Finds the highest peak in the power density spectrum and returns the
	 * frequency. The function changes values in arrays time and data!
	 * 
	 * @param time
	 * @param data
	 * @return Frequency of first harmonic
	 */
	private static double getFreq(double[] time, double[] data) {

		double freq = 0;

		int len1 = time.length;

		double ts = time[1] - time[0];

		for (int k = 0; k < len1; k++) {
			// check if sample points are equidistant
			if ((k > 1) && ((Math.abs(time[k] - time[k - 1] - ts) / ts) > 1e-3)) {
				double[][] dataI = TPeriodic.interp(time, data);
				time = dataI[0];
				data = dataI[1];
				// recalculate ts and len1
				ts = time[1] - time[0];
				len1 = time.length;
				break;
			}
		}

		// Calculate mean value;
		double mean = 0.0;
		for (int k = 0; k < len1; k++) {
			mean += data[k];
		}
		mean = mean / data.length;

		for (int k = 0; k < len1; k++) {
			// remove mean and apply hann-window
			data[k] = (data[k] - mean) * 0.5 * (1.0 - Math.cos((2 * Math.PI * k) / len1));
		}

		int logn = (int) Math.ceil(((Math.log(len1) / Math.log(2.0)) + 6.0));
		int N = 1 << logn;

		// Zero-padding
		double[] data2 = Arrays.copyOf(data, N);

		FastFourierTransformer transformer = new FastFourierTransformer(DftNormalization.STANDARD);
		Complex[] spectrum = transformer.transform(data2, TransformType.FORWARD);

		int idxMax = 0;
		double max = 0.0;

		for (int k = 1; k < (N / 2); k++) {
			double pdens = Math.pow(spectrum[k].abs(), 2.0);
			if (pdens > max) {
				max = pdens;
				idxMax = k;
			}
		}

		freq = idxMax / (ts * N);

		return freq;
	}

	/**
	 * Calculates fourier representation from sample points
	 * 
	 * @param Angle
	 *            array of angles
	 * @param Vec
	 *            array of sampled values
	 */
	public TPeriodic(Double Angle[], Double Vec[]) {

		int N = Angle.length; // length of data vector

		double vec[] = new double[N];
		double angle[] = new double[N];

		if (vec.length != N) {
			throw new IllegalArgumentException("Vectors have different length!");
		}
		for (int q = 0; q < N; q++) {
			vec[q] = Vec[q].doubleValue();
			angle[q] = Angle[q].doubleValue();
		}
		this.periodicFit(angle, vec, TPeriodic.SIG_REGULAR, -1);
	}

	/**
	 * Sorting utility function
	 * 
	 * @param data
	 *            double array to be sorted
	 * @return Index array data[] will be sorted after execution! Clone before
	 *         if needed!
	 */
	private static int[] insertionSort(double data[]) {

		int N = data.length;

		int idx[] = new int[N];
		for (int i = 0; i < N; i++) {
			idx[i] = i;
		}

		// insertion sort datas
		for (int i = 1; i < N; i++) {
			Double t = data[i];
			int u = idx[i];
			int j = i - 1;
			while ((j >= 0) && (data[j] > t)) {
				data[j + 1] = data[j];
				idx[j + 1] = idx[j];
				j--;
			}
			data[j + 1] = t;
			idx[j + 1] = u;
		}

		return (idx);
	}

	/**
	 * Calculates the fourier representation of a given signal
	 * 
	 * @param angle
	 *            array of angles
	 * @param vec
	 *            array of sampled values
	 * @param sigType
	 *            not ready implemented; use TPeriodic.SIG_REGULAR
	 * @param nMaxOrders
	 *            number of max order (zero or negative if not used)
	 */
	private void periodicFit(double angle[], double vec[], int sigType, int nMaxOrders) {

		int N = angle.length; // length of data vector

		// wrap at 2pi
		for (int i = 0; i < angle.length; i++) {
			angle[i] = Math.IEEEremainder(angle[i] - Math.PI, 2.0 * Math.PI) + Math.PI;
		}

		int idx[] = TPeriodic.insertionSort(angle);

		double maxAngle = 0.0;

		for (int i = 0; i < (N - 1); i++) {
			double tmp = angle[i + 1] - angle[i];
			if (tmp > maxAngle) {
				maxAngle = tmp;
			}
		}

		// check at overflow
		double tmp = angle[0] - (angle[N - 1] - (2.0 * Math.PI));
		if (tmp > maxAngle) {
			maxAngle = tmp;
		}

		// These: Auch fr nicht quidistante Abtastung ist es ist
		// hinreichend,
		// wenn die hchste Harmonische das Abtasttheorem erfllt. Faktor
		// 0.9
		// als Reserve.
		int maxOrder = (int) ((0.9 * Math.PI) / maxAngle);

		if (maxOrder < 2) {
			throw new IllegalArgumentException("No periodic fitting possible!");
		}

		// limit maxOrder
		if ((nMaxOrders > 0) && (maxOrder > nMaxOrders)) {
			maxOrder = nMaxOrders;
		}

		int NA, NB;

		switch (sigType) {
		case SIG_REGULAR:
			if (maxOrder > ((N - 1) / 2)) {
				maxOrder = (N - 1) / 2; // Gleichungssystem muss lbar sein
			}
			NA = maxOrder;
			NB = maxOrder;
			break;

		case SIG_EVEN:
			if (maxOrder > (N - 1)) {
				maxOrder = (N - 1); // Gleichungssystem muss lsbar sein
			}
			NA = maxOrder;
			NB = 0;
			break;

		case SIG_ODD:
			if (maxOrder > (N - 1)) {
				maxOrder = (N - 1); // Gleichungssystem muss lsbar sein
			}
			NA = 0;
			NB = maxOrder;
			break;

		default:
			throw new IllegalArgumentException("Illegal signal type!");
		}

		BlockRealMatrix A = new BlockRealMatrix(N, 1 + NA + NB);
		BlockRealMatrix B = new BlockRealMatrix(N, 1);

		for (int i = 0; i < N; i++) {

			A.setEntry(i, 0, 1.0);

			for (int j = 1; j <= NA; j++) {
				A.setEntry(i, j, Math.cos(j * angle[i]));
			}

			for (int j = 1; j <= NB; j++) {
				A.setEntry(i, NA + j, Math.sin(j * angle[i]));
			}

			B.setEntry(i, 0, vec[idx[i]]);
		}

		// double coeffs[] = A.solve(B).getColumnPackedCopy();
		double coeffs[] = new QRDecomposition(A).getSolver().solve(B).getColumn(0);

		this.setCoeffs(coeffs[0], Arrays.copyOfRange(coeffs, 1, NA + 1),
				Arrays.copyOfRange(coeffs, NA + 1, NA + NB + 1));

	}

	/**
	 * Saves relevant fourier-coefficients
	 * 
	 * @param A0
	 *            DC-Offset
	 * @param coeffsA
	 *            array of coeffs
	 * @param coeffsB
	 *            array pf coeffs
	 */
	public void setCoeffs2(double A0, ArrayRealVector coeffsA, ArrayRealVector coeffsB) {
		this.coeffs.clear();
		this.DCOffset = A0;

		for (int k = 0; k < coeffsA.getDimension(); k++) {
			this.coeffs.add(new Coeff(k + 1, Coeff.COS, coeffsA.getEntry(k)));

		}

		for (int k = 0; k < coeffsB.getDimension(); k++) {
			this.coeffs.add(new Coeff(k + 1, Coeff.SIN, coeffsB.getEntry(k)));

		}
	}

	/**
	 * Saves relevant fourier-coefficients
	 * 
	 * @param A0
	 *            DC-Offset
	 * @param coeffsA
	 *            array of coeffs
	 * @param coeffsB
	 *            array pf coeffs
	 */
	public void setCoeffs(double A0, double[] coeffsA, double[] coeffsB) {
		this.coeffs.clear();
		this.DCOffset = A0;

		for (int k = 0; k < coeffsA.length; k++) {
			this.coeffs.add(new Coeff(k + 1, Coeff.COS, coeffsA[k]));

		}

		for (int k = 0; k < coeffsB.length; k++) {
			this.coeffs.add(new Coeff(k + 1, Coeff.SIN, coeffsB[k]));

		}
	}

	/**
	 * Suppress weak harmonics to save memory
	 */
	public void compress() {
		if (this.compress) {
			ArrayList<Coeff> coeffs = new ArrayList<Coeff>();
			double p_dc = Math.pow(this.DCOffset, 2.0) / 4.0;
			double p_ac = 0;
			double maxCoeffValue = 0;

			// @TODO Fallabfrage wenn p_ac==0 !!! wegen div /0

			for (Coeff coeff : this.coeffs) {

				p_ac += Math.pow(coeff.value, 2.0) / 2.0;

				if (Math.abs(coeff.value) > maxCoeffValue) {
					maxCoeffValue = Math.abs(coeff.value);
				}
			}

			// DC offset vernachlssigbar
			// TODO: sqrt nicht richtig. Leistungsverhltnis sollte berechnet
			// werden!
			if (Math.sqrt(p_dc / p_ac) < TPeriodic.POWER_TRESHOLD) {
				this.DCOffset = 0;
			}

			coeffs.addAll(this.coeffs);
			coeffs.sort(new Comparator<Coeff>() {

				@Override
				public int compare(Coeff o1, Coeff o2) {
					double v1, v2;

					v1 = Math.abs(o1.value);
					v2 = Math.abs(o2.value);
					if (v1 < v2) {
						return 1;
					} else if (v1 > v2) {
						return -1;
					} else {
						return 0;
					}
				}

			});

			double tmp, sum = 0.0;

			// Koeffizienten lschen und neu eintragen
			this.coeffs.clear();

			for (Coeff c : coeffs) {
				tmp = (p_ac - sum) / p_ac;

				// Koeffizient hinzufgen wenn:
				// erste Bedingung: Leistung nicht erreicht
				// oder zweite Bedingung: Amplitude gro genug
				if ((tmp > TPeriodic.POWER_TRESHOLD)
						|| (Math.abs(c.value / maxCoeffValue) > TPeriodic.AMPLITUDE_TRESHOLD)) {
					sum += Math.pow(c.value, 2.0) / 2.0;
					this.coeffs.add(c);
				} else {
					break;
				}
			}
		}
	}

	/**
	 * Get enable to compress data (coeffs)
	 * 
	 * @return Enable compress
	 */
	public boolean getCompress() {
		return this.compress;
	}

	/**
	 * Set enable to compress data (coeffs)
	 * 
	 * @param compress
	 *            Enable compress
	 */
	public void setCompress(boolean compress) {
		this.compress = compress;
		this.compress();
	}

	/**
	 * Returns complex number representation
	 * 
	 * @param order
	 *            number of order
	 * @return complex number
	 */
	public Complex getComplex(int order) {
		return new Complex(this.getCoeffA(order), -this.getCoeffB(order));
	}

	/**
	 * Returns complex number representation
	 * 
	 * @return Complex number
	 */
	public Complex getComplex() {
		return this.getComplex(1);
	}

	/**
	 * Returns complex number representation
	 * 
	 * @return complex number
	 * @throws TDataException
	 *             Unable to create data-object
	 */
	public TNumeric getComplexTN() throws TDataException {
		return new TNumeric(this.getComplex());
	}

	// /**
	// * Returns fourier-coefficients
	// *
	// * @return double array of actual cos-coefficients.
	// */
	// private double[] getCoeffsA() {
	// double coeffsA[] = new double[nMaxOrder];
	//
	// for (Coeff coeff : this.coeffs) {
	// if (coeff.type == Coeff.COS) {
	// coeffsA[coeff.order - 1] = coeff.value;
	// }
	// }
	// return (coeffsA);
	// }

	/**
	 * Returns fourier-coefficient
	 * 
	 * @param order
	 *            number of order
	 * @return double cos-coefficient.
	 */
	public double getCoeffA(int order) {
		double c = 0;

		for (Coeff coeff : this.coeffs) {
			if ((coeff.order == order) && (coeff.type == Coeff.COS)) {
				c = coeff.value;
				break;
			}
		}
		return c;
	}

	/**
	 * Returns the fourier-coefficients of the given harmonics
	 * 
	 * @param order
	 *            orders of harmonics
	 * @return magnitudes
	 */
	public double[] getCoeffA(int[] order) {
		double[] ret = new double[order.length];
		for (int k = 0; k < order.length; k++) {
			ret[k] = this.getCoeffA(order[k]);
		}
		return ret;
	}

	/**
	 * Returns fourier-coefficient
	 * 
	 * @param order
	 *            number of order
	 * @return double sin-coefficient.
	 */
	public double getCoeffB(int order) {
		double c = 0;

		for (Coeff coeff : this.coeffs) {
			if ((coeff.order == order) && (coeff.type == Coeff.SIN)) {
				c = coeff.value;
				break;
			}
		}
		return c;
	}

	// /**
	// * Returns fourier-coefficients
	// *
	// * @return double array of actual sin-coefficients.
	// */
	// private double[] getCoeffsB() {
	// double coeffsB[] = new double[nMaxOrder];
	//
	// for (Coeff coeff : this.coeffs) {
	// if (coeff.type == Coeff.SIN) {
	// coeffsB[coeff.order - 1] = coeff.value;
	// }
	// }
	// return (coeffsB);
	// }

	/**
	 * Returns the fourier-coefficients of the given harmonics
	 * 
	 * @param order
	 *            orders of harmonics
	 * @return magnitudes
	 */
	public double[] getCoeffB(int[] order) {
		double[] ret = new double[order.length];
		for (int k = 0; k < order.length; k++) {
			ret[k] = this.getCoeffB(order[k]);
		}
		return ret;
	}

	/**
	 * Samples the signal at given angles
	 * 
	 * @param angle
	 *            array of angles
	 * @return double array of sampled values
	 */
	public double[] valueAt(double[] angle) {
		double[] value = new double[angle.length];

		// Set DC Offset
		for (int k = 0; k < angle.length; k++) {
			value[k] = this.DCOffset;
		}

		for (Coeff coeff : this.coeffs) {
			for (int k = 0; k < angle.length; k++) {
				switch (coeff.type) {
				case Coeff.SIN:
					value[k] += coeff.value * Math.sin(coeff.order * angle[k]);
					break;
				case Coeff.COS:
					value[k] += coeff.value * Math.cos(coeff.order * angle[k]);
					break;
				}
			}
		}
		return value;
	}

	/**
	 * Samples the signal at given angle
	 * 
	 * @param angle
	 *            angle
	 * @return sampled value
	 */
	public double valueAt(double angle) {
		double[] angle2 = new double[1];
		double[] value2 = new double[1];

		angle2[0] = angle;
		value2 = this.valueAt(angle2);

		return value2[0];
	}

	/**
	 * Returns the highest relevant harmonic order
	 * 
	 * @return highest order
	 */
	public int getMaxOrder() {
		int maxOrder = 0;

		this.compress();
		for (Coeff coeff : this.coeffs) {
			if (coeff.order > maxOrder) {
				maxOrder = coeff.order;
			}
		}
		return maxOrder;
	}

	/**
	 * Returns the peak-peak value of the signal
	 * 
	 * @return peak-peak value
	 */
	protected double[] extrema() {

		// hchster Harmonischer
		double[] minmax = new double[2];
		double[] angle, value;
		int n;

		minmax[0] = Double.POSITIVE_INFINITY;
		minmax[1] = Double.NEGATIVE_INFINITY;
		n = (int) ((2.0 * Math.PI * this.getMaxOrder()) / TPeriodic.resolution);

		if (n > 0) {
			angle = new double[n];
			for (int k = 0; k < n; k++) {
				angle[k] = (2.0 * Math.PI * k) / n;
			}

			value = this.valueAt(angle);

			for (int k = 0; k < n; k++) {
				if (value[k] > minmax[1]) {
					minmax[1] = value[k];
				}
				if (value[k] < minmax[0]) {
					minmax[0] = value[k];
				}
			}

		} else {
			minmax[0] = this.mean();
			minmax[1] = this.mean();
		}

		return minmax;
	}

	/**
	 * Returns the peak-peak value of the signal
	 * 
	 * @return peak-peak value
	 */
	public double pp() {
		double[] minmax = this.extrema();
		return minmax[1] - minmax[0];
	}

	/**
	 * Returns the maximum value of the signal
	 * 
	 * @return maximum value
	 */
	public double max() {
		return this.extrema()[1];
	}

	/**
	 * Returns the minimum value of the signal
	 * 
	 * @return minimum value
	 */
	public double min() {
		return this.extrema()[0];
	}

	/**
	 * Returns the magnitude of the n-th harmonic
	 * 
	 * @param order
	 *            of harmonic
	 * @return magnitude
	 */
	public double magnitude(int order) {
		double mag = 0;

		for (Coeff coeff : this.coeffs) {
			if (coeff.order == order) {
				mag += Math.pow(coeff.value, 2);
			}
		}
		return Math.sqrt(mag);
	}

	/**
	 * Returns the magnitudes of the given harmonics
	 * 
	 * @param order
	 *            orders of harmonics
	 * @return magnitudes
	 */
	public double[] magnitude(int[] order) {
		double[] ret = new double[order.length];
		for (int k = 0; k < order.length; k++) {
			ret[k] = this.magnitude(order[k]);
		}
		return ret;
	}

	/**
	 * Returns the phase of the n-th harmonic
	 * 
	 * @param order
	 *            of harmonic
	 * @return phase
	 */
	public double phase(int order) {
		double real = 0;
		double imag = 0;

		for (Coeff coeff : this.coeffs) {
			if ((coeff.order == order) && (coeff.type == Coeff.COS)) {
				real = coeff.value;
			}
			if ((coeff.order == order) && (coeff.type == Coeff.SIN)) {
				imag = coeff.value;
			}
		}
		return (-Math.atan2(imag, real));
	}

	/**
	 * Returns the phases of the given harmonics
	 * 
	 * @param order
	 *            orders of harmonics
	 * @return phases
	 */
	public double[] phase(int[] order) {
		double[] ret = new double[order.length];
		for (int k = 0; k < order.length; k++) {
			ret[k] = this.phase(order[k]);
		}
		return ret;
	}

	/**
	 * Returns the RMS value of the signal
	 * 
	 * @return RMS value
	 */
	public double rms() {
		double temp = Math.pow(this.DCOffset, 2);

		for (Coeff coeff : this.coeffs) {
			temp += Math.pow(coeff.value, 2) / 2;
		}
		return Math.sqrt(temp);
	}

	/**
	 * Returns the DC Offset of the signal
	 * 
	 * @return DC Offset
	 */
	public double mean() {
		return this.DCOffset;
	}

	/**
	 * Returns the THD-value (total harmonic distortion) of the signal
	 * 
	 * @return THD-value
	 */
	public double thd() {
		double harmonics = 0;
		double base = 0;
		double temp;

		for (Coeff coeff : this.coeffs) {
			temp = Math.pow(coeff.value, 2.0) / 2.0;

			if (coeff.order > 1) {
				harmonics += temp;
			} else {
				base += temp;
			}
		}

		return (Math.sqrt(harmonics / base));
	}

	/**
	 * Scales the given signal. Source object remains unchanged!
	 * 
	 * @param scale
	 *            scale-factor
	 * @return Scaled Signal
	 */
	public TPeriodic scale(double scale) {
		TPeriodic result = new TPeriodic();

		result.DCOffset = this.DCOffset * scale;

		for (Coeff coeff : this.coeffs) {
			result.coeffs.add(new Coeff(coeff.order, coeff.type, coeff.value * scale));
		}

		return result;
	}

	// /**
	// * Scales the given signal. Source object remains unchanged!
	// * complex svaling factor possible
	// *
	// * @param factor
	// * scale-factor
	// * @return Scaled Signal
	// * @throws Exception
	// */
	// public TPeriodic scale(TNumeric factor) throws Exception {
	//
	// if(factor.getNumColumns()!=1 || factor.getNumRows()!=1){
	// throw new Exception("dimension of scale should be 1x1!");
	// }
	//
	// TPeriodic res;
	// double real=factor.getReal(0, 0);
	// double imag=factor.getImag(0, 0);
	//
	// return this.scale(real).add(this.deriv(imag));
	// }
	//

	/**
	 * Transform frequency. Source object remains unchanged! new_order =
	 * order*p/q
	 * 
	 * @param p
	 *            factor to multiply frequency with
	 * @param q
	 *            frequency division factor
	 * @return transformed signal
	 */
	public TPeriodic transformOrder(int p, int q) {
		this.compress();

		TPeriodic tmp = new TPeriodic();
		tmp.DCOffset = this.DCOffset;

		for (Coeff c : this.coeffs) {
			int newOrder = (c.order * p) / q;

			// check if result is really a integer
			if (((newOrder * q) / p) == c.order) {
				tmp.coeffs.add(new Coeff(newOrder, c.type, c.value));
			}
		}

		return tmp;
	}

	/**
	 * calculates transfer function response of this signal. Source object
	 * remains unchanged!
	 * 
	 * @param b
	 *            numerator coefficients, starting at highest s order (like
	 *            Matlab function "tf")
	 * @param a
	 *            denominator coefficients, starting at highest s order
	 * @param omega
	 *            frequency rad/s
	 * @return transfer function response of signal
	 */
	public TPeriodic tf(double[] b, double[] a, double omega) {

		this.compress();

		TPeriodic res = new TPeriodic();

		// do not integrate dc offset!
		double b0 = b[b.length - 1];
		double a0 = a[a.length - 1];
		if (a0 != 0.0) {
			res.DCOffset = ((this.DCOffset * b0) / a0);
		}

		for (int k = 1; k <= this.getMaxOrder(); k++) {

			Complex c = this.getComplex(k);
			Complex jomega_k = new Complex(0.0, omega * k);

			Complex num = new Complex(0.0, 0.0);
			Complex tmp = new Complex(1.0, 0.0);
			for (int p = b.length - 1; p >= 0; p--) {
				num = num.add(tmp.multiply(b[p]));
				tmp = tmp.multiply(jomega_k);
			}

			Complex denom = new Complex(0.0, 0.0);
			tmp = new Complex(1.0, 0.0);
			for (int p = a.length - 1; p >= 0; p--) {
				denom = denom.add(tmp.multiply(a[p]));
				tmp = tmp.multiply(jomega_k);
			}

			c = c.multiply(num).divide(denom);

			res.coeffs.add(new Coeff(k, Coeff.COS, c.getReal()));
			res.coeffs.add(new Coeff(k, Coeff.SIN, -c.getImaginary()));

		}

		return res;
	}

	/**
	 * calculates infinite, symmetric, linear skewing of the signal. The angle
	 * will be swept between -angle/2 and angle/2. Source object remains
	 * unchanged!
	 * 
	 * @param angle
	 *            Skewing angle
	 * @return skewed signal
	 * @throws CloneException
	 *             Unable to clone (serialize)
	 */
	public TPeriodic skew(double angle) throws CloneException {
		this.compress();

		TPeriodic res = this.deepClone();

		if (angle != 0.0) {
			for (Coeff c : res.coeffs) {

				// skewing angle of actual harmonic
				// 1/L*integrate(exp(j*k*angle/L*lambda), lambda, -L/2, L/2)

				double tmp = c.order * angle;
				c.value *= (2.0 / tmp) * Math.sin(tmp / 2.0);
			}
			res.compress();
		}
		return res;
	}

	/**
	 * Adds an other signal. Source object remains unchanged!
	 * 
	 * @param other
	 *            periodic to add
	 * @return sum of periodic signals
	 */
	public TPeriodic add(TPeriodic other) {
		TPeriodic result;

		result = new TPeriodic();
		result.DCOffset = this.DCOffset + other.DCOffset;
		// add this coefficients
		for (Coeff c : this.coeffs) {
			result.coeffs.add(new Coeff(c.order, c.type, c.value));
		}
		// merge other coefficients
		for (Coeff c : other.coeffs) {
			boolean match = false;
			for (Coeff c1 : result.coeffs) {
				if ((c.order == c1.order) && (c.type == c1.type)) {
					c1.value += c.value;
					match = true;
				}
			}
			if (!match) {
				result.coeffs.add(new Coeff(c.order, c.type, c.value));
			}
		}

		return result;
	}

	/**
	 * Subtracts the argument from the given signal. Source object remains
	 * unchanged!
	 * 
	 * @param other
	 *            Signal to subtract from Object
	 * @return Resulting periodic signal
	 * @throws CloneException
	 *             Unable to clone (serialize)
	 */
	public TPeriodic subtract(TPeriodic other) throws CloneException {
		return this.add(other.scale(-1.0));
	}

	/**
	 * Adds DC Offset. Source object remains unchanged!
	 * 
	 * @param offset
	 *            offset
	 * @return Signal with changed DC offset
	 */
	public TPeriodic offset(double offset) {
		TPeriodic result = new TPeriodic();

		result.DCOffset = this.DCOffset + offset;

		for (Coeff coeff : this.coeffs) {
			result.coeffs.add(new Coeff(coeff.order, coeff.type, coeff.value));
		}

		return result;
	}

	/**
	 * Removes DC Offset. Source object remains unchanged!
	 * 
	 * @return Signal without DC Offset
	 */
	public TPeriodic removeDC() {
		TPeriodic result = new TPeriodic();

//		result.DCOffset = 0.0;

		for (Coeff coeff : this.coeffs) {
			result.coeffs.add(new Coeff(coeff.order, coeff.type, coeff.value));
		}

		return result;
	}

	/**
	 * Phase shift the given signal. Source object remains unchanged!
	 * 
	 * @param shift
	 *            shift
	 * @return Phase shifted signal
	 */
	public TPeriodic shift(double shift) {
		TPeriodic result;
		int nMaxOrder = this.getMaxOrder();

		double[] coeffsA = new double[nMaxOrder];
		double[] coeffsB = new double[nMaxOrder];

		for (Coeff c : this.coeffs) {
			switch (c.type) {
			case Coeff.COS:
				coeffsA[c.order - 1] += c.value * Math.cos(c.order * shift);
				coeffsB[c.order - 1] -= c.value * Math.sin(c.order * shift);
				break;
			case Coeff.SIN:
				coeffsA[c.order - 1] += c.value * Math.sin(c.order * shift);
				coeffsB[c.order - 1] += c.value * Math.cos(c.order * shift);
				break;
			}
		}

		result = new TPeriodic();
		result.setCoeffs(this.DCOffset, coeffsA, coeffsB);
		return result;
	}

	/**
	 * Flip the given signal. Source object remains unchanged!
	 * 
	 * @return Flipped signal
	 * @throws CloneException
	 *             Unable to clone (serialize)
	 */
	public TPeriodic flip() throws CloneException {
		TPeriodic theClone;
		theClone = this.deepClone();

		for (Coeff c : theClone.coeffs) {
			if (c.type == Coeff.SIN) {
				c.value = -c.value;
			}
		}

		return theClone;
	}

	/**
	 * Calculates the time derivative of a signal. Source object remains
	 * unchanged!
	 * 
	 * @param omega
	 *            frequency in rad/s
	 * @return Time derivative of signal
	 */
	public TPeriodic deriv(double omega) {
		double value;

		TPeriodic result = new TPeriodic();

		for (Coeff c : this.coeffs) {
			value = c.value * omega * c.order;
			switch (c.type) {
			case Coeff.COS:
				result.coeffs.add(new Coeff(c.order, Coeff.SIN, -value));
				break;
			case Coeff.SIN:
				result.coeffs.add(new Coeff(c.order, Coeff.COS, value));
				break;
			}
		}

		return result;
	}

	/**
	 * Calculates the time integration of a signal. Source object remains
	 * unchanged! Attention: DC Offset will be set zero!
	 * 
	 * @param omega
	 *            frequency in rad/s
	 * @return Time integrated periodic signal
	 */
	public TPeriodic integ(double omega) {
		double value;

		TPeriodic result = new TPeriodic();

		for (Coeff c : this.coeffs) {
			value = c.value / (omega * c.order);
			switch (c.type) {
			case Coeff.COS:
				result.coeffs.add(new Coeff(c.order, Coeff.SIN, value));
				break;
			case Coeff.SIN:
				result.coeffs.add(new Coeff(c.order, Coeff.COS, -value));
				break;
			}
		}

		return result;
	}

	/**
	 * Multiplies with other signal. Source object remains unchanged!
	 * 
	 * @param other
	 *            periodic to multiply
	 * @return product of periodic signals
	 */
	public TPeriodic multiply(TPeriodic other) {
		int nMaxOrder1 = this.getMaxOrder();
		int nMaxOrder2 = other.getMaxOrder();
		int nMaxOrder;
		if (nMaxOrder1 > nMaxOrder2) {
			nMaxOrder = nMaxOrder1;
		} else {
			nMaxOrder = nMaxOrder2;
		}

		double[] A1 = new double[nMaxOrder + 1];
		double[] B1 = new double[nMaxOrder + 1];
		double[] A2 = new double[nMaxOrder + 1];
		double[] B2 = new double[nMaxOrder + 1];

		double[] Aprod = new double[2 * (nMaxOrder + 1)];
		double[] Bprod = new double[2 * (nMaxOrder + 1)];

		// // get complex Coeffs
		this.fillCoeffs(A1, B1);
		other.fillCoeffs(A2, B2);

		// multiply
		int nue;
		for (int nue1 = 0; nue1 <= nMaxOrder; nue1++) {
			for (int nue2 = 0; nue2 <= nMaxOrder; nue2++) {

				nue = nue1 - nue2;
				if (nue >= 0) {
					Aprod[nue] += 0.5 * A1[nue1] * A2[nue2];
					Aprod[nue] += 0.5 * B1[nue1] * B2[nue2];

					Bprod[nue] += 0.5 * A2[nue2] * B1[nue1];
					Bprod[nue] -= 0.5 * A1[nue1] * B2[nue2];

				} else if (nue < 0) {
					Aprod[-nue] += 0.5 * A1[nue1] * A2[nue2];
					Aprod[-nue] += 0.5 * B1[nue1] * B2[nue2];

					Bprod[-nue] -= 0.5 * A2[nue2] * B1[nue1];
					Bprod[-nue] += 0.5 * A1[nue1] * B2[nue2];

				}

				nue = nue1 + nue2;
				Aprod[nue] += 0.5 * A1[nue1] * A2[nue2];
				Aprod[nue] -= 0.5 * B1[nue1] * B2[nue2];

				Bprod[nue] += 0.5 * A1[nue1] * B2[nue2];
				Bprod[nue] += 0.5 * B1[nue1] * A2[nue2];
			}
		}

		TPeriodic result = new TPeriodic();
		result.setCoeffs(Aprod[0], Arrays.copyOfRange(Aprod, 1, Aprod.length),
				Arrays.copyOfRange(Bprod, 1, Bprod.length));
		return result;
	}

	/**
	 * Fill a and b arrays with coeffs. index=order, offset=index 0
	 * 
	 * @param A
	 *            Array A
	 * @param B
	 *            Array B
	 */
	private void fillCoeffs(double[] A, double[] B) {
		A[0] = this.DCOffset;
		B[0] = 0.0;

		for (Coeff c : this.coeffs) {
			switch (c.type) {
			case Coeff.COS:
				A[c.order] = c.value;
				break;
			case Coeff.SIN:
				B[c.order] = c.value;
				break;
			}
		}
	}

	/**
	 * Divides signal by other signal (pointwise!). Source object remains
	 * unchanged!
	 * 
	 * @param other
	 *            periodic to divide by
	 * @return pointpise quotient of periodic signals
	 * @throws CloneException
	 *             Unable to clone (serialize)
	 */
	public TPeriodic dividePointWise(TPeriodic other) throws CloneException {

		int n;
		TPeriodic result = null;

		int maxOrder = Math.max(this.getMaxOrder(), other.getMaxOrder());

		n = (int) ((2.0 * Math.PI * maxOrder) / TPeriodic.resolution);

		if (n > 0) {

			LinkedList<Double> values = new LinkedList<Double>();
			LinkedList<Double> angle = new LinkedList<Double>();

			for (int k = 0; k < n; k++) {
				double tmp = (2.0 * Math.PI * k) / n;

				double val = this.valueAt(tmp) / other.valueAt(tmp);
				if (!Double.isInfinite(val)) {
					angle.add(tmp);
					values.add(val);
				}
			}

			result = new TPeriodic(angle.toArray(new Double[0]), values.toArray(new Double[0]));

		} else {

			// only dc components!
			result = new TPeriodic();
			result.offset(this.mean() / other.mean());
		}

		return result;

	}

	/**
	 * Raises signal to the power of a. Source object remains unchanged!
	 * 
	 * @param a
	 *            exponent
	 * @return pointpise result
	 * @throws CloneException
	 *             Unable to clone (serialize)
	 */
	public TPeriodic powPointWise(double a) throws CloneException {

		int n;
		TPeriodic result = null;

		n = (int) ((2.0 * Math.PI * this.getMaxOrder()) / TPeriodic.resolution);

		if (n > 0) {

			LinkedList<Double> values = new LinkedList<Double>();
			LinkedList<Double> angle = new LinkedList<Double>();

			for (int k = 0; k < n; k++) {
				double tmp = (2.0 * Math.PI * k) / n;

				double val = Math.pow(this.valueAt(tmp), a);

				if (!Double.isInfinite(val)) {
					angle.add(tmp);
					values.add(val);
				}
			}

			result = new TPeriodic(angle.toArray(new Double[0]), values.toArray(new Double[0]));

		} else {

			// only dc components!
			result = new TPeriodic();
			result.offset(Math.pow(this.mean(), a));
		}

		return result;

	}

	/**
	 * Calculates the perimeter of the closed parametric curve x=x(phi),
	 * y=y(phi), with x(phi) being the object itself.
	 * 
	 * 
	 * @param y
	 *            function y(phi)
	 * @return enclosed area
	 */
	public double curve2dPerimeter(TPeriodic y) {

		double p = 0;

		int n;

		int maxOrder = Math.max(this.getMaxOrder(), y.getMaxOrder());

		n = (int) ((2.0 * Math.PI * maxOrder) / TPeriodic.resolution);

		TPeriodic xdot = this.deriv(1);
		TPeriodic ydot = y.deriv(1);

		// parameter increment
		double dt = (2.0 * Math.PI) / n;

		// integration loop
		for (int k = 0; k < n; k++) {
			p += dt * Math.hypot(xdot.valueAt(k * dt), ydot.valueAt(k * dt));
		}

		return p;

	}

	/**
	 * Calculates the enclosed area of the closed parametric curve x=x(phi),
	 * y=y(phi), with x(phi) being the object itself.
	 * 
	 * calculation is based on the leibnitz sector formula.
	 * (http://de.wikipedia.org/wiki/Sektorformel_von_Leibniz)
	 * 
	 * @param y
	 *            function y(phi)
	 * @return enclosed area
	 * @throws CloneException
	 *             Unable to clone (serialize)
	 */
	public double enclosedArea(TPeriodic y) throws CloneException {

		TPeriodic dx = this.deriv(1.0);
		TPeriodic dy = y.deriv(1.0);
		TPeriodic tmp = this.multiply(dy).subtract(y.multiply(dx));

		// integration over a whole period is the mean value multiplied by 2pi.
		return (tmp.mean() * Math.PI);

	}

	/**
	 * Finds zero crossing near phi0 using newton-approximation
	 * 
	 * @param phi
	 *            Initial angle for iterative search
	 * @return Angle of zero crossing
	 * @throws TDataException
	 *             Unable to find crossing
	 */
	public double findZeroCrossing(double phi) throws TDataException {

		double xi = 1; // > tolerance!
		double tolerance = 2 * Math.PI * 1e-5;
		double gradient;

		int count = 0;

		TPeriodic derivative = this.deriv(1);

		while (Math.abs(xi) > tolerance) {

			if (count > 100) {
				throw new TDataException("approximation did not converge within 100 iterations");
			}

			gradient = derivative.valueAt(phi);

			if (gradient == 0.0) {
				throw new TDataException("approximation stucked: zero gradient");
			}

			xi = -this.valueAt(phi) / gradient;

			phi += xi;
			count = count + 1;
		}

		return Math.IEEEremainder(phi - Math.PI, 2.0 * Math.PI) + Math.PI;
	}

	/**
	 * Returns new TPeriodic which contains the given harmonics only.
	 * 
	 * @param order
	 *            orders of harmonics
	 * @return Selected TPeriodic
	 */
	public TPeriodic select(int[] order) {
		TPeriodic tmp = new TPeriodic();
		tmp.DCOffset = this.DCOffset;

		for (Coeff coeff : this.coeffs) {
			for (int ord : order) {
				if (coeff.order == ord) {
					tmp.coeffs.add(new Coeff(coeff.order, coeff.type, coeff.value));
				}
			}
		}
		return tmp;
	}

	/**
	 * Apply angular symmetry
	 * 
	 * @param nSubPeriods
	 *            Number of sub-periods
	 * @return Selected TPeriodic
	 */
	public TPeriodic angularSymmetry(int nSubPeriods) {

		TPeriodic tmp = new TPeriodic();
		tmp.DCOffset = this.DCOffset;

		for (Coeff coeff : this.coeffs) {
			if ((coeff.order % nSubPeriods) == 0) {
				tmp.coeffs.add(new Coeff(coeff.order, coeff.type, coeff.value));
			}
		}
		return tmp;
	}

	/**
	 * 
	 * @param a
	 * @param b
	 * @return magnitude
	 */
	static double coeffs2magnitude(double a, double b) {

		return Math.hypot(a, b);

	}

	/**
	 * 
	 * @param a
	 * @param b
	 * @return phase
	 */
	static double coeffs2phase(double a, double b) {

		return -Math.atan2(b, a);

	}

	/**
	 * 
	 * @param magnitude
	 * @param phase
	 * @return A-coefficient
	 */
	static double magphase2a(double magnitude, double phase) {

		return magnitude * Math.cos(phase);
	}

	/**
	 * 
	 * @param magnitude
	 * @param phase
	 * @return B-coefficient
	 */
	static double magphase2b(double magnitude, double phase) {

		return magnitude * Math.sin(-phase);

	}

	/**
	 * Show preview in a new JFrame
	 * 
	 * @return Plot of preview
	 * @throws Exception
	 *             Unable to show plot
	 */
	public Plot showPreview() throws Exception {

		return this.showPreview("Periodic signal", 1, false, false);
	}

	/**
	 * Show full preview including signal plot and harmonics
	 * 
	 * @return Plot of preview
	 * @throws Exception
	 *             Unable to show plot
	 */
	public Plot showFullPreview() throws Exception {
		return this.showPreview("Periodic signal", 1, true, true);
	}

	/**
	 * Show preview in a new JFrame
	 * 
	 * @param title
	 *            Title of frame
	 * @param nPeriods
	 *            Number of periods to display
	 * @param showFundamental
	 *            Show first harmonic
	 * @param showSpectrum
	 *            Show harmonics
	 * @return Plot of preview
	 * @throws Exception
	 *             Unable to show plot
	 */
	public Plot showPreview(String title, int nPeriods, boolean showFundamental, boolean showSpectrum)
			throws Exception {
		Plot plot, plot1, plot2;
		SubPlot sub;
		ClassLoader classLoader = TPeriodic.class.getClassLoader();

		@SuppressWarnings("unchecked")
		Class<TPeriodicPreview> c = (Class<TPeriodicPreview>) classLoader
				.loadClass("at.lcm.bu21.general.dtypes.preview.TPeriodicPreview");
		Method mpreviewWave = c.getMethod("previewWave",
				new Class[] { TPeriodic.class, String.class, int.class, boolean.class });

		JFrame f = new JFrame("Periodic signal preview");
		f.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		// wave
		plot1 = (Plot) mpreviewWave.invoke(null, this, title, nPeriods, showFundamental);

		// spectrum
		if (showSpectrum) {
			Method mpreviewSpectrum = c.getMethod("previewSpectrum", new Class[] { TPeriodic.class, String.class });
			plot2 = (Plot) mpreviewSpectrum.invoke(null, this, "");

			sub = new SubPlot(2, 1);
			sub.setPlot(0, 0, plot1);
			sub.setPlot(1, 0, plot2);
			plot = sub;
		} else {
			plot = plot1;
		}

		f.setContentPane(plot.getPanel());
		f.pack();
		f.setVisible(true);

		return plot;
	}

	/**
	 * Show spectrum preview
	 * 
	 * @param title
	 *            Title of preview
	 * @return Plot of preview
	 * @throws Exception
	 *             Unable to show plot
	 */
	public Plot showSpectrum(String title) throws Exception {
		return this.showSpectrum(title, this.getMaxOrder(), false);
	}

	/**
	 * Show spectrum preview
	 * 
	 * @param title
	 *            Title of preview
	 * @param maxorder
	 *            highest order
	 * @param rms
	 *            true: rms, false: magnitude
	 * @return Plot of preview
	 * @throws Exception
	 *             Unable to show plot
	 */
	public Plot showSpectrum(String title, int maxorder, boolean rms) throws Exception {

		ClassLoader classLoader = TPeriodic.class.getClassLoader();

		JFrame f = new JFrame("spectrum preview");
		f.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		@SuppressWarnings("unchecked")
		Class<TPeriodicPreview> c = (Class<TPeriodicPreview>) classLoader
				.loadClass("at.lcm.bu21.general.dtypes.preview.TPeriodicPreview");
		Method mpreviewSpectrum = c.getMethod("previewSpectrum",
				new Class[] { TPeriodic.class, String.class, int.class, boolean.class });

		Plot plot = (Plot) mpreviewSpectrum.invoke(null, this, title, maxorder, rms);
		f.setContentPane(plot.getPanel());

		f.pack();
		f.setVisible(true);

		return plot;
	}

	/**
	 * Show preview of 2d parametric curve x(phi), y(phi) with x(phi) being the
	 * object itself
	 * 
	 * @param y
	 *            y-signal
	 * @param title
	 *            Title of preview
	 * @return Plot of preview
	 * @throws Exception
	 *             Unable to show plot
	 */
	public Plot show2dCurvePreview(TPeriodic y, String title) throws Exception {

		ClassLoader classLoader = TPeriodic.class.getClassLoader();

		JFrame f = new JFrame("2d curve preview");
		f.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		@SuppressWarnings("unchecked")
		Class<TPeriodicPreview> c = (Class<TPeriodicPreview>) classLoader
				.loadClass("at.lcm.bu21.general.dtypes.preview.TPeriodicPreview");
		Method mpreview2dPanel = c.getMethod("preview2d",
				new Class[] { TPeriodic.class, TPeriodic.class, String.class });

		Plot plot = (Plot) mpreview2dPanel.invoke(null, this, y, title);
		f.setContentPane(plot.getPanel());

		f.pack();
		f.setVisible(true);

		return plot;
	}

	/**
	 * Show preview of 2d parametric curve x(phi), y(phi) with x(phi) being the
	 * object itself
	 * 
	 * @param y
	 *            y-signal
	 * @return Plot of preview
	 * @throws Exception
	 *             Unable to show plot
	 */
	public Plot show2dCurvePreview(TPeriodic y) throws Exception {
		return this.show2dCurvePreview(y, "");
	}

	@Override
	public String toString() {
		String m, pp;

		m = Double.valueOf(String.format(Locale.ENGLISH, "%.5e", this.mean())).toString().replace('E', 'e');
		pp = Double.valueOf(String.format(Locale.ENGLISH, "%.5e", this.pp())).toString().replace('E', 'e');

		return "[periodic: mean=" + m + ", pp=" + pp + "]";
	}

	@Override
	public TPeriodic deepClone() throws CloneException {
		return (TPeriodic) Clone.deepClone(this);
	}

	@Override
	public void xml2data(XMLStreamReader xmlReader) throws XMLStreamException, TDataException {
		String element, str;
		String[] values, values2;
		int order, type;
		double val;

		xmlReader.next();
		while (xmlReader.getEventType() == XMLStreamConstants.START_ELEMENT) {
			element = xmlReader.getLocalName();
			if (element.equals("DCOffset")) {
				xmlReader.next();
				this.DCOffset = Double.parseDouble(xmlReader.getText());
				xmlReader.next();
			} else if (element.equals("Coeffs")) {

				xmlReader.next();
				str = new String();

				while (xmlReader.getEventType() != XMLStreamConstants.END_ELEMENT) {
					str += xmlReader.getText();
					xmlReader.next();
				}

				if (!str.startsWith("[") || !str.endsWith("]")) {
					throw new TDataException("Invalid Periodic data");
				}

				if (!str.equals("[]")) {
					str = str.substring(1, str.length() - 1);
					values = str.split(";");

					for (int i = 0; i < values.length; i++) {
						values2 = values[i].split(",");
						order = Integer.parseInt(values2[0]);
						type = Integer.parseInt(values2[1]);
						val = Double.parseDouble(values2[2]);
						this.coeffs.add(new Coeff(order, type, val));
					}
				}

			} else {
				throw new TDataException("Undifined element: " + element);
			}
			xmlReader.next();
		}
		xmlReader.next();

	}

	@Override
	public void data2xml(XMLStreamWriter xmlWriter) throws XMLStreamException {

		this.compress();

		xmlWriter.writeStartElement("Periodic");

		xmlWriter.writeStartElement("DCOffset");
		xmlWriter.writeCharacters(Double.toString(this.DCOffset));
		xmlWriter.writeEndElement();

		xmlWriter.writeStartElement("Coeffs");

		if (this.coeffs.size() > 0) {
			StringBuffer str = new StringBuffer();

			str.append("[");

			for (Coeff coeff : this.coeffs) {
				str.append(Integer.toString(coeff.order));
				str.append(",");
				str.append(Integer.toString(coeff.type));
				str.append(",");
				str.append(Double.toString(coeff.value));
				str.append(";");
			}

			str.setCharAt(str.length() - 1, ']');
			xmlWriter.writeCharacters(str.toString());
		} else {
			xmlWriter.writeCharacters("[]");
		}

		xmlWriter.writeEndElement();

		xmlWriter.writeEndElement();

	}

	@Override
	public boolean equals(Object obj) {
		return this.equals(obj, 0.0);
	}

	/**
	 * Equality test
	 * 
	 * @param obj
	 *            Object to check
	 * @param tol
	 *            Tolerance
	 * 
	 * @return true if equal
	 */
	public boolean equals(Object obj, double tol) {

		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (this.getClass() != obj.getClass()) {
			return false;
		}
		TPeriodic other = (TPeriodic) obj;

		this.compress();
		other.compress();
		if ((this.rms() != 0.0) || (other.rms() != 0.0)) {
			try {
				double rms = Math.max(this.rms(), other.rms());
				double sub = this.subtract(other).rms();

				if ((sub > (rms * TPeriodic.AMPLITUDE_TRESHOLD)) && (sub > tol)) {
					return false;
				}
			} catch (Exception e) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Interpolates data vector so that time steps are equidistant (step size
	 * equals mimimum time gap in time)
	 * 
	 * @param time
	 *            original time vector
	 * @param data
	 *            original data vector
	 * @return interpolated data data[0]: interpolated time vector data[1]:
	 *         interpolated data vector
	 */
	private static double[][] interp(double[] time, double[] data) {
		// get mimimum sampling time
		double tsMin = time[1] - time[0];
		double tmp, d, k, deltaT, deltaX;
		int idx = 0;
		for (int q = 1; q < (time.length - 1); q++) {
			if ((tmp = time[q + 1] - time[q]) < tsMin) {
				tsMin = tmp;
			}
		}
		// get new time vector with mimimum sampling time as stepsize
		double[] timeI = new double[(int) Math.ceil((time[time.length - 1] - time[0]) / tsMin)];
		double[] dataI = new double[timeI.length];
		timeI[0] = time[0];
		dataI[0] = data[0];
		for (int q = 1; q < timeI.length; q++) {
			timeI[q] = timeI[q - 1] + tsMin;
			// interpolate data vector
			while ((idx < time.length) && (timeI[q] > time[idx])) {
				idx++;
			}
			// Sttzpunkte time[idx-1] time[idx] bzw. data[idx-1] data[idx]
			deltaT = time[idx] - time[idx - 1];
			deltaX = data[idx] - data[idx - 1];
			k = deltaX / deltaT;
			d = ((data[idx - 1] * time[idx]) - (data[idx] * time[idx - 1])) / deltaT;
			dataI[q] = (k * timeI[q]) + d;
		}

		return new double[][] { timeI, dataI };
	}
}
