/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.awt.image.ImageObserver;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.File;
import java.text.MessageFormat;
import java.util.Vector;

import javax.swing.AbstractListModel;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTable.PrintMode;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.ListModel;
import javax.swing.RepaintManager;
import javax.swing.WindowConstants;
import javax.swing.event.TableModelListener;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import at.lcm.bu21.general.dtypes.TBoolean;
import at.lcm.bu21.general.dtypes.TData;
import at.lcm.bu21.general.dtypes.TDataException;
import at.lcm.bu21.general.dtypes.TNumeric;
import at.lcm.bu21.general.dtypes.TPeriodic;
import at.lcm.bu21.general.dtypes.TString;
import at.lcm.bu21.general.dtypes.editor.icons.IconEditor;
import at.lcm.bu21.general.dtypes.preview.TPeriodicPreview;
import at.lcm.bu21.general.plot.Plot2Line;
import at.lcm.bu21.general.plot.Plot2XY;
import at.lcm.bu21.general.plot.PlotException;

/**
 * @author Christoph Halbmayr
 */
class EditorChangePanel implements ActionListener, Printable, EditorPanelInterface {

	/** Button for col inserting (TNumeric) */
	private JButton addCol;

	/** Button for row inserting (TNumeric) */
	private JButton addRow;

	/** Editor Action Listener */
	private ActionListener al;

	/** Contains Toolbar buttons */
	private Vector<JButton> Buttons;

	/** col field for TNum */
	private TextField cols;

	/** Button for Tperiodic Generator */
	private JButton createTPeriodic;

	/** JTextField for DC-Offset */
	private JTextField dcOffestTB;

	/** Button for deleting an Order in TPeriodic */
	private JButton del;

	/** Button for col deleting (TNumeric) */
	private JButton delCol;

	/** Button for row deleting (TNumeric) */
	private JButton delRow;

	/** MenuBarPanel */
	private JPanel menuBarPanel;

	/** Contains Table Titels and Table Data */
	private TableModelChangePanel model;

	/** toolbar for TNum */
	private JToolBar newvar;

	/** JPanel */
	private JPanel panel;

	/** preview plot (for TPeriodic) */
	private Plot2XY plot;

	/** preview Window (for TPeriodic) */
	private JFrame preview;

	/** Preview Button (TPeriodic) */
	private JButton prv;

	/** resize Button for TNum */
	private JButton resize;

	/** Table row */
	private Vector<String> row;

	/** row field for TNum */
	private TextField rows;

	/** JScrollPane */
	private JScrollPane scroll;

	/** The selectet TData */
	private TData selection;

	/** spectrum Button (TPeriodic) */
	private JButton spec;

	/** spectrum Window (for TPeriodic) */
	private JFrame spectrum;

	/** Signal Editor Table */
	private JTable table;

	/** ToolBar for Navigation und editing Signals */
	private JToolBar tb;

	/** TableModelListener */
	private TableModelListener tml;

	/**
	 * Creats an empty Object
	 */
	public EditorChangePanel() {
		this.initialize();

	}

	/**
	 * Creats an Object
	 * 
	 * @param selection
	 *            selectet TData
	 */
	public EditorChangePanel(final TData selection) {

		this.initialize();
		this.selection = selection;
		this.fillTable();

		this.columsResize(this.table, 90);
	}

	/**
	 * Not in use
	 */
	@Override
	public void actionPerformed(final ActionEvent arg0) {

		if (arg0.getActionCommand().equals("x/y plot")) {
			this.preview();
		}

		if (arg0.getActionCommand().equals("spectrum")) {
			this.spectrum();
		}

		if (arg0.getActionCommand().equals("up")) {

			if (this.preview != null) {
				this.preview.dispose();
			}

			if (this.spectrum != null) {
				this.spectrum.dispose();
			}
		}
		if (arg0.getActionCommand().equals("orderDelete")) {

			final int[] selRows = this.table.getSelectedRows();

			for (int i = selRows.length - 1; i >= 0; i--)

			{

				final String orderCell = this.model.getValueAt(selRows[i], 0).toString();
				final int order = Integer.parseInt(orderCell);
				this.orderDelete(order);
				this.reload();
				this.createHeaders();
			}

			this.reload();
		}
	}

	/**
	 * Adds a Action Listener
	 * 
	 * @param a
	 *            ActionListener
	 */
	public void addListener(final ActionListener a) {
		for (final JButton i : this.Buttons) {
			i.addActionListener(a);
		}
		if (this.dcOffestTB != null) {
			this.dcOffestTB.addActionListener(a);
		}
		if (this.resize != null) {
			this.resize.addActionListener(a);
		}
		this.al = a;
	}

	/**
	 * Adds a Table Model Listener to the
	 * 
	 * @param tml
	 *            TableModelListener
	 */
	public void addTableModelListener(final TableModelListener tml) {
		this.tml = tml;
		this.model.addTableModelListener(tml);
	}

	/**
	 * Adds a Mouse Listener
	 * 
	 * @param ml
	 *            MouseListener
	 */
	public void adMListener(final MouseListener ml) {
		this.table.addMouseListener(ml);
	}

	/**
	 * This method is triggered by the TableModelListener
	 * 
	 * @param row
	 *            row
	 * @param column
	 *            column
	 */

	public void changed(final int row, final int column) {
		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TPeriodic")) {
			if ((row != (this.table.getRowCount() - 1)) && (column > 0)) {
				this.valueChanged(row, column);
				this.reload();

			} else if ((row != (this.table.getRowCount() - 1)) && (column == 0)) {
				this.orderChanged();
			} else if (row == (this.table.getRowCount() - 1)) {
				this.addNewOrder(row, column);
			}
		} else {

			this.valueChanged(row, column);
			this.reload();
			this.columsResize(this.table, 90);
		}
		this.columsResize(this.table, 90);
		this.createHeaders();
	}

	/**
	 * changes the DC-Offset to the input Value
	 */
	public void changeOffset() {
		final double mean;
		try {
			mean = Double.parseDouble(this.dcOffestTB.getText());
		} catch (final Exception e) {
			return;
		}

		final TPeriodic temp = (TPeriodic) this.selection;
		((TPeriodic) this.selection).setCoeffs(mean, temp.getCoeffA(temp.usedOrders()),
				temp.getCoeffB(temp.usedOrders()));
		((TPeriodic) this.selection).compress();

		this.reload();

	}

	@Override
	public void delete() {

	}

	/**
	 * deletes the selected columns
	 */
	@Override
	public void deleteColumn() {
		final int[] col = this.table.getSelectedColumns();

		for (int i = col.length - 1; i >= 0; i--) {
			this.deleteColumn(i);
		}

		this.reload();
	}

	/**
	 * deletes the selectet Rows
	 */
	@Override
	public void deleteRow() {

		final int[] rows = this.table.getSelectedRows();

		for (int i = rows.length - 1; i >= 0; i--) {
			this.deleteRow(rows[i]);
		}

		this.reload();

	}

	@Override
	public JPanel getPanel() {
		return this.panel;
	}

	/**
	 * Returns the TData
	 * 
	 * @return selectet TData
	 */
	@Override
	public TData getSelectedTData() {
		return this.selection;
	}

	@Override
	public TData getTData() {

		return this.selection;
	}

	@Override
	public void importTData(final File f) {

	}

	/**
	 * inserts a row
	 */
	@Override
	public void insertColumn() {

		final int column = this.table.getSelectedColumn();

		this.insertColumn(column);

		this.reload();

	}

	/**
	 * inserts a row
	 */
	@Override
	public void insertRow() {

		final int row = this.table.getSelectedRow();

		this.insertRow(row);

		this.reload();

	}

	/**
	 * opens a preview window (TNumeric & TPeriodic)
	 */
	public void preview() {

		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TPeriodic")) {

			this.preview = new JFrame(this.selection.toString());
			this.preview.setIconImage(IconEditor.ICON_XYPLOT.getImage());
			this.plot = new Plot2Line();
			this.plot.setGrid(true);
			try {
				this.plot.addSignal(this.selection.toString(), (TPeriodic) this.selection, 500);
			} catch (PlotException e) {
			}

			this.preview.add(this.plot.getPanel());
			this.preview.setSize(800, 600);
			this.preview.setVisible(true);
			this.preview.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		}

	}

	@Override
	public int print(final Graphics g, final PageFormat pageFormat, final int pageIndex) throws PrinterException {

		if (pageIndex > 0) {
			return Printable.NO_SUCH_PAGE;
		} else {
			final int x = (int) pageFormat.getImageableX() + 1;
			final int y = (int) pageFormat.getImageableY() + 1;
			g.translate(x, y);
			final RepaintManager currentManager = RepaintManager.currentManager(this.panel);

			currentManager.setDoubleBufferingEnabled(true);
			this.table.paintComponents(g);
			currentManager.setDoubleBufferingEnabled(true);
			return Printable.PAGE_EXISTS;
		}

	}

	/**
	 * Prints out the whole Editor View
	 */
	@Override
	public void printme() {
		final String TDataType = this.selection.getClass().getName().split("\\.")[5];
		final PrinterJob printJob = PrinterJob.getPrinterJob();
		final MessageFormat headerFormat = new MessageFormat(TDataType + " Page {0}");
		final MessageFormat footerFormat = new MessageFormat("- {0} -");
		printJob.setPrintable(this.table.getPrintable(PrintMode.NORMAL, headerFormat, footerFormat));
		if (printJob.printDialog()) {
			try {
				printJob.print();
			} catch (final PrinterException pe) {
			}
		}
	}

	@Override
	public void reload() {

		this.model.getDataVector().removeAllElements();
		this.fillModel();
		this.table.repaint();
		this.updatePlots();

	}

	@Override
	public void saveTData(final File f) {

	}

	/**
	 * Open and show the TPeriodic Genatator
	 */
	public void showGenerator() {

		final JFrame jd = new JFrame("TPeriodic Genatator");
		jd.setIconImage(IconEditor.ICON_GENERATOR.getImage());
		jd.setSize(440, 535);
		jd.setVisible(true);

		final TPeriodicGenerator tpg = new TPeriodicGenerator();

		final JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new GridLayout(1, 4));

		final JButton apply = new JButton("apply");
		final JButton cancel = new JButton("cancel");

		final ActionListener alis = new ActionListener() {

			@Override
			public void actionPerformed(final ActionEvent arg0) {

				if (arg0.getActionCommand().equals("cancel")) {
					jd.dispose();

					if (tpg.overview != null) {
						tpg.overview.dispose();
					}
					if (tpg.spectrum != null) {
						tpg.spectrum.dispose();
					}
					if (tpg.preview != null) {
						tpg.preview.dispose();
					}
				}

				if (arg0.getActionCommand().equals("apply")) {
					EditorChangePanel.this.selection = tpg.getData();

					if (EditorChangePanel.this.selection != null) {
						EditorChangePanel.this.al.actionPerformed(new ActionEvent(this, 0, "writeback&reload"));
						jd.setVisible(false);
						EditorChangePanel.this.dcOffestTB
								.setText("" + ((TPeriodic) EditorChangePanel.this.selection).mean());
					}

				}
			}
		};

		cancel.addActionListener(alis);
		apply.addActionListener(alis);
		buttonPanel.add(new JPanel());
		buttonPanel.add(new JPanel());
		buttonPanel.add(apply);
		buttonPanel.add(cancel);

		jd.add(tpg.getPanel());
		jd.add(buttonPanel, BorderLayout.SOUTH);
		jd.repaint();
		tpg.getPanel().repaint();
		tpg.getPanel().revalidate();

	}

	/**
	 * opens a spectrum window (TNumeric & TPeriodic)
	 */
	public void spectrum() {

		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TPeriodic")) {

			this.spectrum = new JFrame(this.selection.toString());
			this.spectrum.setIconImage(IconEditor.ICON_SPECTRUM.getImage());
			this.spectrum.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			this.spectrum.add(TPeriodicPreview.previewSpectrum((TPeriodic) this.selection, null).getPanel());
			this.spectrum.setSize(800, 600);
			this.spectrum.setVisible(true);
		}

	}

	@Override
	public void updateChangedSignal(final TData selection) {

	}

	/**
	 * Adds a new Order to TPeriodic
	 * 
	 * @param row
	 * @param column
	 */
	private void addNewOrder(final int row, final int column) {
		String orderCell;
		String coeffACell;
		String coeffBCell;
		String magnitudeCell;
		String phaseCell;
		try {
			orderCell = this.model.getValueAt(row, 0).toString();
			coeffACell = this.model.getValueAt(row, 1).toString();
			coeffBCell = this.model.getValueAt(row, 2).toString();
			magnitudeCell = this.model.getValueAt(row, 3).toString();
			phaseCell = this.model.getValueAt(row, 4).toString();
		} catch (final Exception e) {
			this.reload();
			return;
		}

		if ((orderCell.equals("[ORDER]") == false) && (coeffACell.equals("[A]") == false)
				&& (coeffBCell.equals("[B]") == false)) {
			final int order = Integer.parseInt(orderCell);
			final double[] coeffsA = new double[order];
			final double[] coeffsB = new double[order];

			final double coeffA = Double.parseDouble(coeffACell);
			final double coeffB = Double.parseDouble(coeffBCell);

			coeffsA[order - 1] = coeffA;
			coeffsB[order - 1] = coeffB;

			TPeriodic temp = new TPeriodic();
			temp.setCoeffs(temp.mean(), coeffsA, coeffsB);

			temp = ((TPeriodic) this.selection).add(temp);

			((TPeriodic) this.selection).setCoeffs(temp.mean(), temp.getCoeffA(temp.usedOrders()),
					temp.getCoeffB(temp.usedOrders()));
			((TPeriodic) this.selection).compress();
			this.reload();

		} else if ((orderCell.equals("[ORDER]") == false) && (magnitudeCell.equals("[MAGNITUDE]") == false)
				&& (phaseCell.equals("[PHASE]") == false)) {
			final int order = Integer.parseInt(orderCell);
			final double magnidute = Double.parseDouble(magnitudeCell);
			double phase = Double.parseDouble(phaseCell);
			phase *= Math.PI / 180;
			TPeriodic temp = new TPeriodic();
			temp = ((TPeriodic) this.selection).add(new TPeriodic(order, magnidute, phase));

			((TPeriodic) this.selection).setCoeffs(temp.mean(), temp.getCoeffA(temp.usedOrders()),
					temp.getCoeffB(temp.usedOrders()));
			((TPeriodic) this.selection).compress();

			this.reload();
		}
	}

	/**
	 * adds a new row for adding a new order
	 */
	private void addNewOrderRow() {
		this.model.removeTableModelListener(this.tml);
		this.row = new Vector<String>();

		this.row.add("[ORDER]");

		this.row.add("[A]");

		this.row.add("[B]");

		this.row.add("[MAGNITUDE]");

		this.row.add("0.0");

		this.model.addRow(this.row);
		this.model.addTableModelListener(this.tml);

	}

	/**
	 * Creats an ToolBar for DC-Offset (TPeriodic only)
	 */
	private void createDCOffsetBar() {
		if ((this.selection != null)
				&& this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TPeriodic")) {
			final String text = "" + ((TPeriodic) this.selection).mean();
			this.dcOffestTB = new JTextField(text);
			this.dcOffestTB.addFocusListener(new java.awt.event.FocusAdapter() {
				@Override
				public void focusGained(final java.awt.event.FocusEvent evt) {
					EditorChangePanel.this.dcOffestTB.selectAll();
				}
			});

			final JLabel lable = new JLabel();
			lable.setText("DC-Offset: ");
			final JToolBar dcBar = new JToolBar();
			dcBar.setPreferredSize(new Dimension(ImageObserver.WIDTH, 20));

			dcBar.add(lable);
			dcBar.add(this.dcOffestTB);

			this.menuBarPanel.add(dcBar, BorderLayout.SOUTH);
			this.panel.repaint();
		}

	}

	/**
	 * creats ToolBar for TNumeric
	 */
	private void createTNumToolBar() {
		this.newvar = new JToolBar();

		this.resize = new JButton("RESIZE");

		this.rows = new TextField("rows");
		this.cols = new TextField("columns");

		this.rows.addFocusListener(new java.awt.event.FocusAdapter() {
			@Override
			public void focusGained(final java.awt.event.FocusEvent evt) {
				EditorChangePanel.this.rows.selectAll();
			}
		});

		this.cols.addFocusListener(new java.awt.event.FocusAdapter() {
			@Override
			public void focusGained(final java.awt.event.FocusEvent evt) {
				EditorChangePanel.this.cols.selectAll();
			}
		});

		this.rows.setEditable(true);
		this.cols.setEditable(true);

		this.newvar.setPreferredSize(new Dimension(ImageObserver.WIDTH, 20));
		this.newvar.add(this.rows);
		this.newvar.add(this.cols);
		this.newvar.add(this.resize);

		this.panel.add(this.newvar, BorderLayout.SOUTH);

	}

	/**
	 * Creats a MenuBar
	 */
	private void createToolbars() {
		this.tb = new JToolBar();
		this.tb.setPreferredSize(new Dimension(ImageObserver.WIDTH, 25));
		this.Buttons = new Vector<JButton>();

		final JButton up = new JButton(IconEditor.ICON_UP);
		up.setActionCommand("up");
		up.setToolTipText("move one level up");
		this.Buttons.add(up);

		final JButton print = new JButton(IconEditor.ICON_PRINTER);
		print.setActionCommand("Print");
		print.setToolTipText("print");
		this.Buttons.add(print);

		for (final JButton i : this.Buttons) {
			i.addActionListener(this);
			this.tb.add(i);
		}

		this.prv = new JButton(IconEditor.ICON_XYPLOT);
		this.prv.setActionCommand("x/y plot");
		this.prv.setToolTipText("show x/y plot");
		this.prv.addActionListener(this);
		this.prv.setVisible(false);
		this.tb.add(this.prv);

		this.spec = new JButton(IconEditor.ICON_SPECTRUM);
		this.spec.setActionCommand("spectrum");
		this.spec.setToolTipText("show spectrum plot");
		this.spec.addActionListener(this);
		this.spec.setVisible(false);
		this.tb.add(this.spec);

		this.del = new JButton(IconEditor.ICON_REMOVE);
		this.del.setActionCommand("orderDelete");
		this.del.setToolTipText("delete selected signal");
		this.del.addActionListener(this);
		this.del.setVisible(false);
		this.tb.add(this.del);
		this.Buttons.add(this.del);

		this.createTPeriodic = new JButton(IconEditor.ICON_GENERATOR);
		this.createTPeriodic.setActionCommand("show Generator");
		this.createTPeriodic.setToolTipText("show TPeriodic generator");
		this.createTPeriodic.addActionListener(this);
		this.createTPeriodic.setVisible(false);
		this.tb.add(this.createTPeriodic);
		this.Buttons.add(this.createTPeriodic);

		this.addRow = new JButton("Row", IconEditor.ICON_ADD);
		this.addRow.setToolTipText("insert a row ");
		this.addRow.setActionCommand("addRow");
		this.addRow.addActionListener(this);
		this.addRow.setVisible(false);
		this.tb.add(this.addRow);
		this.Buttons.add(this.addRow);

		this.delRow = new JButton("Row", IconEditor.ICON_REMOVE);
		this.delRow.setToolTipText("delete selected row");
		this.delRow.addActionListener(this);
		this.delRow.setVisible(false);
		this.tb.add(this.delRow);
		this.Buttons.add(this.delRow);

		this.addCol = new JButton("Column", IconEditor.ICON_ADD);
		this.addCol.setToolTipText("insert a column");
		this.addCol.setActionCommand("addCol");
		this.addCol.addActionListener(this);
		this.addCol.setVisible(false);
		this.tb.add(this.addCol);
		this.Buttons.add(this.addCol);

		this.delCol = new JButton("Column", IconEditor.ICON_REMOVE);
		this.delCol.setToolTipText("delete selected column");
		this.delCol.addActionListener(this);
		this.delCol.setVisible(false);
		this.tb.add(this.delCol);
		this.Buttons.add(this.delCol);

		this.menuBarPanel.add(this.tb, BorderLayout.NORTH);

	}

	/**
	 * deletes the selected column
	 * 
	 * @param col
	 *            selected column
	 */
	private void deleteColumn(final int col) {

		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TNumeric") && (col != -1)) {

			final TNumeric temp = (TNumeric) this.selection;

			final double[][] real = temp.getRealMatrix().getData();
			final double[][] imag = temp.getImagMatrix().getData();

			final int rowSize = real.length;
			final int colSize = real[0].length;

			final double[][] newReal = new double[rowSize][colSize - 1];
			final double[][] newImag = new double[rowSize][colSize - 1];

			final int newRowSize = newReal.length;
			final int newColSize = newReal[0].length;

			for (int m = 0; m < newRowSize; m++) {
				int index = 0;
				for (int n = 0; n < newColSize; n++) {
					if (n == col) {
						index = 1;
						if ((n + index) <= newColSize) {

							newReal[m][n] = real[m][n + index];
							newImag[m][n] = imag[m][n + index];

						}
					} else if ((n + index) <= newColSize) {
						newReal[m][n] = real[m][n + index];
						newImag[m][n] = imag[m][n + index];
					}
				}
			}

			try {
				this.selection = new TNumeric(newReal, newImag);
			} catch (final TDataException e) {

				e.printStackTrace();
			}
		}

	}

	/**
	 * deletes the selectet Row
	 * 
	 * @param row
	 *            selected row
	 */
	private void deleteRow(final int row) {
		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TNumeric") && (row != -1)) {

			final TNumeric temp = (TNumeric) this.selection;

			final double[][] real = temp.getRealMatrix().getData();
			final double[][] imag = temp.getImagMatrix().getData();

			final int rowSize = real.length;
			final int colSize = real[1].length;

			final double[][] newReal = new double[rowSize - 1][colSize];
			final double[][] newImag = new double[rowSize - 1][colSize];

			final int newRowSize = newReal.length;
			final int newColSize = newReal[1].length;

			int index = 0;
			for (int m = 0; m < newRowSize; m++) {
				for (int n = 0; n < newColSize; n++) {
					if (m == row) {
						index = 1;
						if ((m + index) <= newRowSize) {

							newReal[m][n] = real[m + index][n];
							newImag[m][n] = imag[m + index][n];
						}
					} else if ((m + index) <= newRowSize) {
						newReal[m][n] = real[m + index][n];
						newImag[m][n] = imag[m + index][n];
					}
				}
			}

			try {
				this.selection = new TNumeric(newReal, newImag);

			} catch (final TDataException e) {

				e.printStackTrace();
			}
		}
	}

	/**
	 * fills the Model with TData
	 */
	private void fillModel() {
		this.model.removeTableModelListener(this.tml);
		this.model.removeTableModelListener(this.tml);
		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TString")) {
			this.model.addTData(this.selection);
		}

		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TBoolean")) {

			final String[] b = new String[2];
			b[0] = "true";
			b[1] = "false";

			final JComboBox cb = new JComboBox(b);
			cb.setEditable(false);

			this.model.addTData(this.selection);
			this.table.getColumnModel().getColumn(0).setCellEditor(new DefaultCellEditor(cb));

		}

		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TNumeric")) {

			this.delCol.setVisible(true);
			this.delRow.setVisible(true);
			this.addCol.setVisible(true);
			this.addRow.setVisible(true);
			this.table.setColumnSelectionAllowed(true);
			this.table.setRowSelectionAllowed(true);
			this.model.addTData(this.selection);

			if (this.newvar == null) {
				this.createTNumToolBar();
			}
		}

		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TPeriodic")) {

			this.del.setVisible(true);
			this.prv.setVisible(true);
			this.spec.setVisible(true);
			this.createTPeriodic.setVisible(true);

			this.model.addTData(this.selection);

			this.addNewOrderRow();

			if (this.dcOffestTB == null) {
				this.createDCOffsetBar();
			}

		}
		this.model.addTableModelListener(this.tml);
	}

	/**
	 * Fills the Table
	 */
	private void fillTable() {

		this.fillModel();
		this.createHeaders();
		this.columsResize(this.table, 90);

	}

	/**
	 * initializes the plot
	 */
	private void initialize() {
		this.menuBarPanel = new JPanel();
		this.menuBarPanel.setLayout(new BorderLayout());
		this.model = new TableModelChangePanel();
		this.table = new JTable(this.model);
		this.table.setAutoResizeMode(0);
		this.table.setDefaultEditor(Object.class, new DefaultCellEditor(new JTextField()) {

			/** Serialization version number */
			private static final long serialVersionUID = 1L;

			@Override
			public Component getTableCellEditorComponent(final JTable table, final Object value,
					final boolean isSelected, final int row, final int column) {
				Component c;
				c = super.getTableCellEditorComponent(table, value, isSelected, row, column);
				((JTextField) c).selectAll();
				return c;
			}
		});
		this.table.getTableHeader().setReorderingAllowed(false);
		this.table.setRowSelectionAllowed(true);
		this.table.setColumnSelectionAllowed(true);
		this.row = new Vector<String>();
		this.panel = new JPanel();
		this.panel.setLayout(new BorderLayout());
		this.panel.add(this.menuBarPanel, BorderLayout.NORTH);
		this.createToolbars();
		this.createDCOffsetBar();

	}

	/**
	 * inserts a column
	 * 
	 * @param col
	 *            the column to delete
	 */
	private void insertColumn(final int col) {
		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TNumeric") && (col != -1)) {

			final TNumeric temp = (TNumeric) this.selection;

			final double[][] real = temp.getRealMatrix().getData();
			final double[][] imag = temp.getImagMatrix().getData();

			final int rowSize = real.length;
			final int colSize = real[1].length;

			final double[][] newReal = new double[rowSize][colSize + 1];
			final double[][] newImag = new double[rowSize][colSize + 1];

			final int newRowSize = newReal.length;
			final int newColSize = newReal[1].length;

			for (int m = 0; m < newRowSize; m++) {
				for (int n = 0; n < newColSize; n++) {
					if (n == col) {

						newReal[m][n] = 0.0;
						newImag[m][n] = 0.0;

					} else if (n < col) {
						newReal[m][n] = real[m][n];
						newImag[m][n] = imag[m][n];
					} else if (n > col) {
						newReal[m][n] = real[m][n - 1];
						newImag[m][n] = imag[m][n - 1];
					}
				}
			}

			try {
				this.selection = new TNumeric(newReal, newImag);

			} catch (final TDataException e) {
			}
		}
	}

	/**
	 * inserts a Row
	 * 
	 * @param row
	 *            the row to delete
	 */
	private void insertRow(final int row) {
		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TNumeric") && (row != -1)) {

			final TNumeric temp = (TNumeric) this.selection;

			final double[][] real = temp.getRealMatrix().getData();
			final double[][] imag = temp.getImagMatrix().getData();

			final int rowSize = real.length;
			final int colSize = real[1].length;

			final double[][] newReal = new double[rowSize + 1][colSize];
			final double[][] newImag = new double[rowSize + 1][colSize];

			final int newRowSize = newReal.length;
			final int newColSize = newReal[1].length;

			for (int m = 0; m < newRowSize; m++) {
				for (int n = 0; n < newColSize; n++) {
					if (m == row) {

						newReal[m][n] = 0.0;
						newImag[m][n] = 0.0;

					} else if (m < row) {
						newReal[m][n] = real[m][n];
						newImag[m][n] = imag[m][n];
					} else if (m > row) {
						newReal[m][n] = real[m - 1][n];
						newImag[m][n] = imag[m - 1][n];
					}
				}
			}

			try {

				this.selection = new TNumeric(newReal, newImag);

			} catch (final TDataException e) {
			}
		}
	}

	/**
	 * handle an order change in table
	 */
	private void orderChanged() {

		TPeriodic temp = new TPeriodic();

		String orderCell = null;
		String magnitudeCell = null;
		String phaseCell = null;

		for (int row = 0; row <= (this.table.getRowCount() - 2); row++) {

			orderCell = this.model.getValueAt(row, 0).toString();
			magnitudeCell = this.model.getValueAt(row, 3).toString();
			phaseCell = this.model.getValueAt(row, 4).toString();

			int order = 0;
			double magnidute = 0;
			double phase = 0;
			try {
				order = Integer.parseInt(orderCell);
				magnidute = Double.parseDouble(magnitudeCell);
				phase = Double.parseDouble(phaseCell);
			} catch (final NumberFormatException e) {
				this.reload();
				return;
			}

			temp = temp.add(new TPeriodic(order, magnidute, phase));
		}

		final double mean = ((TPeriodic) this.selection).mean();
		((TPeriodic) this.selection).setCoeffs(mean, temp.getCoeffA(temp.usedOrders()),
				temp.getCoeffB(temp.usedOrders()));
		((TPeriodic) this.selection).compress();

		this.reload();
	}

	/**
	 * deletes an Order
	 * 
	 * @param order
	 *            to delete
	 */
	private void orderDelete(final int order) {
		final TPeriodic temp = (TPeriodic) this.selection;

		final double[] coeffsA = new double[temp.getMaxOrder()];
		final double[] coeffsB = new double[temp.getMaxOrder()];

		for (int i = 0; i < temp.getMaxOrder(); i++) {
			if ((i + 1) != order) {
				coeffsA[i] = temp.getCoeffA(i + 1);
				coeffsB[i] = temp.getCoeffB(i + 1);
			}
		}

		((TPeriodic) this.selection).setCoeffs(((TPeriodic) this.selection).mean(), coeffsA, coeffsB);

		((TPeriodic) this.selection).compress();

	}

	/**
	 * Updates preview and spectrum plot
	 */
	private void updatePlots() {
		if ((this.preview != null) && this.preview.isVisible()) {
			final Point loc = this.preview.getLocation();
			this.preview.setTitle(this.selection.toString().toString());
			this.preview.setSize(800, 600);

			this.plot = new Plot2Line();
			this.plot.setGrid(true);
			try {
				this.plot.addSignal(this.selection.toString(), (TPeriodic) this.selection, 500);
			} catch (PlotException e) {
			}

			this.preview.add(this.plot.getPanel());
			this.preview.setLocation(loc);
			this.preview.setVisible(true);

		}
		if ((this.spectrum != null) && this.spectrum.isVisible()) {
			final Point loc = this.spectrum.getLocation();
			this.spectrum.setTitle(this.selection.toString().toString());
			this.spectrum.add(TPeriodicPreview.previewSpectrum((TPeriodic) this.selection, null).getPanel());
			this.spectrum.setSize(800, 600);
			this.spectrum.setLocation(loc);
			this.spectrum.setVisible(true);
		}

	}

	/**
	 * Evaluets the change and alters TData
	 * 
	 * @param row
	 *            row
	 * @param column
	 *            column
	 */
	private void valueChanged(final int row, final int column) {
		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TString")) {

			final String text = this.table.getCellEditor(row, column).getCellEditorValue().toString();

			this.selection = new TString(text);
		}

		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TBoolean")) {

			final String text = this.table.getValueAt(row, column).toString();

			if (text.equals("true")) {
				this.selection = new TBoolean(true);
			} else {
				this.selection = new TBoolean(false);
			}

		}

		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TNumeric")) {

			final String tabletext = this.table.getValueAt(row, column).toString();

			final int rowNumber = this.table.getSelectedRow();
			final int colNumber = this.table.getSelectedColumn();

			try {
				((TNumeric) this.selection).setStringVal(rowNumber, colNumber, tabletext);
			} catch (final TDataException e) {
				// reload();
				// return;
			}

		}

		if (this.selection.getClass().getName().equals("at.lcm.bu21.general.dtypes.TPeriodic")) {

			final TPeriodic tp = (TPeriodic) this.selection;

			final int usedOrders[] = tp.usedOrders();

			final double[] coeffsA = tp.getCoeffA(usedOrders);
			final double[] coeffsB = tp.getCoeffB(usedOrders);

			int coeffNumber = 0;
			double coeffA = 0;
			double coeffB = 0;
			try {
				coeffNumber = Integer.parseInt(this.table.getValueAt(row, 0).toString()) - 1;

				coeffA = Double.parseDouble(this.table.getValueAt(row, 1).toString());
				coeffB = Double.parseDouble(this.table.getValueAt(row, 2).toString());
			} catch (final NumberFormatException e) {
				this.reload();
				return;
			}

			// order A|B
			if (column <= 2) {

				coeffsA[coeffNumber] = coeffA;
				coeffsB[coeffNumber] = coeffB;
				((TPeriodic) this.selection).setCoeffs(tp.mean(), coeffsA, coeffsB);
			}

			else { // magnitude|phase
				TPeriodic temp = new TPeriodic();
				int order = 0;
				double magnitude = 0.0;
				double phase = 0.0;

				final int rowCount = this.model.getRowCount() - 2;

				for (int n = 0; n <= rowCount; n++) {
					order = Integer.parseInt(this.table.getValueAt(n, 0).toString());
					magnitude = Double.parseDouble(this.table.getValueAt(n, 3).toString());
					phase = Double.parseDouble(this.table.getValueAt(n, 4).toString());
					phase *= Math.PI / 180;
					temp = temp.add(new TPeriodic(order, magnitude, phase));

				}

				((TPeriodic) this.selection).setCoeffs(tp.mean(), temp.getCoeffA(temp.usedOrders()),
						temp.getCoeffB(temp.usedOrders()));
				((TPeriodic) this.selection).compress();

			}

		}

	}

	/**
	 * Adjusts the table colum size to the colum content length
	 * 
	 * @param t
	 *            JTable
	 * @param minimumSize
	 *            minimum column size
	 */
	protected void columsResize(final JTable t, final int minimumSize) {
		final JTableHeader header = t.getTableHeader();
		TableCellRenderer defaultHeaderRenderer = null;

		if (header != null) {
			defaultHeaderRenderer = header.getDefaultRenderer();
		}

		final TableColumnModel columns = t.getColumnModel();
		final TableModel data = t.getModel();
		final int margin = columns.getColumnMargin();
		final int rowCount = data.getRowCount();

		for (int i = columns.getColumnCount() - 1; i >= 0; --i) {
			final javax.swing.table.TableColumn column = columns.getColumn(i);
			final int columnIndex = column.getModelIndex();
			int width = -1;

			TableCellRenderer h = column.getHeaderRenderer();

			if (h == null) {
				h = defaultHeaderRenderer;
			}

			if (h != null) {
				final Component c = h.getTableCellRendererComponent(t, column.getHeaderValue(), false, false, -1, i);
				width = c.getPreferredSize().width;

			}

			for (int row = rowCount - 1; row >= 0; --row) {
				final TableCellRenderer r = t.getCellRenderer(row, i);
				final Component c = r.getTableCellRendererComponent(t, data.getValueAt(row, columnIndex), false, false,
						row, i);
				width = Math.max(width, c.getPreferredSize().width);
			}

			if (width >= 0) {
				if ((width + margin) > minimumSize) {
					column.setPreferredWidth(width + margin);
				} else {
					column.setPreferredWidth(minimumSize);
				}
			}
		}
	}

	/**
	 * creats Table row headers
	 */
	void createHeaders() {
		final String[] headers = new String[this.table.getRowCount()];

		for (int i = 0; i < this.table.getRowCount(); i++) {
			headers[i] = "" + i;
		}

		final ListModel lm = new AbstractListModel() {

			/** Serialization version number */
			private static final long serialVersionUID = 1L;

			@Override
			public Object getElementAt(final int index) {
				return headers[index];
			}

			@Override
			public int getSize() {
				return headers.length;
			}
		};

		final JList rowHeader = new JList(lm);
		rowHeader.setFixedCellWidth(25);

		rowHeader.setFixedCellHeight(this.table.getRowHeight());
		rowHeader.setCellRenderer(new EditorPanelRowRenderer(this.table));

		if (this.scroll == null) {
			this.scroll = new JScrollPane(this.table);
			this.panel.add(this.scroll);
		}
		this.scroll.setRowHeaderView(rowHeader);
	}

	/**
	 * resizes Tnumeric
	 */
	void tnumResize() {

		int rows;
		int columns;
		double re[][] = null;
		double im[][] = null;

		final double[][] oldRe = ((TNumeric) this.selection).getRealMatrix().getData();
		final double[][] oldIm = ((TNumeric) this.selection).getImagMatrix().getData();

		try {
			rows = Integer.parseInt(this.rows.getText());
			columns = Integer.parseInt(this.cols.getText());
		} catch (final Exception e) {
			return;
		}
		this.rows.setText("rows");
		this.cols.setText("columns");

		re = new double[rows][columns];
		im = new double[rows][columns];

		for (int x = 0; x < rows; x++) {
			for (int y = 0; y < columns; y++) {

				try {
					re[x][y] = oldRe[x][y];
				} catch (final Exception e) {
					re[x][y] = 0.0;

				}

				try {
					im[x][y] = oldIm[x][y];
				} catch (final Exception e) {
					im[x][y] = 0.0;
				}

			}
		}

		try {
			this.selection = new TNumeric(re, im);
		} catch (final TDataException e) {
			e.printStackTrace();
		}
		this.reload();
		this.columsResize(this.table, 90);
		this.createHeaders();
	}

}
