/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor;

import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Vector;

import javax.swing.AbstractListModel;
import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListModel;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;

import at.lcm.bu21.general.dtypes.TPeriodic;

/**
 * Generator Table Class
 * 
 * @author Christoph Halbmayr
 */
class TPeriodicGeneratorTable {

	/** DefaultTableModel */
	static DefaultTableModel model;

	/** column Names */
	private Vector<String> columnNames;

	/** Table Editor for datatypes */
	private JComboBox dataTypes;

	/** Ground Signal Generator */
	private TPeriodicGroundSignGen gsg;

	/**
	 * JScrollPane
	 */
	private JScrollPane scroll;

	/** Table Model Listener */
	private TableModelListener tml;

	/** TPeriodic Vector */
	Vector<TPeriodic> data = new Vector<TPeriodic>();

	/** new Row Flag */
	boolean newPerRow = false;

	/** Generator Table */
	JTable table;

	/**
	 * Creates a new Generator Table
	 */
	TPeriodicGeneratorTable() {
		this.gsg = new TPeriodicGroundSignGen();

		this.columnNames = new Vector<String>();
		this.columnNames.add("Signal");
		this.columnNames.add("Offset");

		this.columnNames.add("Amplitude");
		this.columnNames.add("Phase");
		this.columnNames.add("Einschaltdauer");

		TPeriodicGeneratorTable.model = new DefaultTableModel(this.columnNames, 0);

		this.table = new JTable(TPeriodicGeneratorTable.model);

		final MouseListener ml = new MouseListener() {

			@Override
			public void mouseClicked(final MouseEvent e) {

			}

			@Override
			public void mouseEntered(final MouseEvent e) {

			}

			@Override
			public void mouseExited(final MouseEvent e) {

			}

			@Override
			public void mousePressed(final MouseEvent e) {

				final int row = TPeriodicGeneratorTable.this.table.getSelectedRow();
				final int col = TPeriodicGeneratorTable.this.table.getSelectedColumn();

				TPeriodicGeneratorTable.this.setHeaders();

				if (e.getClickCount() == 2) {
					if (col != 0) {
						TPeriodicGeneratorTable.this.table.editCellAt(row, col);
					}
					TPeriodicGeneratorTable.this.table.changeSelection(row, col, false, false);
				} else {
					TPeriodicGeneratorTable.this.table.changeSelection(row, col, false, false);
				}
			}

			@Override
			public void mouseReleased(final MouseEvent e) {

			}
		};

		this.table.setAutoResizeMode(0);
		this.table.addMouseListener(ml);
		this.table.setDefaultEditor(Object.class, new DefaultCellEditor(new JTextField()) {

			/** Serialization version number */
			private static final long serialVersionUID = 1L;

			@Override
			public Component getTableCellEditorComponent(final JTable table, final Object value,
					final boolean isSelected, final int row, final int column) {
				Component c;
				c = super.getTableCellEditorComponent(table, value, isSelected, row, column);
				((JTextField) c).selectAll();

				return c;
			}
		});

		this.dataTypes = new JComboBox();

		this.dataTypes.addItem("rect");
		this.dataTypes.addItem("triangle");
		this.dataTypes.addItem("sin");

		this.table.getColumnModel().getColumn(0).setCellEditor(new DefaultCellEditor(this.dataTypes));

		this.table.getTableHeader().setReorderingAllowed(false);
		this.table.getTableHeader().setResizingAllowed(false);
		this.table.setRowSelectionAllowed(true);
		this.table.setColumnSelectionAllowed(true);

		this.addNewVarRow();
		this.createHeaders();
	}

	/**
	 * Adds a TableModelListener
	 * 
	 * @param tml
	 *            TableModelListener
	 */
	public void addTML(final TableModelListener tml) {
		TPeriodicGeneratorTable.model.addTableModelListener(tml);
		this.tml = tml;
	}

	/**
	 * Clears the TableModel
	 */
	public void clear() {
		TPeriodicGeneratorTable.model.getDataVector().removeAllElements();
		this.newPerRow = false;
		this.data.removeAllElements();
		this.addNewVarRow();
		this.createHeaders();
	}

	/**
	 * @return the scroll
	 */
	public JScrollPane getScrollPane() {
		return this.scroll;
	}

	/**
	 * Is triggered by the Table Model Listener
	 * 
	 * @param maxOrder
	 */
	public void tableChanged(final int maxOrder) {
		final int row = this.table.getSelectedRow();
		try {
			final String waveform = TPeriodicGeneratorTable.model.getValueAt(row, 0).toString();

			if ((waveform != "[Signal]")) {
				if (row == (this.table.getRowCount() - 1)) {
					this.newTPeriodic(maxOrder, row);
				} else {
					this.changeTPeriodic(maxOrder, row);
				}
			}
		} catch (final Exception e) {
		}

	}

	/**
	 * changes the selected TPeriodic
	 * 
	 * @param maxOrder
	 * @param row
	 */
	private void changeTPeriodic(final int maxOrder, final int row) {
		this.gsg.setMaxOrder(maxOrder);

		final String waveform = TPeriodicGeneratorTable.model.getValueAt(row, 0).toString();
		final double offs = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 1).toString());
		final double ampl = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 2).toString());
		final double phas = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 3).toString());
		final double atim = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 4).toString());

		TPeriodic per = null;

		if (waveform.equals("rect")) {
			per = this.gsg.genRect(offs, ampl, phas, atim);
		} else if (waveform.equals("triangle")) {
			per = this.gsg.genTriangle(offs, ampl, phas, atim);
		} else if (waveform.equals("sin")) {
			per = this.gsg.genSin(offs, ampl, phas, (int) atim);
		}
		this.data.set(row, per);
	}

	/**
	 * creats Table row headers
	 */
	private void createHeaders() {
		final String[] headers = new String[this.table.getRowCount()];

		for (int i = 0; i < this.table.getRowCount(); i++) {
			headers[i] = "" + i;
		}

		final ListModel lm = new AbstractListModel() {

			/** Serialization version number */
			private static final long serialVersionUID = 1L;

			@Override
			public Object getElementAt(final int index) {
				return headers[index];
			}

			@Override
			public int getSize() {
				return headers.length;
			}
		};

		final JList rowHeader = new JList(lm);
		rowHeader.setFixedCellWidth(25);

		rowHeader.setFixedCellHeight(this.table.getRowHeight());
		rowHeader.setCellRenderer(new EditorPanelRowRenderer(this.table));
		this.table.getTableHeader().setReorderingAllowed(false);
		if (this.scroll == null) {
			this.scroll = new JScrollPane(this.table);
		}
		this.scroll.setRowHeaderView(rowHeader);
	}

	/**
	 * creates a new TPeriodic
	 * 
	 * @param maxOrder
	 * @param row
	 */
	private void newTPeriodic(final int maxOrder, final int row) {

		this.gsg.setMaxOrder(maxOrder);

		final String waveform = TPeriodicGeneratorTable.model.getValueAt(row, 0).toString();
		final double offs = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 1).toString());
		final double ampl = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 2).toString());
		final double phas = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 3).toString());

		TPeriodic per = null;

		if (waveform.equals("rect")) {
			final double atim = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 4).toString());

			per = this.gsg.genRect(offs, ampl, phas, atim);

		} else if (waveform.equals("triangle")) {
			final double atim = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 4).toString());

			per = this.gsg.genTriangle(offs, ampl, phas, atim);
		} else if (waveform.equals("sin")) {

			TPeriodicGeneratorTable.model.removeTableModelListener(this.tml);
			this.table.setValueAt("1", row, 4);
			per = this.gsg.genSin(offs, ampl, phas, 1);
			TPeriodicGeneratorTable.model.addTableModelListener(this.tml);

		}

		this.data.add(per);
		this.newPerRow = false;
		this.addNewVarRow();
		this.createHeaders();
		this.setHeaders();
	}

	/**
	 * sets the headers Order/Einschaltdauer
	 */
	private void setHeaders() {
		final int row = this.table.getSelectedRow();

		final String waveform = TPeriodicGeneratorTable.model.getValueAt(row, 0).toString();
		if ((row != -1) && (waveform.equals("[Signal]") == false)) {
			if (waveform.equals("sin") && this.columnNames.get(4).equals("Einschaltdauer")) {

				if (TPeriodicGeneratorTable.model.getValueAt(row, 4).equals("180.0")) {
					TPeriodicGeneratorTable.model.setValueAt("1", row, 4);
				}

				this.columnNames.set(4, "Order");
				TPeriodicGeneratorTable.model.setColumnIdentifiers(this.columnNames);
				this.table.getColumnModel().getColumn(0).setCellEditor(new DefaultCellEditor(this.dataTypes));

			} else if ((waveform.equals("sin") == false) && this.columnNames.get(4).equals("Order")) {
				if (TPeriodicGeneratorTable.model.getValueAt(row, 4).equals("180.0")) {
				}
				this.columnNames.set(4, "Einschaltdauer");
				TPeriodicGeneratorTable.model.setColumnIdentifiers(this.columnNames);
				this.table.getColumnModel().getColumn(0).setCellEditor(new DefaultCellEditor(this.dataTypes));

			}
		}

	}

	/**
	 * Adds the adding row
	 */
	void addNewVarRow() {

		if (this.newPerRow == false) {

			final String[] o = { "[Signal]", "0.0", "1.0", "0.0", "180.0" };

			if (TPeriodicGeneratorTable.model.getColumnCount() == 1) {
				TPeriodicGeneratorTable.model.addColumn("1");
			}
			this.newPerRow = true;
			TPeriodicGeneratorTable.model.addRow(o);
		}

	}

	/**
	 * Handles a change in the max Order Field
	 * 
	 * @param maxorder
	 */
	void maxOrderChanged(final int maxorder) {
		this.gsg.setMaxOrder(maxorder);

		System.out.println(this.table.getRowCount());
		for (int row = 0; row < (this.table.getRowCount() - 1); row++) {

			final String waveform = TPeriodicGeneratorTable.model.getValueAt(row, 0).toString();
			final double offs = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 1).toString());
			final double ampl = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 2).toString());
			final double phas = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 3).toString());

			TPeriodic per = null;

			if (waveform.equals("rect")) {
				final double atim = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 4).toString());

				per = this.gsg.genRect(offs, ampl, phas, atim);

			} else if (waveform.equals("triangle")) {
				final double atim = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 4).toString());
				per = this.gsg.genTriangle(offs, ampl, phas, atim);
			}

			else if (waveform.equals("sin")) {
				final double order = Double.parseDouble(TPeriodicGeneratorTable.model.getValueAt(row, 4).toString());
				per = this.gsg.genSin(offs, ampl, phas, (int) order);
			}

			this.data.set(row, per);
		}
	}

	/**
	 * removes the adding row
	 */
	void removeNewVarRow() {
		if (this.newPerRow == true) {
			TPeriodicGeneratorTable.model.removeRow(TPeriodicGeneratorTable.model.getRowCount() - 1);
			this.newPerRow = false;
		}

	}

}
