/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor2.TDataModel;

import java.util.Arrays;

import at.lcm.bu21.general.dtypes.TData;
import at.lcm.bu21.general.dtypes.TNull;
import at.lcm.bu21.general.dtypes.TStruct;

/**
 * Representation of a TStructModel - type
 * 
 * @author Johannes Kaisinger
 */
public class TStructModel extends TDataParentModel {

	/** Data variable */
	private TStruct data;

	/** key of child for data manipulation */
	private String childKey;

	/**
	 * Create object
	 * 
	 */
	public TStructModel() {
		this.data = new TStruct();
		this.data.put("[NAME]_0", new TNull());
	}

	/**
	 * Create object with given data (TStruct)
	 * 
	 * @param data
	 *            Reference of data (TStruct)
	 */
	public TStructModel(TStruct data) {
		this.data = data;
	}

	/**
	 * Get all data (TStruct)
	 * 
	 * @return data Reference of data (TStruct)
	 */
	public TStruct getTStruct() {
		return this.data;
	}

	/**
	 * Get all data (TData)
	 * 
	 * @return Data Reference of data (TData)
	 */
	@Override
	public TData getData() {
		return this.data;
	}

	/**
	 * Get a single value as String
	 * 
	 * @param key
	 *            Key of child
	 * 
	 * @return child Values of child (String)
	 */
	public String getValue(String key) {
		TData child = this.data.get(key);
		return child.toString();
	}

	/**
	 * Get the class name of a child
	 * 
	 * @param key
	 *            key of child
	 * 
	 * @return className Class of child (String)
	 */
	public String getClassName(String key) {
		TData child = this.data.get(key);
		String className = child.getClassName();
		return className;
	}

	/**
	 * Get all child keys
	 * 
	 * @return allKeys Keys of children
	 */
	public String[] getKeys() {
		String[] allKeys = this.data.getKeys();
		return allKeys;
	}

	/**
	 * Get child
	 * 
	 * @param key
	 *            Key of child
	 * @return child Child (TData)
	 */
	public TDataModel getChildModel(String key) {
		this.childKey = key;
		return TDataModel.createTDataModel(this.data.get(key));
	}

	@Override
	public String getChildName() {
		return this.childKey;
	}

	/**
	 * Replace child after editing.
	 * 
	 */
	@Override
	public void replaceChild(TData value) {
		this.data.put(this.childKey, value);
	}

	/**
	 * Add a new child (TNull)
	 * 
	 * @return key New child key (String)
	 */
	public String addChild() {

		TData value = new TNull();

		String key = "[NAME]_";
		String newKey = key;

		int i = 0;
		do {
			newKey = key + i;
			i++;
		} while (this.data.get(newKey) != null);

		this.data.put(newKey, value);
		return newKey;
	}

	/**
	 * Removes a child
	 * 
	 * @param key
	 *            Removes with the key
	 */
	public void removeChild(String key) {
		this.data.remove(key);
	}

	/**
	 * Set a child
	 * 
	 * @param key
	 *            Key of child
	 * @param value
	 *            Value of child (TData)
	 */
	public void setChild(String key, TData value) {
		this.data.put(key, value);
	}

	/**
	 * Change the key name
	 * 
	 * @param oldName
	 *            old key name of child
	 * @param newName
	 *            new key name of child
	 */
	public void changeKey(String oldName, String newName) {
		TData child = this.data.get(oldName);

		String[] allKeys = this.data.getKeys();

		if (!Arrays.asList(allKeys).contains(newName)) {
			this.data.remove(oldName);
			this.data.put(newName, child);
		}
	}
}
