/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor2;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.table.JTableHeader;

import at.lcm.bu21.general.dtypes.editor.icons.IconEditor;
import at.lcm.bu21.general.dtypes.editor2.TDataModel.TPeriodicMatrixModel;

/**
 * This panel allows to analyze and manipulate TPeriodicMatrices. It based on
 * TPeriodicMatrixModel and tPeriodicMatrixTableModel for JTables.
 * 
 * @author Johannes Kaisinger
 *
 */
public class TPeriodicMatrixPanel extends JPanel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/**
	 * Local reference of the tPeriodicMatrixModel. It allows to manipulate the
	 * TPeriodicMatrix.
	 */
	private TPeriodicMatrixModel tPeriodicMatrixModel;

	/** Contains a table model for the JTable. */
	private TPeriodicMatrixTableModel tableModel;

	/** Editor as table. */
	private JTable table;

	/** Contains a Listener. */
	private EditChildListener editChildListener;

	/**
	 * @param tPeriodicMatrixModel
	 *            TPeriodicMatrixModel
	 */
	public TPeriodicMatrixPanel(TPeriodicMatrixModel tPeriodicMatrixModel) {

		this.tPeriodicMatrixModel = tPeriodicMatrixModel;

		this.setLayout(new BorderLayout());

		// setup toolbar
		JToolBar toolbar = this.createToolBar();
		this.add(toolbar, BorderLayout.NORTH);

		// set up table
		this.tableModel = new TPeriodicMatrixTableModel(tPeriodicMatrixModel);
		this.table = new JTable(this.tableModel) {

			/** Serialization version number */
			private static final long serialVersionUID = 1L;

			// Implement table cell tool tips.
			@Override
			public String getToolTipText(MouseEvent e) {
				String tip = null;
				java.awt.Point p = e.getPoint();
				int rowIndex = this.rowAtPoint(p);
				int colIndex = this.columnAtPoint(p);

				try {
					tip = this.getValueAt(rowIndex, colIndex).toString();
				} catch (RuntimeException e1) {
					// catch null pointer exception if mouse is over an empty
					// line
				}

				return tip;
			}
		};
		this.table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

		this.table.setCellSelectionEnabled(true);
		this.table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		this.table.addRowSelectionInterval(0, 0);
		this.table.addColumnSelectionInterval(0, 0);

		// configurate table Header
		JTableHeader tableHeader = this.table.getTableHeader();
		tableHeader.setReorderingAllowed(false);

		JScrollPane scroll = new JScrollPane(this.table);

		RowHeaderTable rowHeaderTable = new RowHeaderTable(this.table);
		scroll.setRowHeaderView(rowHeaderTable);
		scroll.setCorner(ScrollPaneConstants.UPPER_LEFT_CORNER, rowHeaderTable.getTableHeader());

		this.createPopupMenu();

		this.add(scroll, BorderLayout.CENTER);
	}

	/**
	 * Set a edit child listener.
	 * 
	 * @param editChildListener
	 *            The EditChildListener
	 */
	public void setEditChildListener(EditChildListener editChildListener) {
		this.editChildListener = editChildListener;
	}

	/**
	 * Throws an edit child event.
	 */
	private void editChild() {
		int row = this.table.getSelectedRow();
		int column = this.table.getSelectedColumn();

		if ((row > -1) && (column > -1)) {
			EditChildEvent ev = new EditChildEvent(this, row, column, "");

			if (this.editChildListener != null) {
				this.editChildListener.editChildOccurred(ev);
			}
		}
	}

	/**
	 * Insert a row above the selected row; Selects the new row;
	 */
	private void insertRowAbove() {
		int row = this.table.getSelectedRow();
		int column = this.table.getSelectedColumn();

		if ((row > -1) && (column > -1)) {
			this.tPeriodicMatrixModel.insertRow(row);
			this.tableModel.fireTableDataChanged();

			this.table.addRowSelectionInterval(row, row);
			this.table.addColumnSelectionInterval(column, column);
		}
	}

	/**
	 * Insert a row above the selected row; Selects the new row;
	 */
	private void insertRowBelow() {
		int row = this.table.getSelectedRow();
		int column = this.table.getSelectedColumn();

		if ((row > -1) && (column > -1)) {
			this.tPeriodicMatrixModel.insertRow(row + 1);
			this.tableModel.fireTableDataChanged();

			this.table.addRowSelectionInterval(row + 1, row + 1);
			this.table.addColumnSelectionInterval(column, column);
		}
	}

	/**
	 * Insert a column to the left side the selected column; Selects the new
	 * column;
	 */
	private void insertColumnLeft() {
		int row = this.table.getSelectedRow();
		int column = this.table.getSelectedColumn();

		if ((row > -1) && (column > -1)) {
			this.tPeriodicMatrixModel.insertColumn(column);
			this.tableModel.fireTableStructureChanged();

			this.table.addRowSelectionInterval(row, row);
			this.table.addColumnSelectionInterval(column, column);
		}
	}

	/**
	 * Insert a column to the right side the selected column; Selects the new
	 * column;
	 */
	private void insertColumnRight() {
		int row = this.table.getSelectedRow();
		int column = this.table.getSelectedColumn();

		if ((row > -1) && (column > -1)) {
			this.tPeriodicMatrixModel.insertColumn(column + 1);
			this.tableModel.fireTableStructureChanged();

			this.table.addRowSelectionInterval(row, row);
			this.table.addColumnSelectionInterval(column + 1, column + 1);
		}
	}

	/**
	 * Delete the selected row;
	 */
	private void deleteRow() {
		int row = this.table.getSelectedRow();
		int column = this.table.getSelectedColumn();
		int maxRow = this.table.getRowCount();

		if ((maxRow > 1) && (row > -1) && (column > -1)) {
			this.tPeriodicMatrixModel.deleteRow(row);
			this.tableModel.fireTableDataChanged();

			if (row < (maxRow - 1)) {
				this.table.addRowSelectionInterval(row, row);
				this.table.addColumnSelectionInterval(column, column);
			} else if (row == (maxRow - 1)) {
				this.table.addRowSelectionInterval(row - 1, row - 1);
				this.table.addColumnSelectionInterval(column, column);
			}
		}
	}

	/**
	 * Delete the selected column;
	 */
	private void deleteColumn() {
		int row = this.table.getSelectedRow();
		int column = this.table.getSelectedColumn();
		int maxColumn = this.table.getColumnCount();

		if ((maxColumn > 1) && (row > -1) && (column > -1)) {
			this.tPeriodicMatrixModel.deleteColumn(column);
			this.tableModel.fireTableStructureChanged();

			if (column < (maxColumn - 1)) {
				this.table.addRowSelectionInterval(row, row);
				this.table.addColumnSelectionInterval(column, column);
			} else if (column == (maxColumn - 1)) {
				this.table.addRowSelectionInterval(row, row);
				this.table.addColumnSelectionInterval(column - 1, column - 1);
			}
		}
	}

	/**
	 * Create the toolBar for the TPeriodicMatrixPanel;
	 * 
	 * @return toolBAr
	 */
	private JToolBar createToolBar() {

		JToolBar tb = new JToolBar();
		tb.setPreferredSize(new Dimension(500, 50));

		Vector<JButton> buttons = new Vector<JButton>();

		final JButton addRowAbove = new JButton("Above", IconEditor.ICON_ADD);
		addRowAbove.setToolTipText("Insert a row ");
		addRowAbove.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.insertRowAbove();
			}
		});
		buttons.add(addRowAbove);

		final JButton addRowBelow = new JButton("Below", IconEditor.ICON_ADD);
		addRowBelow.setToolTipText("Insert a row ");
		addRowBelow.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.insertRowBelow();
			}
		});
		buttons.add(addRowBelow);

		final JButton addColLeft = new JButton("Left", IconEditor.ICON_ADD);
		addColLeft.setToolTipText("Insert a column");
		addColLeft.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.insertColumnLeft();
			}
		});
		buttons.add(addColLeft);

		final JButton addColRight = new JButton("Right", IconEditor.ICON_ADD);
		addColRight.setToolTipText("Insert a column");
		addColRight.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.insertColumnRight();
			}
		});
		buttons.add(addColRight);

		final JButton delRow = new JButton("Row", IconEditor.ICON_REMOVE);
		delRow.setToolTipText("Delete selected row");
		delRow.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.deleteRow();
			}
		});
		buttons.add(delRow);

		final JButton delCol = new JButton("Column", IconEditor.ICON_REMOVE);
		delCol.setToolTipText("Delete selected column");
		delCol.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.deleteColumn();
			}
		});
		buttons.add(delCol);

		final JButton showTime = new JButton("Time", IconEditor.ICON_XYPLOT);
		showTime.setToolTipText("Show time plot");
		showTime.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.tPeriodicMatrixModel.showTime();
			}
		});
		buttons.add(showTime);

		final JButton showSpectrum = new JButton("Spectrum", IconEditor.ICON_SPECTRUM);
		showSpectrum.setToolTipText("Show spectrum plot");
		showSpectrum.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.tPeriodicMatrixModel.showSpectrum();
			}
		});
		buttons.add(showSpectrum);

		for (final JButton i : buttons) {

			Dimension d = new Dimension(55, 50);
			i.setVerticalTextPosition(SwingConstants.BOTTOM);
			i.setHorizontalTextPosition(SwingConstants.CENTER);
			i.setMaximumSize(d);
			i.setMinimumSize(d);
			i.setPreferredSize(d);
			tb.add(i);
		}

		return tb;
	}

	/**
	 * Create the popupMenu and mousListener for the TPeriodicMatrixPanel;
	 * 
	 * @return popupMenu
	 */
	private JPopupMenu createPopupMenu() {

		final JPopupMenu popupMenu = new JPopupMenu();

		JMenuItem insertRowAbove = new JMenuItem("Insert row above");
		JMenuItem insertRowBelow = new JMenuItem("Insert row below");
		JMenuItem insertColumnLeft = new JMenuItem("Insert column to the left");
		JMenuItem insertColumnRight = new JMenuItem("Insert column to the right");
		JMenuItem deleteRow = new JMenuItem("Delete row");
		JMenuItem deleteColumn = new JMenuItem("Delete column");
		JMenuItem editChild = new JMenuItem("Edit child");

		popupMenu.add(insertRowAbove);
		popupMenu.add(insertRowBelow);
		popupMenu.add(insertColumnLeft);
		popupMenu.add(insertColumnRight);
		popupMenu.add(deleteRow);
		popupMenu.add(deleteColumn);
		popupMenu.addSeparator();
		popupMenu.add(editChild);

		insertRowAbove.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.insertRowAbove();
			}
		});

		insertRowBelow.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.insertRowBelow();
			}
		});

		insertColumnLeft.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.insertColumnLeft();
			}
		});

		insertColumnRight.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.insertColumnRight();
			}
		});

		deleteRow.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.deleteRow();
			}
		});

		deleteColumn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.deleteColumn();
			}
		});

		editChild.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				TPeriodicMatrixPanel.this.editChild();
			}
		});

		this.table.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					TPeriodicMatrixPanel.this.editChild();
				}
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				if (e.isPopupTrigger()) {
					JTable source = (JTable) e.getSource();
					int row = source.rowAtPoint(e.getPoint());
					int column = source.columnAtPoint(e.getPoint());

					if (!source.isRowSelected(row)) {
						source.changeSelection(row, column, false, false);
					}
					popupMenu.show(e.getComponent(), e.getX(), e.getY());
				}
			}
		});

		return popupMenu;
	}
}
