/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 997 $
 * $Date: 2020-03-19 13:49:34 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.general.math.autodiff;

import at.lcm.bu21.general.math.interpolator2d.DelaunayInt2d;
import at.lcm.bu21.general.math.interpolator2d.Interpolator2d;
import at.lcm.bu21.general.math.interpolator2d.RbfInt2d;

/**
 * 
 * 
 * @author Gnther Weidenholzer
 */
public class Interp2d extends Func {

	/** Interpolator */
	private Interpolator2d interp;

	/** x-variable */
	Func x;

	/** y-variable */
	Func y;

	/**
	 * Algorithm selection
	 */
	public static enum Algorithm {
		/** Radial basis function interpolation */
		RBF,
		/** Interpolation by Delaunay triangulation */
		DELAUNAY; // ; is optional
	}

	/**
	 * 
	 * @param xdata
	 *            x data vector
	 * @param ydata
	 *            y data vector
	 * @param zdata
	 *            z data vector
	 * @param x
	 *            Function x
	 * @param y
	 *            Function y
	 * @param alg
	 *            Algorithm
	 * @param allowX
	 *            Allow extrapolation
	 * @throws Exception
	 *             Interpolate exception
	 */
	public Interp2d(double[] xdata, double[] ydata, double[] zdata, Func x,
			Func y, Algorithm alg, boolean allowX) throws Exception {

		super(x, y);

		switch (alg) {
		case RBF:
			this.interp = new RbfInt2d(xdata, ydata, zdata);
			break;
		case DELAUNAY:
			this.interp = new DelaunayInt2d(xdata, ydata, zdata);
			break;
		}
		this.interp.allowExtrapolation(allowX);

	}

	/**
	 * 2d Interpolation. Using default algorithm: RBF
	 * 
	 * @param xdata
	 *            x data vector
	 * @param ydata
	 *            y data vector
	 * @param zdata
	 *            z data vector
	 * @param x
	 *            Function x
	 * @param y
	 *            Function y
	 * @throws Exception
	 *             Interpolate exception
	 */
	public Interp2d(double[] xdata, double[] ydata, double[] zdata, Func x,
			Func y) throws Exception {

		this(xdata, ydata, zdata, x, y, Interp2d.Algorithm.RBF, true);

	}

	@Override
	public double value() throws Exception {

		double x = this.getArg(0).value();
		double y = this.getArg(1).value();

		return this.interp.value(x, y);
	}

	@Override
	protected Func derivativeArg(Func f) throws Exception {
		Func func = null;

		if (f == this.getArg(0)) {
			func = new d_dx(this.getArg(0), this.getArg(1));
		} else if (f == this.getArg(1)) {
			func = new d_dy(this.getArg(0), this.getArg(1));
		}

		return func;
	}

	/**
	 * 
	 * @author gweidenholzer
	 * 
	 */
	class d_dx extends Func {

		/**
		 * 
		 * @param x
		 * @param y
		 */
		public d_dx(Func x, Func y) {
			super(x, y);
		}

		@Override
		public double value() throws Exception {
			double x = this.getArg(0).value();
			double y = this.getArg(1).value();
			return Interp2d.this.interp.dz_dx(x, y);
		}

		@Override
		protected Func derivativeArg(Func f) throws Exception {
			Func func = null;

			if (f == this.getArg(0)) {
				func = new d_dx_dx(this.getArg(0), this.getArg(1));
			} else if (f == this.getArg(1)) {
				func = new d_dx_dy(this.getArg(0), this.getArg(1));
			}

			return func;
		}

		/**
		 * 
		 * @author gweidenholzer
		 * 
		 */
		class d_dx_dx extends Func {

			/**
			 * 
			 * @param x
			 * @param y
			 */
			public d_dx_dx(Func x, Func y) {
				super(x, y);
			}

			@Override
			public double value() throws Exception {
				double x = this.getArg(0).value();
				double y = this.getArg(1).value();
				return Interp2d.this.interp.dz_dx2(x, y);
			}

			@Override
			protected Func derivativeArg(Func f) throws Exception {
				return new Constant(0.0);
			}
		}

		/**
		 * 
		 * @author gweidenholzer
		 * 
		 */
		class d_dx_dy extends Func {

			/**
			 * 
			 * @param x
			 * @param y
			 */
			public d_dx_dy(Func x, Func y) {
				super(x, y);
			}

			@Override
			public double value() throws Exception {
				double x = this.getArg(0).value();
				double y = this.getArg(1).value();
				return Interp2d.this.interp.dz_dxy(x, y);
			}

			@Override
			protected Func derivativeArg(Func f) throws Exception {
				return new Constant(0.0);
			}
		}

	}

	/**
	 * 
	 * @author gweidenholzer
	 * 
	 */
	class d_dy extends Func {

		/**
		 * 
		 * @param x
		 * @param y
		 */
		public d_dy(Func x, Func y) {
			super(x, y);
		}

		@Override
		public double value() throws Exception {
			double x = this.getArg(0).value();
			double y = this.getArg(1).value();
			return Interp2d.this.interp.dz_dy(x, y);
		}

		@Override
		protected Func derivativeArg(Func f) throws Exception {
			Func func = null;

			if (f == this.getArg(0)) {
				func = new d_dy_dx(this.getArg(0), this.getArg(1));
			} else if (f == this.getArg(1)) {
				func = new d_dy_dy(this.getArg(0), this.getArg(1));
			}

			return func;
		}

		/**
		 * 
		 * @author gweidenholzer
		 * 
		 */
		class d_dy_dx extends Func {

			/**
			 * 
			 * @param x
			 * @param y
			 */
			public d_dy_dx(Func x, Func y) {
				super(x, y);
			}

			@Override
			public double value() throws Exception {
				double x = this.getArg(0).value();
				double y = this.getArg(1).value();
				return Interp2d.this.interp.dz_dxy(x, y);
			}

			@Override
			protected Func derivativeArg(Func f) throws Exception {
				return new Constant(0.0);
			}
		}

		/**
		 * 
		 * @author gweidenholzer
		 * 
		 */
		class d_dy_dy extends Func {

			/**
			 * 
			 * @param x
			 * @param y
			 */
			public d_dy_dy(Func x, Func y) {
				super(x, y);
			}

			@Override
			public double value() throws Exception {
				double x = this.getArg(0).value();
				double y = this.getArg(1).value();
				return Interp2d.this.interp.dz_dy2(x, y);
			}

			@Override
			protected Func derivativeArg(Func f) throws Exception {
				return new Constant(0.0);
			}
		}

	}

}
