/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 997 $
 * $Date: 2020-03-19 13:49:34 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.general.math.interpolator2d;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Path2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * @author gweidenholzer
 * 
 */
public class ConvexHull {

	private Path2D hull = null;
	private Point[] points = null;

	/**
	 * @param x
	 * @param y
	 * @throws Exception
	 */
	public ConvexHull(double[] x, double[] y) throws Exception {

		ArrayList<Point> pts = new ArrayList<Point>();

		if (x.length != y.length) {
			throw new Exception(
					"ConvexHull error: length of x and y-coordinate arrays must be equal!");
		}

		// eliminate duplicates
		double xmin = Double.POSITIVE_INFINITY;
		double xmax = Double.NEGATIVE_INFINITY;
		double ymin = Double.POSITIVE_INFINITY;
		double ymax = Double.NEGATIVE_INFINITY;

		for (int k = 0; k < x.length; k++) {
			xmin = Math.min(xmin, x[k]);
			xmax = Math.max(xmax, x[k]);
			ymin = Math.min(ymin, y[k]);
			ymax = Math.max(ymax, y[k]);
		}

		double xrange = xmax - xmin;
		double yrange = ymax - ymin;

		for (int k = 0; k < x.length; k++) {
			boolean eq = false;
			for (Point p : pts) {
				double xdist = (p.x - x[k]) / xrange;
				double ydist = (p.y - y[k]) / yrange;
				if (Math.hypot(xdist, ydist) < 1e-4) {
					eq = true;
					break;
				}
			}
			if (!eq) {
				pts.add(new Point(x[k], y[k]));
			}
		}

		this.points = pts.toArray(new Point[0]);

		this.hull = this.findHull(this.points);

	}

	/**
	 * Tests if convex hull contains point
	 * 
	 * @param x
	 * @param y
	 * @return true if hull contains point
	 */
	public boolean contains(double x, double y) {
		return this.hull.contains(x, y);
	}

	/**
	 * calculate area inside convex hull
	 * 
	 * @return area
	 */
	public double area() {

		double area = 0;

		PathIterator pit = this.hull.getPathIterator(new AffineTransform());

		double[] p1 = new double[2];
		double[] p2 = new double[2];

		int cnt = 0;

		while (true) {
			pit.currentSegment(p1);
			pit.next();
			pit.currentSegment(p2);

			if (pit.isDone()) {
				break;
			}

			area += (p1[0] * p2[1]) - (p2[0] * p1[1]);
			// System.out.format("%f %f %f %f\n", p1[0],p1[1],p2[0],p2[1]);
			cnt++;
		}

		return Math.abs(area / 2);
	}

	/**
	 * 
	 * @param p
	 * @return minimal distance to convex hull
	 */
	public double minDistToHull(double[] p) {

		PathIterator pit = this.hull.getPathIterator(new AffineTransform());
		double r = Double.POSITIVE_INFINITY;

		int cnt = 0;
		while (true) {

			cnt++;

			double[] p1 = new double[2];
			pit.currentSegment(p1);

			pit.next();

			double[] p2 = new double[2];
			pit.currentSegment(p2);

			if (pit.isDone()) {
				break;
			}

			double tmp = Line2D.ptLineDist(p1[0], p1[1], p2[0], p2[1], p[0],
					p[1]);
			if (!Double.isNaN(tmp)) {
				r = Math.min(r, tmp);
			}
		}

		return r;
	}

	/**
	 * 
	 * @author gweidenholzer
	 * 
	 */
	private class Point implements Comparable<Point> {
		double x, y;

		Point(double x, double y) {
			this.x = x;
			this.y = y;
		}

		// sort first on x then on y
		@Override
		public int compareTo(Point other) {
			if (this.x == other.x) {
				return (int) Math.signum(this.y - other.y);
			} else {
				return (int) Math.signum(this.x - other.x);
			}
		}

		// cross product of two vectors
		public double cross(Point p) {
			return (this.x * p.y) - (this.y * p.x);
		}

		// subtraction of two points
		public Point sub(Point p) {
			return new Point(this.x - p.x, this.y - p.y);
		}

		public Point2D p2d() {
			return new Point2D.Double(this.x, this.y);
		}

	}

	/**
	 * 
	 * @param points
	 * @return
	 */
	private Path2D findHull(Point[] points) {
		int n = points.length;
		Arrays.sort(points);
		Point[] ans = new Point[2 * n]; // In between we may have a 2n points
		int k = 0;
		int start = 0; // start is the first insertion point

		for (int i = 0; i < n; i++) // Finding lower layer of hull
		{
			Point p = points[i];
			while (((k - start) >= 2)
					&& (p.sub(ans[k - 1]).cross(p.sub(ans[k - 2])) > 0)) {
				k--;
			}
			ans[k++] = p;
		}

		k--; // drop off last point from lower layer
		start = k;

		for (int i = n - 1; i >= 0; i--) // Finding top layer from hull
		{
			Point p = points[i];
			while (((k - start) >= 2)
					&& (p.sub(ans[k - 1]).cross(p.sub(ans[k - 2])) > 0)) {
				k--;
			}
			ans[k++] = p;
		}
		k--; // drop off last point from top layer

		Path2D ret = new Path2D.Double(new Line2D.Double(ans[0].p2d(),
				ans[1].p2d())); // hull would contain our convex hull to be
								// returned.

		for (int i = 1; i < k; i++) {
			ret.lineTo(ans[i].x, ans[i].y);
		}

		// close path
		ret.lineTo(ans[0].x, ans[0].y);

		return ret;
	}

	/**
	 * 
	 */
	public void preview() {

		JFrame f = new JFrame();
		f.setLayout(new BorderLayout());
		f.add(this.viewPanel());
		f.setSize(700, 700);
		f.setVisible(true);

	}

	/**
	 * 
	 * @return preview panel
	 */
	public JPanel viewPanel() {

		/**
		 * 
		 */
		JPanel pan = new JPanel() {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;

			@Override
			public void paintComponent(Graphics g) {

				double xmin = Double.POSITIVE_INFINITY;
				double xmax = Double.NEGATIVE_INFINITY;
				double ymin = Double.POSITIVE_INFINITY;
				double ymax = Double.NEGATIVE_INFINITY;

				for (int k = 0; k < ConvexHull.this.points.length; k++) {
					xmin = Math.min(xmin, ConvexHull.this.points[k].x);
					xmax = Math.max(xmax, ConvexHull.this.points[k].x);
					ymin = Math.min(ymin, ConvexHull.this.points[k].y);
					ymax = Math.max(ymax, ConvexHull.this.points[k].y);
				}

				double xscale = 600 / (xmax - xmin);
				double yscale = 600 / (ymax - ymin);

				AffineTransform at = AffineTransform.getScaleInstance(xscale,
						yscale);
				at.concatenate(AffineTransform.getTranslateInstance(-xmin,
						-ymin));

				Graphics2D g2 = (Graphics2D) g;
				g2.setPaint(Color.blue);
				g2.draw(at.createTransformedShape(ConvexHull.this.hull));

				g2.setPaint(Color.red);

				for (int k = 0; k < ConvexHull.this.points.length; k++) {
					g.fillOval(
							(int) (((ConvexHull.this.points[k].x - xmin) * xscale) - 3),
							(int) (((ConvexHull.this.points[k].y - ymin) * yscale) - 3),
							6, 6);
				}

			}
		};

		return pan;
	}

}
