"""
Created on 11.10.2016

@author: gweidenholzer

Radial basis function interpolations
ref. https://en.wikipedia.org/wiki/Polyharmonic_spline

"""

import numpy as np
from lcm.utils.pytperiodic import PyTPeriodicMatrix
from lcm.math.rbf import kernels
from lcm.math.rbf.core import calc_weigths, eval_z, eval_dz_dx, eval_r
from lcm.utils.serialization import Serializable


class RBF:
    """
    Solves multidimensional rbf problems and
    provide an interpolation function, therefor

    Attributes
    ----------
    xc_t : (xdim, ncenters) ndarray
        RBF center coordinates in normalized basis
    t : (x_dim, x_dim) ndarray of float
        Transformation matrix for input x
    kernel : lcm.rbf.kernel.AbstractKernel
        Function to calculate the RBF kernel.
        It expects an array_like argument (n_centers, n_samples) of radii
        and returns the RBF values in the same shape
    c : (zdim, ncenters) ndarray
        RBF weights
    d : (zdim, xdim+1) ndarray
        polynomial coefficients
    """
    @classmethod
    def fromMatrices(cls, xc, c, d, kernel, t=None, max_dist=0.) -> 'RBF':
        """
        Instantiate StackedMultiRBF from pre-calculated matrices

        Parameters
        ----------
        xc : np.ndarray
        c : np.ndarray
        d : np.ndarray
        kernel : kernels.AbstractKernel
        t : np.ndarray
        max_dist : float

        Returns
        -------
        RBF
        """
        return cls(xc, None, kernel, t, max_dist=max_dist, c=c, d=d)

    def __init__(self, xc, zc, kernel=None, t=None, gamma=0., max_dist=0., max_cond=0., **kwargs):
        """
        xc : (x_dim, n_centers) ndarray
            RBF center coordinates
        zc : (z_dim, n_centers) ndarray
            RBF center values.
        kernel : lcm.rbf.kernel.AbstractKernel
            Instance of type AbstractKernel to calculate the RBF kernel.
        t : (x_dim, x_dim) ndarray of float
            Transformation matrix for input x. Default matrix scales xc coordinates to a range of 1.0
        gamma : float
            smoothing parameter
        max_cond : float
            maximum condition number
        """

        ''' Preconditioning Matrix '''
        if t is None:
            # ToDo: should a normalization be used instead instead?
            t = np.eye(xc.shape[0])
            
        if kernel is None:
            kernel = kernels.KernelTPS()

        self.xc = xc
        self.t = t
        self.kernel = kernel
        self.max_dist = max_dist
        
        if zc is None:
            xc_t, c, d = self.__initFromMatrices(kwargs['c'], kwargs['d'])
        else:
            xc_t, c, d = self.__initFromSamples(zc, gamma, max_cond)

        self.xc_t = xc_t
        self.c = c
        self.d = d
        
    def __initFromSamples(self, zc, gamma, max_cond):
        """
        Initailize Xc_T, C and D matrices from Zc samples
        
        Parameters
        ----------
        zc : np.ndarray
        gamma: float
        max_cond : float

        Returns
        -------
        xc_t : np.ndarray
        c : np.ndarray
        d : np.ndarray
        """
        (xc_t, c, d) = calc_weigths(self.xc[:, :, np.newaxis], self.t[:, :, np.newaxis], zc[:, :, np.newaxis], 
                                    self.kernel, gamma, max_cond)
        xc_t = xc_t[:, :, 0]
        c = c[:, :, 0]
        d = d[:, :, 0]
        return xc_t, c, d

    def __initFromMatrices(self, c, d):
        """
        Initailize Xc_T, C and D matrices from matrices

        Parameters
        ----------
        c : np.ndarray
        d : np.ndarray

        Returns
        -------
        xc_t : np.ndarray
        c : np.ndarray
        d : np.ndarray
        """
        xc_t = np.matmul(self.t, self.xc)
        return xc_t, c, d

    def __call__(self, x):
        """
        Evaluates the RBF model

        Parameters
        ----------
        x : (x_dim, n_interpolations) ndarray of float
            periodic signals representing the coordinates where to interpolate

        Returns
        -------
        z : (z_dim, n_interpolations) ndarray
            result array
        """
        z = eval_z(self.xc_t[:, :, np.newaxis], self.c[:, :, np.newaxis], self.d[:, :, np.newaxis],
                   self.t[:, :, np.newaxis], x[:, :, np.newaxis], self.kernel, self.max_dist)
        z = z[:, :, 0]
        return z

    def evalRadius(self, x):
        """
        Evaluates the radius

        Parameters
        ----------
        x : (x_dim, n_interpolations) ndarray of float
            signal representing the coordinates where to evaluate

        Returns
        -------
        r : float
            radius
        """
        r = eval_r(self.xc_t[:, :, np.newaxis],
                   self.t[:, :, np.newaxis], x[:, :, np.newaxis])
        return r

    @property
    def xdim(self):
        return self.xc_t.shape[0]

    @property
    def zdim(self):
        return self.c.shape[0]

    def eval_dz_dx(self, x):
        """
        Helper function: evaluates the jacobian matrix

        Parameters
        ----------
        x : (x_dim, n_interpolations) ndarray of float
            periodic signals representing the coordinates where to interpolate

        Returns
        -------
        z : (z_dim, x_dim, n_interpolations) ndarray
            result array
        """
        z = eval_dz_dx(self.xc_t[:, :, np.newaxis], self.c[:, :, np.newaxis], self.d[:, :, np.newaxis],
                       self.t[:, :, np.newaxis], x[:, :, np.newaxis], self.kernel, self.max_dist)
        z = z[:, :, :, 0]
        return z

    def to_msgpack(self):
        """
        Create msgpack serializable dict representing an instance of this class
        ref: http://msgpack.org/

        Returns
        -------
        msgpack : dict
            msgpack representation

        """

        msgpack = {"kernel": self.kernel.to_msgpack(),
                   "max_dist": self.max_dist,
                   "xc_t_per": PyTPeriodicMatrix(self.xc_t, mode='dc', nopack=True).to_msgpack(),
                   "t_per": PyTPeriodicMatrix(self.t, mode='dc', nopack=True).to_msgpack(),
                   "c_per": PyTPeriodicMatrix(self.c, mode='dc', nopack=True).to_msgpack(),
                   "d_per": PyTPeriodicMatrix(self.d, mode='dc', nopack=True).to_msgpack(),
                   "guid": "{8c4e810f-3df3-4a00-8276-176fa3c9f000}"}

        return msgpack

    def write_msgpack(self, file):
        """
        Create msgpack file representing an instance of this class
        ref: http://msgpack.org/


        Parameters
        ----------
        file : String
            path to msgpack file
        """
        import msgpack
        data = self.to_msgpack()

        with open(file, "wb") as fh:
            msgpack.dump(data, fh)

    def toCBOR(self):
        """
        Create CBOR serializable dict representing an instance of this class
        ref: http://cbor.io/

        Returns
        -------
        cbor_obj : dict
                cbor representation

        """

        ''' create cbor serializable dict '''
        xc_t_per = PyTPeriodicMatrix(self.xc_t, mode='dc', nopack=True)
        t_per = PyTPeriodicMatrix(self.t, mode='dc', nopack=True)
        c_per = PyTPeriodicMatrix(self.c, mode='dc', nopack=True)
        d_per = PyTPeriodicMatrix(self.d, mode='dc', nopack=True)

        cbor_obj = {'type': '__periodicRBF__',
                    'xi_per': xc_t_per.toCBOR(),
                    't_per': t_per.toCBOR(),
                    'c_per': c_per.toCBOR(),
                    'd_per': d_per.toCBOR(),
                    'kernel': Serializable.toCBOR(self.kernel.id),
                    'params': Serializable.toCBOR(self.kernel.params),
                    'max_dist': Serializable.toCBOR(self.max_dist)
                    }

        return cbor_obj

    # def plotMinRadiusNDPLY(self, n_xsamples=None):
    #
    #     import plotly.graph_objs as go
    #     from plotly import tools
    #     xc = np.linalg.inv(self.t) @ self.xc_t
    #
    #     fig = tools.make_subplots(rows=self.xdim - 1, cols=self.xdim - 1)
    #
    #     x_grid = range(1, self.xdim + 1)
    #     grid = np.meshgrid(x_grid, x_grid)
    #     table_x = np.tril(grid[0], k=-1)
    #
    #     index = 1
    #     for row in range(self.xdim):
    #         for col in range(self.xdim):
    #             if table_x[row, col] != 0:
    #                 x = xc[row, :]
    #                 y = xc[col, :]
    #                 X, Y = np.meshgrid(x, y)
    #
    #                 x_array, r_array = self.calcMinRadiusND(n_xsamples)
    #
    #                 xi = x_array[row, :]
    #                 yi = x_array[col, :]
    #                 zi = r_array[0, :]
    #                 # Xi, Yi = np.meshgrid(xi, yi)
    #
    #                 contour = go.Contour(z=zi, x=xi, y=yi,
    #                                      colorscale='Jet',
    #                                      # colorscale='Viridis',
    #                                      # legendgroup=title,
    #                                      showscale=False,
    #                                      # zmin=0, zmax=self.max_dist,
    #                                      contours=dict(
    #                                          coloring='heatmap',
    #                                          showlabels=True,
    #                                          # labelfont=dict(
    #                                          #     family='Raleway',
    #                                          #     size=12,
    #                                          #     color='white',
    #                                          # )
    #                                      )
    #                                      )
    #
    #                 centers = go.Scatter(x=X.flatten(), y=Y.flatten(), mode='markers',
    #                                      showlegend=False,
    #                                      # opacity=0.8,
    #                                      marker=dict(
    #                                          color='white',
    #                                          size=15,
    #                                          showscale=False, line=dict(color='grey', width=3)
    #                                      )
    #                                      )
    #                 # centers = go.Scatter(x=Xi.flatten(), y=Yi.flatten(), mode='markers',
    #                 #                      showlegend=False,
    #                 #                      # opacity=0.8,
    #                 #                      marker=dict(
    #                 #                          color='white',
    #                 #                          showscale=False, line=dict(color='grey', width=3)
    #                 #                      )
    #                 #                      )
    #
    #                 # data = [contour, centers]
    #                 # index = (row-1) * (self.xdim - 1) + col
    #                 # index = col * (self.xdim - 1) + (row - 1)
    #                 fig.add_trace(contour, row, col + 1)
    #                 fig.add_trace(centers, row, col + 1)
    #                 # xaxis = 'xaxis{}'.format(index) if index != 1 else 'xaxis'
    #                 # yaxis = 'yaxis{}'.format(index) if index != 1 else 'yaxis'
    #                 # fig['layout'][xaxis].update(title='Input[{}]'.format(row))
    #                 # fig['layout'][yaxis].update(title='Input[{}]'.format(col))
    #             index += 1
    #
    #     return fig

    def calcMinRadius2D(self, xaxis=0, yaxis=1, nx_samples=(10, 10)):
        xc = np.linalg.inv(self.t) @ self.xc_t

        x = xc[xaxis, :]
        y = xc[yaxis, :]

        nx, ny = nx_samples

        xi = np.linspace(np.min(x), np.max(x), nx)
        yi = np.linspace(np.min(y), np.max(y), ny)
        ri = np.zeros((nx, ny))

        for row in range(nx):
            for col in range(ny):
                x = np.array([[xi[row]], [yi[col]]])
                ri[row, col] = self.evalRadius(x)

        return xi, yi, ri

    def calcMinRadius3D(self, z_values, xaxis=0, yaxis=1, nx_samples=(10, 10)):
        xc = np.linalg.inv(self.t) @ self.xc_t

        x = xc[xaxis, :]
        y = xc[yaxis, :]

        nx, ny = nx_samples
        nz = z_values.size

        xi = np.linspace(np.min(x), np.max(x), nx)
        yi = np.linspace(np.min(y), np.max(y), ny)
        ri = np.zeros((nx, ny, nz))

        for index, zi in enumerate(z_values):
            for row in range(nx):
                for col in range(ny):
                    x_in = np.array([[xi[row]], [yi[col]], [zi]])
                    ri[row, col, index] = self.evalRadius(x_in)

        return xi, yi, ri

    def calcMinRadiusND(self, nx_samples=None):
        xc = np.linalg.inv(self.t) @ self.xc_t

        if nx_samples is None:
            nx_samples = [10] * self.xdim

        xmin = np.min(xc, axis=1)[:, np.newaxis]
        xmax = np.max(xc, axis=1)[:, np.newaxis]

        pp = (xmax - xmin) * 0.5
        xmin -= pp
        xmax += pp

        space = [np.linspace(xmin[index], xmax[index], nx_samples[index]) for index in range(xmin.size)]
        grid = np.meshgrid(*space)

        x_array = np.vstack(map(np.ravel, grid))
        n_loadpoints = x_array.shape[1]

        r_array = np.zeros((1, n_loadpoints))

        for index in range(n_loadpoints):
            r_array[0, index] = self.evalRadius(x_array[:, index, np.newaxis])

        return x_array, r_array

    def plotMinRadius2DPLY(self, xaxis=0, yaxis=1):
        import plotly.graph_objs as go

        xi, yi, ri = self.calcMinRadius2D(xaxis, yaxis)

        contour = go.Contour(z=ri, x=xi, y=yi, colorscale='Jet',
                             showscale=True,
                             zmin=0, zmax=self.max_dist,
                             contours=dict(
                                 coloring='heatmap',
                                 showlabels=True,
                             )
                             )

        fig = go.Figure(data=[contour])
        fig['layout']['xaxis'].update(title='Input[{}]'.format(xaxis))
        fig['layout']['yaxis'].update(title='Input[{}]'.format(yaxis))
        return fig

    def plotMinRadius3DPLY(self, z_values, xaxis=0, yaxis=1, nx_samples=(10, 10)):
        from plotly import tools
        import plotly.graph_objs as go

        nz = z_values.size

        xi, yi, ri = self.calcMinRadius3D(z_values, xaxis=xaxis, yaxis=yaxis, nx_samples=nx_samples)

        fig = tools.make_subplots(rows=nz, cols=1, subplot_titles=['z-axis @ {}'.format(z) for z in z_values])

        for index in range(nz):
            contour = go.Contour(z=ri[:, :, index], x=xi, y=yi, colorscale='Jet',
                                 showscale=True,
                                 zmin=0, zmax=self.max_dist,
                                 contours=dict(
                                     coloring='heatmap',
                                     showlabels=True,
                                 )
                                 )
            fig.add_trace(contour, index + 1, 1)
            l_xaxis = 'xaxis{}'.format(index + 1) if index != 0 else 'xaxis'
            l_yaxis = 'yaxis{}'.format(index + 1) if index != 0 else 'yaxis'
            fig['layout'][l_xaxis].update(title='Input[{}]'.format(xaxis))
            fig['layout'][l_yaxis].update(title='Input[{}]'.format(yaxis))

        return fig
