#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 944 $
$LastChangedDate:: 2019-02-20 16:44:30 +0100#$

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import numpy as np
from abc import ABCMeta, abstractmethod
import matplotlib.pyplot as plt
from matplotlib.gridspec import GridSpec


class AbstractAnalyticFunction(metaclass=ABCMeta):

    def __init__(self, n_x, n_y, n_phi):
        self.n_fncs = 4
        self.n_x = n_x
        self.n_y = n_y
        self.n_phi = n_phi
        self.phi = np.linspace(0, 2 * np.pi, self.n_phi, endpoint=False)

    def getXc(self):
        xc_samples = np.empty((2, self.n_x * self.n_y, self.n_phi), dtype=np.float)
        for col, y0 in enumerate(np.linspace(-2, 2, self.n_y)):
            for row, x0 in enumerate(np.linspace(-1, 1, self.n_x)):
                idx = col * self.n_x + row
                xc_samples[0, idx] = x0 * np.ones_like(self.phi)
                xc_samples[1, idx] = y0 * np.ones_like(self.phi)

        return xc_samples

    def getZc(self):
        xc_samples = self.getXc()

        '''zc_samples'''
        n_samples = xc_samples.shape[1]
        zc_samples = np.empty((self.n_fncs, n_samples, self.n_phi), dtype=np.float)
        '''dz/dx[0]'''
        for (idx_p, p) in enumerate(self.phi):
            for idx_center in range(0, n_samples):
                zc_samples[:, idx_center, idx_p] = self.getZiDiscrete(*xc_samples[:, idx_center, idx_p], p)

        return zc_samples

    def getZi(self, x, y, phi):
        """
        Definition der Testfunktion [f1, f2, f3] = fun1(x, y, phi)
        """

        zi_disc = self.getZiDiscrete(x, y, phi)

        samples = np.empty((self.n_fncs, 1, phi.size), dtype=np.float)
        for row in range(self.n_fncs):
            samples[row, 0] = zi_disc[row, :]
        return samples

    @abstractmethod
    def getZiDiscrete(self, x, y, phi):
        raise NotImplementedError

    @abstractmethod
    def get_dzi_dx(self, x, y, phi):
        raise NotImplementedError

    def plotComparison(self, x_interp, x_analytic, atol, rtol, x_periodic=None):

        nrows = x_interp.shape[0]
        ncols = x_interp.shape[1]
        fig = plt.figure(figsize=(6 * 2, 4 * 2))
        gs = GridSpec(nrows, ncols * 2, figure=fig)

        for row in range(nrows):
            for col in range(ncols):
                a = x_interp[row, col, :].flatten()
                b = x_analytic[row, col, :].flatten()

                error = self.calcError(a, b)
                threshold = atol + rtol * np.abs(b)

                ax1 = fig.add_subplot(gs[row, col])
                ax1.plot(self.phi, a, 'r-.', label='z_rbf[{},{}]'.format(row, col))
                ax1.plot(self.phi, b, 'g--', label='z_analytic[{},{}]'.format(row, col))
                if x_periodic is not None:
                    ax1.plot(self.phi, x_periodic[row, col, :], 'b:', label='periodic[{},{}]'.format(row, col))
                ax1.grid(True)
                ax1.legend(loc=1)

                ax2 = fig.add_subplot(gs[row, col + ncols])
                ax2.plot(self.phi, error, 'r-.', label='error[{},{}]'.format(row, col))
                ax2.plot(self.phi, threshold, 'g--', label='threshold[{},{}]'.format(row, col))
                ax2.grid(True)
                ax2.legend(loc=1)

                plt.tight_layout()

        plt.show()

    def calcError(self, a, b):
        return np.abs(a - b)


class AnalyticFunction1(AbstractAnalyticFunction):
    def getZiDiscrete(self, x, y, phi):
        """
        Definition der Testfunktion [f1, f2, f3] = fun1(x, y, phi)
        """

        f1 = 2. * x - 3. * y
        f2 = 2. * x - 3. * y + 0.5
        f3 = 2. * x - 3. * y - x * y ** 2
        f4 = 2. * x - 3. * y - x * y ** 2

        return np.array([f1, f2, f3, f4])

    def get_dzi_dx(self, x, y, phi):
        f11 = 2. * np.ones_like(x)
        f21 = 2. * np.ones_like(x)
        f31 = 2. - y ** 2
        f41 = 2. - y ** 2

        f12 = - 3. * np.ones_like(x)
        f22 = - 3. * np.ones_like(x)
        f32 = - 3. - 2 * x * y
        f42 = - 3. - 2 * x * y

        return np.array([[f11, f12], [f21, f22], [f31, f32], [f41, f42]])
