/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.docu;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.imageio.ImageIO;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.ClientAnchor;
import org.apache.poi.ss.usermodel.FormulaEvaluator;
import org.apache.poi.ss.usermodel.Name;
import org.apache.poi.ss.usermodel.Picture;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.AreaReference;
import org.apache.poi.ss.util.CellReference;
import org.apache.poi.util.IOUtils;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import at.lcm.bu21.general.dtypes.TData;
import at.lcm.bu21.general.dtypes.TNumeric;
import at.lcm.bu21.magopt.structure.DocFile;
import at.lcm.bu21.magopt.structure.Parameter;
import at.lcm.bu21.magopt.structure.Project;

/**
 * Fill excel-template with data of image from project
 * 
 * @author Werner Koppelsttter
 */
public class FillExcelReport {

	/** Factor between EMU - English Metric Units - and millimeters */
	private static final int EMU_PER_MM = 36000;

	/** Width of a column */
	public static final int COLUMN_WIDTH = 1023;

	/** Height of a row */
	public static final int ROW_HEIGHT = 255;

	/**
	 * Fill excel workbook with the data of project or docFile of project. <br>
	 * To insert data name of the named-cell is the parameter-identifier. <br>
	 * To inset a docfile-image the name of the named-cell is
	 * IMAGETAG.Identifer.DocFile
	 * 
	 * @param proj
	 *            Project with data
	 * @param template
	 *            Excel-template
	 * @param export
	 *            Export for filled Template
	 * @return Filled excel file
	 * @throws IOException
	 */
	public File fillExcel(Project proj, File template, File export)
			throws IOException {
		Workbook wb;
		FileInputStream in;
		FileOutputStream out;
		Cell cell;
		Name name;
		AreaReference aref;
		CellReference[] crefs;
		String tag;

		if (template.exists()) {
			// open file
			in = new FileInputStream(template);
			if (template.getName().toLowerCase().endsWith(".xlsx")) {
				wb = new XSSFWorkbook(in);
				if (!export.getName().toLowerCase().endsWith(".xlsx")) {
					export = new File(export.getParentFile(), export.getName()
							+ ".xlsx");
				}
			} else {
				wb = new HSSFWorkbook(in);
				if (!export.getName().toLowerCase().endsWith(".xls")) {
					export = new File(export.getParentFile(), export.getName()
							+ ".xls");
				}
			}
			in.close();

			for (int k = 0; k < wb.getNumberOfNames(); k++) {
				name = wb.getNameAt(k);
				tag = name.getNameName();
				// select cell
				aref = new AreaReference(name.getRefersToFormula());
				crefs = aref.getAllReferencedCells();
				cell = this.getCell(wb, crefs[0].getSheetName(),
						crefs[0].getRow(), crefs[0].getCol());

				if (tag.startsWith("IMAGETAG.")) {
					// insert image
					try {
						this.insertImage(cell, tag, proj);
					} catch (Exception ex) {
						proj.error("Error writing image (" + tag
								+ ") to excel file");
					}
				} else {
					// write value of parameter
					this.writeValue(cell, tag, proj);
				}
			}

			// evaluate
			this.evaluate(wb);

			// save
			out = new FileOutputStream(export);
			wb.write(out);
			out.close();
		} else {
			export = null;
		}

		return export;
	}

	/**
	 * Add image to sheet on the position of the reference cell
	 * 
	 * @param cell
	 *            Reference cell for the upper and left edge and the
	 *            image-properties
	 * @param tag
	 *            Tag with the name of Parameter and name of docFile
	 * @param proj
	 *            Project
	 * @throws Exception
	 */
	private void insertImage(Cell cell, String tag, Project proj)
			throws Exception {
		String paramName, docName, imageProperties;
		Parameter param;
		DocFile doc;
		String[] props;
		String str;
		double width = 0.0, height = 0.0;

		// split names
		paramName = tag.substring(tag.indexOf(".") + 1);
		if (paramName.contains(".")) {
			docName = paramName.substring(paramName.lastIndexOf(".") + 1);
			paramName = paramName.substring(0, paramName.lastIndexOf("."));
			param = proj.getObject(paramName);
			if (param != null) {
				doc = param.getDocFile(docName);
				if (doc != null) {
					// image properties
					imageProperties = cell.getStringCellValue();
					if (imageProperties != null) {
						props = imageProperties.split(",");
						for (int k = 0; k < props.length; k += 2) {
							str = props[k].trim();
							if (str.compareToIgnoreCase("Width") == 0) {
								width = Double.parseDouble(props[k + 1]) * 10.0;
							} else if (str.compareToIgnoreCase("Height") == 0) {
								height = Double.parseDouble(props[k + 1]) * 10.0;
							} else if (str.compareToIgnoreCase("CutTop") == 0) {
							} else if (str.compareToIgnoreCase("CutBottom") == 0) {
							} else if (str.compareToIgnoreCase("CutLeft") == 0) {
							} else if (str.compareToIgnoreCase("CutRight") == 0) {
							} else {
								throw new Exception("Undefined image property");
							}
						}
					}
					this.addImageToSheet(cell,
							new File(proj.getFilesDirectory(), doc.getFile()),
							width, height);
				}
			}
		}

	}

	/**
	 * Write data from parameter to cell
	 * 
	 * @param cell
	 *            Referenced cell
	 * @param paramName
	 *            Name of parameter
	 * @param proj
	 *            Project
	 */
	private void writeValue(Cell cell, String paramName, Project proj) {
		Parameter param;
		TData data;

		param = proj.getObject(paramName);
		if (param != null) {

			// Value
			data = param.getData();
			if ((data instanceof TNumeric) && ((TNumeric) data).isScalar()) {
				cell.setCellValue(((TNumeric) data).getReal(0, 0));
			} else {
				cell.setCellValue(data.toString());
			}
		}
	}

	/**
	 * Get the cell of a spreadsheet
	 * 
	 * @param wb
	 *            Workbook
	 * @param sheetName
	 *            Name of spreadsheet
	 * @param rowIndex
	 *            Row index of cell
	 * @param columnIndex
	 *            Column index of cell
	 * @return Selected cell
	 */
	private Cell getCell(Workbook wb, String sheetName, int rowIndex,
			int columnIndex) {
		Sheet sheet;
		Row row;
		Cell cell;

		// get sheet
		sheet = wb.getSheet(sheetName);
		if (sheet == null) {
			wb.createSheet(sheetName);
		}

		// get or create row
		row = sheet.getRow(rowIndex);
		if (row == null) {
			row = sheet.createRow(rowIndex);
		}

		// get or create cell
		cell = row.getCell(columnIndex);
		if (cell == null) {
			cell = row.createCell(columnIndex);
		}

		return cell;
	}

	/**
	 * Evaluate all formulas of woorkbook
	 * 
	 * @param wb
	 *            Workbook
	 */
	private void evaluate(Workbook wb) {
		Sheet sheet;
		FormulaEvaluator evaluator;

		evaluator = wb.getCreationHelper().createFormulaEvaluator();
		for (int s = 0; s < wb.getNumberOfSheets(); s++) {
			sheet = wb.getSheetAt(s);
			for (Row r : sheet) {
				for (Cell c : r) {
					if (c.getCellType() == Cell.CELL_TYPE_FORMULA) {
						evaluator.evaluateFormulaCell(c);
					}
				}
			}
		}

	}

	/**
	 * Add image to sheet on the position of the reference cell and with the
	 * width and height
	 * 
	 * @param cell
	 *            Reference cell for the upper and left edge
	 * @param image
	 *            File of image to insert
	 * @param width
	 *            Width in millimeter
	 * @param height
	 *            Height in millimeter
	 * @throws Exception
	 */
	private void addImageToSheet(Cell cell, File image, double width,
			double height) throws Exception {
		ClientAnchor anchor = null;
		int imageType = 0;
		int index;
		Picture pict;
		byte[] imageData;
		FileInputStream in;

		// identify type
		String imageName = image.getName().toLowerCase();
		if (imageName.endsWith(".jpg") || imageName.endsWith(".jpeg")) {
			imageType = Workbook.PICTURE_TYPE_JPEG;
		} else if (imageName.endsWith(".png")) {
			imageType = Workbook.PICTURE_TYPE_PNG;
			// } else if (imageName.endsWith(".emf")) {
			// imageType = Workbook.PICTURE_TYPE_EMF;
			// } else if (imageName.endsWith(".eps") && (cell instanceof
			// XSSFCell)) {
			// imageType = XSSFWorkbook.PICTURE_TYPE_EPS;
		} else {
			throw new Exception("Invalid Image file: " + image.getName());
		}

		// calculate size and position of image
		anchor = cell.getSheet().getWorkbook().getCreationHelper()
				.createClientAnchor();
		anchor.setAnchorType(ClientAnchor.MOVE_AND_RESIZE);

		if ((imageType == Workbook.PICTURE_TYPE_JPEG)
				|| (imageType == Workbook.PICTURE_TYPE_PNG)) {
			if ((width == 0.0) && (height == 0.0)) {
				anchor.setCol1(cell.getColumnIndex());
				anchor.setRow1(cell.getRowIndex());
				// } else if ((width != 0.0) && (height != 0.0)) {
				// this.setImageWidth(cell, anchor, width);
				// this.setImageHeight(cell, anchor, height);
			} else {
				if (width == 0.0) {
					BufferedImage im = ImageIO.read(image);
					width = (height * im.getWidth()) / im.getHeight();
					// ByteArrayOutputStream baos = new ByteArrayOutputStream();
					// ImageIO.write(im, "PNG", baos);
					// imagetData = baos.toByteArray();
				} else if (height == 0.0) {
					BufferedImage im = ImageIO.read(image);
					height = (width * im.getHeight()) / im.getWidth();
					// ByteArrayOutputStream baos = new ByteArrayOutputStream();
					// ImageIO.write(im, "PNG", baos);
					// imagetData = baos.toByteArray();
				}
				this.setImageWidth(cell, anchor, width);
				this.setImageHeight(cell, anchor, height);
			}
		} else {
			// other image format
			if ((width == 0.0) || (height == 0.0)) {
				// set to autosize
				width = 0.0;
				height = 0.0;
				anchor.setCol1(cell.getColumnIndex());
				anchor.setRow1(cell.getRowIndex());
			} else {
				this.setImageWidth(cell, anchor, width);
				this.setImageHeight(cell, anchor, height);
			}
		}

		// insert image
		in = new FileInputStream(image);
		imageData = IOUtils.toByteArray(in);
		in.close();
		index = cell.getSheet().getWorkbook().addPicture(imageData, imageType);
		pict = cell.getSheet().createDrawingPatriarch()
				.createPicture(anchor, index);
		if ((width == 0.0) && (height == 0.0)) {
			pict.resize();
		}

	}

	/**
	 * Set image width of anchor (cells and inset)
	 * 
	 * @param cell
	 *            Reference cell for left edge
	 * @param anchor
	 *            Anchor of image
	 * @param width
	 *            Width in millimeter
	 */
	private void setImageWidth(Cell cell, ClientAnchor anchor, double width) {
		double colWidth;
		double overlap = width;
		int toColumn = cell.getColumnIndex();

		// get column width
		colWidth = this.convertWidth2MM(cell.getSheet()
				.getColumnWidth(toColumn));

		// calculate number of columns
		while (overlap > colWidth) {
			overlap -= colWidth;
			if (overlap < 0.0) {
				overlap = 0.0;
			}
			toColumn++;
			colWidth = this.convertWidth2MM(cell.getSheet().getColumnWidth(
					toColumn));
		}

		// define anchor
		anchor.setCol1(cell.getColumnIndex());
		anchor.setCol2(toColumn);
		anchor.setDx1(0);
		if (cell instanceof XSSFCell) {
			anchor.setDx2((int) overlap * FillExcelReport.EMU_PER_MM);
		} else {
			anchor.setDx2((int) (overlap * (FillExcelReport.COLUMN_WIDTH / colWidth)));
		}

	}

	/**
	 * Set image height of anchor (cells and inset)
	 * 
	 * @param cell
	 *            Reference cell for upper edge
	 * @param anchor
	 *            Anchor of image
	 * @param height
	 *            Height in millimeter
	 */
	private void setImageHeight(Cell cell, ClientAnchor anchor, double height) {
		double rowHeight = 0.0;
		double overlap = height;
		int toRow = cell.getRowIndex();

		// get row height
		rowHeight = this.convertHeight2MM(cell.getSheet().getRow(toRow)
				.getHeight());

		// calculate number of rows
		while (overlap > rowHeight) {
			overlap -= rowHeight;
			if (overlap < 0.0) {
				overlap = 0.0;
			}
			toRow++;
			rowHeight = this.convertHeight2MM(cell.getSheet().getRow(toRow)
					.getHeight());
		}

		// define anchor
		anchor.setRow1(cell.getRowIndex());
		anchor.setRow2(toRow);
		anchor.setDy1(0);
		if (cell instanceof XSSFCell) {
			anchor.setDy2((int) (overlap * FillExcelReport.EMU_PER_MM));
		} else {
			anchor.setDy2((int) (overlap * (FillExcelReport.ROW_HEIGHT / rowHeight)));
		}
	}

	/**
	 * Convert height of a row to millimeter
	 * 
	 * @param height
	 *            Height of row
	 * @return Height in millimeter
	 */
	private double convertHeight2MM(int height) {
		// height / (height per pixel) * (mm per pixel)
		return (height / 20.0) * (25.4 / 72);
	}

	/**
	 * Convert width of column to millimeter
	 * 
	 * @param width
	 *            Width of column
	 * @return Width in millimeter
	 */
	private double convertWidth2MM(int width) {
		// width / (column width) * (unit offset) / (pixel per mm)
		return Math.round(((width) / 256.0) * 7.0) / 3.78;
	}
}
