/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.TreeSet;

import at.lcm.bu21.magopt.structure.Parameter.ParameterPath;

/**
 * DependMatrix of Project
 * 
 * @author Werner Koppelsttter
 */
public class DependMatrix implements Serializable {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Project to matrix */
	private Project project;

	/** List of functions handles */
	private Parameter[] handles;

	/** Matrix for functions depend */
	private boolean[][] matrix;

	/** List of rest unlinked functions */
	private LinkedList<Integer> link;

	/**
	 * Create depend-matrix for given Project
	 * 
	 * @param project
	 *            Project for dependencies
	 */
	public DependMatrix(Project project) {
		DependItem[] imports, exports;

		this.project = project;
		this.handles = project.getParameters(Function.class);
		this.link = new LinkedList<Integer>();
		this.matrix = new boolean[this.handles.length][this.handles.length];

		for (int row = 0; row < this.handles.length; row++) {
			for (int column = 0; column < this.handles.length; column++) {
				if (row != column) {
					imports = ((Function) this.handles[column]).getDepend()
							.getImports();
					for (int k = 0; k < imports.length; k++) {
						// check direct dependencies
						this.matrix[row][column] |= (this.handles[row] == imports[k]
								.getHandle());
						// check for indirect dependencies
						exports = ((Function) this.handles[row]).getDepend()
								.getExports();
						for (int l = 0; l < exports.length; l++) {
							this.matrix[row][column] |= (imports[k].getHandle() != null)
									&& (imports[k].getHandle() == exports[l]
											.getHandle());
						}
					}
				}
			}
		}
		this.reset();
	}

	/**
	 * Reset list to initial size
	 */
	public void reset() {
		this.reset(null);
	}

	/**
	 * Reset list to initial size with aims
	 * 
	 * @param aims
	 *            Objective for update
	 */
	public void reset(String[] aims) {
		ParameterPath path;
		Parameter p;
		TreeSet<Integer> set;
		int index;

		// this.reset();
		this.link.clear();

		if ((aims == null) || (aims.length == 0)) {
			for (int row = 0; row < this.handles.length; row++) {
				this.link.add(row);
			}
		} else {
			set = new TreeSet<Integer>();
			for (String aim : aims) {
				path = this.project.getParamByUniqueIdentifier(aim);
				if (path != null) {
					p = path.getParameter();
					if (path.getParameter() != null) {
						for (Parameter param : p.getSetBy()) {
							index = 0;
							while ((index < this.handles.length)
									&& !this.handles[index].equals(param)) {
								index++;
							}
							if (index < this.handles.length) {
								this.addLink(index, set);
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Add recursive links for depend functions
	 * 
	 * @param index
	 *            Index to add
	 * @param set
	 *            Set of added index
	 */
	private void addLink(int index, TreeSet<Integer> set) {
		if (!set.contains(index)) {
			set.add(index);
			this.link.add(index);
			for (int row = 0; row < this.handles.length; row++) {
				if (this.matrix[row][index]) {
					this.addLink(row, set);
				}
			}
		}
	}

	/**
	 * Size of left linked functions
	 * 
	 * @return Size of linked functions
	 */
	public int size() {
		return this.link.size();
	}

	/**
	 * Check if functions are left
	 * 
	 * @return Linked functions are empty
	 */
	public boolean isEmpty() {
		return this.link.size() == 0;
	}

	/**
	 * Remove function from rest unlinked list
	 * 
	 * @param idx
	 *            Index of rest function to remove
	 */
	public void remove(int idx) {
		this.link.remove(idx);
	}

	/**
	 * Remove function from rest unlinked list
	 * 
	 * @param param
	 */
	public void remove(Parameter param) {
		int idx = 0;

		while ((idx < this.link.size())
				&& !this.handles[this.link.get(idx)].equals(param)) {
			idx++;
		}
		if (idx < this.link.size()) {
			this.link.remove(idx);
		}
	}

	/**
	 * Check if function depends to other functions
	 * 
	 * @param idx
	 *            Index of rest function
	 * @return Depends to other functions
	 */
	public boolean depends(int idx) {
		boolean depend = false;

		for (int row : this.link) {
			depend |= this.matrix[row][this.link.get(idx)];
		}

		return depend;
	}

	/**
	 * Get function of index
	 * 
	 * @param idx
	 *            Index of rest functions
	 * 
	 * @return Selected Function
	 */
	public Parameter getFunction(int idx) {
		return this.handles[this.link.get(idx)];
	}

	/**
	 * Get list of non-depend functions
	 * 
	 * @return List of functions to run
	 */
	public Parameter[] getNextFunctions() {
		LinkedList<Parameter> nexts;

		nexts = new LinkedList<Parameter>();

		for (int k = 0; k < this.link.size(); k++) {
			if (!this.depends(k)) {
				nexts.add(this.handles[this.link.get(k)]);
			}
		}

		return nexts.toArray(new Parameter[0]);
	}

	@Override
	public String toString() {
		StringBuffer str;

		str = new StringBuffer();

		for (int k = 0; k < this.link.size(); k++) {
			str.append(String.format("%3d", k) + ": "
					+ this.handles[this.link.get(k)].getName() + "\n");
		}
		str.append("\n");

		str.append("    ");
		for (int k = 0; k < this.link.size(); k++) {
			str.append(String.format("%2d ", k));
		}
		str.append("\n");

		for (int row = 0; row < this.link.size(); row++) {
			str.append(String.format("%3d  ", row));

			for (int col = 0; col < this.link.size(); col++) {
				if (row == col) {
					str.append("*  ");
				} else if (this.matrix[this.link.get(row)][this.link.get(col)]) {
					str.append("X  ");
				} else {
					str.append("-  ");
				}
			}
			str.append("\n");

		}

		return str.toString();

	}
}
