/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9752 $
 * $Date: 2020-04-11 00:01:35 +0200 (Sat, 11 Apr 2020) $
 */
package at.lcm.bu21.magopt.structure;

import java.rmi.RemoteException;

import at.lcm.bu21.general.dtypes.TData;
import at.lcm.bu21.general.dtypes.unit.ConversionException;

/**
 * Interface of field
 * 
 * @author Werner Koppelsttter
 */
public interface IParamField extends IParameter {

	/**
	 * Get Tex-string of Parameter
	 * 
	 * @return tex-string
	 * @throws RemoteException
	 */
	public String getTexString() throws RemoteException;

	/**
	 * Set Tex-string of Parameter
	 * 
	 * @param texString
	 *            Tex-string
	 * @throws RemoteException
	 */
	public void setTexString(String texString) throws RemoteException;

	/**
	 * Get unit of Parameter
	 * 
	 * @return unit
	 * @throws RemoteException
	 */
	public String getUnit() throws RemoteException;

	/**
	 * Set unit of Parameter. Default behavior is to convert values.
	 * 
	 * @param unit
	 *            unit
	 * @throws RemoteException
	 * @throws Exception
	 */
	public void setUnit(String unit) throws RemoteException, Exception;

	/**
	 * Set unit of Parameter
	 * 
	 * @param unit
	 *            unit
	 * @param convert
	 *            Convert to new unit
	 * @throws RemoteException
	 * @throws Exception
	 */
	public void setUnit(String unit, boolean convert) throws RemoteException, Exception;

	/**
	 * Get type of Parameter
	 * 
	 * @return type
	 * @throws RemoteException
	 */
	public String getType() throws RemoteException;

	/**
	 * Set type of Paramter
	 * 
	 * @param type
	 *            type
	 * @throws RemoteException
	 */
	public void setType(String type) throws RemoteException;

	/**
	 * Get the linked database identifier
	 * 
	 * @return Database id
	 * @throws RemoteException
	 */
	public long getDbId() throws RemoteException;

	/**
	 * Clear the database link
	 * 
	 * @throws RemoteException
	 */
	public void clearDbId() throws RemoteException;

	/**
	 * Get lock status
	 * 
	 * @return Lock status
	 * @throws RemoteException
	 */
	public boolean getLock() throws RemoteException;

	/**
	 * Convert to class ParamField
	 * 
	 * @return converted object
	 * @throws RemoteException
	 */
	public ParamField convertToParamField() throws RemoteException;

	/**
	 * Convert to class ParamComboBox
	 * 
	 * @return converted object
	 * @throws RemoteException
	 */
	public ParamField convertToParamComboBox() throws RemoteException;

	/**
	 * Convert to class ParamFormula
	 * 
	 * @return ParamFormula representation of ParamField
	 * @throws RemoteException
	 */
	public ParamFormula convertToParamFormula() throws RemoteException;

	/**
	 * Convert to class ParamLinkedField
	 * 
	 * @return ParamLinkedField representation of ParamField
	 * @throws RemoteException
	 */
	public ParamLinkedField convertToParamLinkedField() throws RemoteException;

	/**
	 * Get data of Parameter
	 * 
	 * @param unit
	 *            Unit of returned data
	 * @return Data
	 * @throws RemoteException
	 * @throws ConversionException
	 */
	public TData getData(String unit) throws RemoteException, ConversionException;

	/**
	 * Get data of Parameter
	 * 
	 * @param index
	 *            Index of dataset
	 * @param unit
	 *            Unit of returned data
	 * @return Data
	 * @throws RemoteException
	 * @throws ConversionException
	 */
	public TData getData(int index, String unit) throws RemoteException, ConversionException;

	/**
	 * Set data of parameter
	 * 
	 * @param data
	 *            Data
	 * @param unit
	 *            Unit of given data
	 * @throws Exception
	 * @throws RemoteException
	 * @throws ConversionException
	 */
	public void setData(TData data, String unit) throws RemoteException, ConversionException, Exception;

	@Override
	public IParamField getClientImpl() throws RemoteException;
}
