/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.event;

import java.rmi.RemoteException;
import java.util.LinkedList;

import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

import at.lcm.bu21.general.utils.Clone;
import at.lcm.bu21.general.utils.CloneException;
import at.lcm.bu21.magopt.structure.client.CEventLog;
import at.lcm.bu21.magopt.structure.rmi.REventLog;

/**
 * Event-logging class
 * 
 * @author Werner Koppelsttter
 */
public class EventLog implements IEventLog {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Maximal buffer size */
	int maxSize = 1;

	/** Event buffer */
	LinkedList<Event> buffer = new LinkedList<Event>();

	/** List of listeners */
	private transient LinkedList<EventLogListener> listeners = null;

	/**
	 * Create a default event-log
	 */
	public EventLog() {
		this(20);
	}

	/**
	 * Create event-log with given size
	 * 
	 * @param size
	 *            Maximal size of buffer
	 */
	public EventLog(int size) {
		if (size > 0) {
			this.maxSize = size;
		}
	}

	/**
	 * Create a deep clone of given data-structure
	 * 
	 * @return Deep clone of structure
	 * @throws CloneException
	 */
	public EventLog deepClone() throws CloneException {
		return (EventLog) Clone.deepClone(this);
	}

	/**
	 * Get the event-buffer of the event-log
	 * 
	 * @return Event-array
	 */
	public Event[] getBuffer() {
		synchronized (this.buffer) {
			return this.buffer.toArray(new Event[0]);
		}
	}

	@Override
	public Event[] getBuffer(boolean info, boolean warning, boolean error) {
		LinkedList<Event> list;

		synchronized (this.buffer) {
			list = new LinkedList<Event>();
			for (Event e : this.buffer) {
				if ((info && (e.getType() == EventType.INFO))
						|| (warning && (e.getType() == EventType.WARNING))
						|| (error && (e.getType() == EventType.ERROR))) {
					list.add(e);
				}

			}
		}
		return list.toArray(new Event[0]);
	}

	@Override
	public int getMaxSize() {
		return this.maxSize;
	}

	@Override
	public void setMaxSize(int size) {
		if (size > 0) {
			this.maxSize = size;
		}
		synchronized (this.buffer) {
			while (this.buffer.size() > this.maxSize) {
				this.buffer.remove(0);
			}
		}
	}

	@Override
	public void add(Event evObj) {
		int index;
		LinkedList<Event> removed = new LinkedList<Event>();

		synchronized (this.buffer) {
			index = this.buffer.size();
			while ((index > 0)
					&& (this.buffer.get(index - 1).getDate()
							.compareTo(evObj.getDate()) > 0)) {
				index--;
			}
			this.buffer.add(index, evObj);
			while (this.buffer.size() > this.maxSize) {
				removed.add(this.buffer.remove(0));
			}
		}

		this.fireEventAdded(this, new Event[] { evObj }, index, index);
		if (!removed.isEmpty()) {
			this.fireEventRemoved(this, removed.toArray(new Event[0]), 0,
					removed.size() - 1);
		}
	}

	@Override
	public void clear() {
		Event[] events;

		synchronized (this.buffer) {
			events = this.buffer.toArray(new Event[0]);
			this.buffer.clear();
		}
		if (events.length > 0) {
			this.fireEventRemoved(this, events, 0, events.length - 1);
		}
	}

	@Override
	public void info(String identifier, String message,
			StackTraceElement[] stack) {
		this.add(new Event(EventType.INFO, identifier, message, stack));
	}

	@Override
	public void warning(String identifier, String message,
			StackTraceElement[] stack) {
		this.add(new Event(EventType.WARNING, identifier, message, stack));
	}

	@Override
	public void error(String identifier, String message,
			StackTraceElement[] stack) {
		this.add(new Event(EventType.ERROR, identifier, message, stack));
	}

	@Override
	public String toString() {
		StringBuffer str = new StringBuffer();

		str.append("EventBuffer:\n");
		str.append("  Size: " + this.buffer.size() + "/" + this.maxSize);
		str.append("\n  Events:\n");
		synchronized (this.buffer) {
			for (int i = 0; i < this.buffer.size(); i++) {
				str.append("    " + this.buffer.get(i) + "\n");
			}
		}

		return str.toString();
	}

	@Override
	public String toFullString() {
		StringBuffer str = new StringBuffer();

		str.append("EventBuffer:\n");
		str.append("  Size: " + this.buffer.size() + "/" + this.maxSize);
		str.append("\n  Events:\n");
		synchronized (this.buffer) {
			for (int i = 0; i < this.buffer.size(); i++) {
				str.append("    " + this.buffer.get(i).toFullString() + "\n");
			}
		}

		return str.toString();
	}

	/**
	 * Read object from xml-file
	 * 
	 * @param xmlReader
	 *            XMLReader to read xml-file
	 * @throws Exception
	 */
	public void xml2obj(XMLStreamReader xmlReader) throws Exception {
		String element;
		Event obj;

		while (xmlReader.getEventType() == XMLStreamConstants.START_ELEMENT) {
			element = xmlReader.getLocalName();
			if (element.equals("BufferSize")) {
				xmlReader.next();
				if (xmlReader.getEventType() != XMLStreamConstants.END_ELEMENT) {
					this.maxSize = Integer.parseInt(xmlReader.getText());
					xmlReader.next();
				}
				xmlReader.next();
			} else if (element.equals("EventBuffer")) {
				xmlReader.next();
				while (xmlReader.getEventType() == XMLStreamConstants.START_ELEMENT) {
					obj = new Event();
					obj.xml2obj(xmlReader);
					this.add(obj);
				}
				xmlReader.next();
			} else {
				throw new Exception("Unrecognized property: "
						+ xmlReader.getLocalName());
			}
		}
	}

	/**
	 * Write object to xml-file
	 * 
	 * @param xmlWriter
	 *            XMLWriter to write xml-file
	 * @throws Exception
	 */
	public void obj2xml(XMLStreamWriter xmlWriter) throws Exception {

		// BufferSize
		xmlWriter.writeStartElement("BufferSize");
		xmlWriter.writeCharacters(Integer.toString(this.maxSize));
		xmlWriter.writeEndElement();

		// EventBuffer
		xmlWriter.writeStartElement("EventBuffer");
		synchronized (this.buffer) {
			for (int i = 0; i < this.buffer.size(); i++) {
				this.buffer.get(i).obj2xml(xmlWriter);
			}
		}
		xmlWriter.writeEndElement();
	}

	/**
	 * Get list of listeners
	 * 
	 * @return <code>EventLogListener</code> list
	 */
	private LinkedList<EventLogListener> getEventLogListenersList() {
		if (this.listeners == null) {
			this.listeners = new LinkedList<EventLogListener>();
		}
		return this.listeners;
	}

	/**
	 * Add event-log listener
	 * 
	 * @param listener
	 *            Listener to add
	 */
	public void addEventLogListener(EventLogListener listener) {
		LinkedList<EventLogListener> list;

		list = this.getEventLogListenersList();
		synchronized (list) {
			if (!list.contains(listener)) {
				list.add(listener);
			}
		}
	}

	/**
	 * Remove event-log listener
	 * 
	 * @param listener
	 *            Listener to remove
	 */
	public void removeEventLogListener(EventLogListener listener) {
		LinkedList<EventLogListener> list;

		list = this.getEventLogListenersList();
		synchronized (list) {
			list.remove(listener);
		}
	}

	/**
	 * Get list of event-log listener
	 * 
	 * @return List of listener
	 */
	public EventLogListener[] getEventLogListeners() {
		LinkedList<EventLogListener> list;

		list = this.getEventLogListenersList();
		synchronized (list) {
			return list.toArray(new EventLogListener[0]);
		}
	}

	/**
	 * Call to fire the event
	 * 
	 * @param event
	 *            Event that occurs
	 */
	private void fireEvent(EventLogEvent event) {
		LinkedList<EventLogListener> list;

		list = this.getEventLogListenersList();
		synchronized (list) {
			for (EventLogListener listener : list) {
				listener.eventLogChanged(event);
			}
		}
	}

	/**
	 * Call after one or more event of <code>EventLog</code> has been changed
	 * 
	 * @param source
	 *            The <code>EventLog</code> that changed
	 * @param events
	 *            List of changed events
	 * @param from
	 *            Lower index of range
	 * @param to
	 *            Upper index of range
	 */
	protected void fireEventChanged(Object source, Event[] events, int from,
			int to) {
		this.fireEvent(new EventLogEvent(this,
				EventLogEvent.EventType.EVENT_CHANGED, events, from, to));
	}

	/**
	 * Call after one or more event has been added to <code>EventLog</code>
	 * 
	 * @param source
	 *            The <code>EventLog</code> that changed
	 * @param events
	 *            List of added events
	 * @param from
	 *            Lower index of range
	 * @param to
	 *            Upper index of range
	 */
	protected void fireEventAdded(Object source, Event[] events, int from,
			int to) {
		this.fireEvent(new EventLogEvent(this,
				EventLogEvent.EventType.EVENT_ADDED, events, from, to));
	}

	/**
	 * Call after one or more event has been removed from <code>EventLog</code>
	 * 
	 * @param source
	 *            The <code>EventLog</code> that changed
	 * @param events
	 *            List of removed events
	 * @param from
	 *            Lower index of range
	 * @param to
	 *            Upper index of range
	 */
	protected void fireEventRemoved(Object source, Event[] events, int from,
			int to) {
		this.fireEvent(new EventLogEvent(this,
				EventLogEvent.EventType.EVENT_REMOVED, events, from, to));
	}

	@Override
	public String toStringI() throws RemoteException {
		return this.toString();
	}

	/**
	 * Get RMI-adapter of event-log
	 * 
	 * @return RMI-adapter
	 * @throws RemoteException
	 */
	public IEventLog getRMIImpl() throws RemoteException {
		return new REventLog(this);
	}

	@Override
	public IEventLog getClientImpl() {
		return new CEventLog(this);
	}
}
