/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.extra;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

import org.xml.sax.SAXException;

import at.lcm.bu21.general.dtypes.TBoolean;
import at.lcm.bu21.general.dtypes.TData;
import at.lcm.bu21.general.dtypes.TDataException;
import at.lcm.bu21.general.dtypes.TNumeric;
import at.lcm.bu21.magopt.structure.IParameter;
import at.lcm.bu21.magopt.structure.IProject;
import at.lcm.bu21.magopt.structure.formula.Parser;
import at.lcm.bu21.magopt.structure.formula.ParserException;
import at.lcm.bu21.magopt.structure.xsd.XMLValidator;

/**
 * Create new datasets (project variants)
 *
 * @author Werner Koppelsttter
 */
public class Variants {

	/** Based project for variants */
	private IProject project;

	/** List of unused parameters */
	private List<IParameter> unused;

	/** List of used parameters */
	private List<IParameter> used;

	/** List of range-strings for used parameters */
	private List<String> range;

	/** Validity string */
	private String validity;

	/** List of used values of all combination */
	List<double[]> combination;

	/** List flag if combination is valid */
	private List<Boolean> valid;

	/** List flag if combination is selected */
	private List<Boolean> selected;

	/**
	 * Create Variants for project
	 * 
	 * @param project
	 *            Based project
	 * @throws RemoteException
	 */
	public Variants(IProject project) throws RemoteException {

		this.project = project;
		this.unused = new LinkedList<IParameter>();
		this.used = new LinkedList<IParameter>();
		this.range = new LinkedList<String>();
		this.validity = "true";
		this.combination = new LinkedList<double[]>();
		this.valid = new LinkedList<Boolean>();
		this.selected = new LinkedList<Boolean>();

		this.reset();
	}

	/**
	 * Append varinats to based project
	 * 
	 * @throws TDataException
	 * @throws Exception
	 */
	public void createVariants() throws TDataException, Exception {
		this.createVariants(this.project);
	}

	/**
	 * Create project with variants
	 * 
	 * @param file
	 *            File of new project
	 * @return New project with variants
	 * @throws RemoteException
	 * @throws Exception
	 */
	public IProject createVariants(File file)
			throws RemoteException, Exception {
		IProject proj = this.project.extractIndex(this.project.getIndex(),
				file);
		this.createVariants(proj);
		proj.setIndex(0);
		proj.reduce();

		return proj;
	}

	/**
	 * Append variants to given project
	 * 
	 * @param proj
	 *            Project to append variants
	 * @throws TDataException
	 * @throws Exception
	 */
	private void createVariants(IProject proj)
			throws TDataException, Exception {
		int index;
		IParameter[] params;
		double[] values;

		params = new IParameter[this.used.size()];
		for (int k = 0; k < params.length; k++) {
			params[k] = proj.getObject(this.used.get(k).getIdentifier());
		}

		index = proj.getIndex();

		for (int v = 0; v < this.combination.size(); v++) {
			values = this.combination.get(v);
			if (this.selected.get(v)) {
				proj.setIndex(index);
				proj.duplicate();

				for (int k = 0; k < values.length; k++) {
					params[k].setData(new TNumeric(values[k]));
				}
			}
		}
	}

	/**
	 * Load variants settings from file
	 * 
	 * @param file
	 *            Settings-file
	 * @throws XMLStreamException
	 * @throws FactoryConfigurationError
	 * @throws IOException
	 * @throws SAXException
	 */
	public void loadSettings(File file) throws XMLStreamException,
			FactoryConfigurationError, IOException, SAXException {
		ZipInputStream zipStream = null;
		ZipEntry entry;
		XMLStreamReader xmlReader = null;

		try {
			// validation of File
			zipStream = new ZipInputStream(new FileInputStream(file));
			entry = zipStream.getNextEntry();
			while ((entry != null)
					&& !entry.getName().equals("ProjectVariants.xml")) {
				entry = zipStream.getNextEntry();
			}
			if (entry == null) {
				throw new IOException(
						"Invalid settings file! " + file.getName());
			}

			// validieren
			new XMLValidator().validate(zipStream, "ProjectVariants.xsd");

			// open File for reading
			zipStream = new ZipInputStream(new FileInputStream(file));
			entry = zipStream.getNextEntry();
			while ((entry != null)
					&& !entry.getName().equals("ProjectVariants.xml")) {
				entry = zipStream.getNextEntry();
			}

			// read file
			xmlReader = XMLInputFactory.newInstance()
					.createXMLStreamReader(zipStream, "utf-8");
			while (xmlReader
					.getEventType() != XMLStreamConstants.START_ELEMENT) {
				xmlReader.next();
			}

			// read data
			this.reset();
			this.xml2obj(xmlReader);

			// end of document
			xmlReader.next();
			if (xmlReader.getEventType() != XMLStreamConstants.END_DOCUMENT) {
				throw new IOException("Undefined error by reading file!");
			}

		} finally {
			if (xmlReader != null) {
				xmlReader.close();
			}
			if (zipStream != null) {
				zipStream.close();
			}
		}
	}

	/**
	 * Store variants settings to file
	 * 
	 * @param file
	 *            Settings-file
	 * @throws XMLStreamException
	 * @throws FactoryConfigurationError
	 * @throws IOException
	 */
	public void storeSettings(File file)
			throws XMLStreamException, FactoryConfigurationError, IOException {
		ZipOutputStream zipStream = null;
		XMLStreamWriter xmlWriter = null;

		try {
			zipStream = new ZipOutputStream(new FileOutputStream(file));
			zipStream.putNextEntry(new ZipEntry("ProjectVariants.xml"));

			xmlWriter = XMLOutputFactory.newInstance()
					.createXMLStreamWriter(zipStream, "utf-8");
			xmlWriter.writeStartDocument("utf-8", "1.0");

			// write data
			this.obj2xml(xmlWriter);

			xmlWriter.writeEndDocument();
		} finally {
			if (xmlWriter != null) {
				xmlWriter.close();
			}
			if (zipStream != null) {
				zipStream.close();
			}
		}
	}

	/**
	 * Reset all data
	 * 
	 * @throws RemoteException
	 */
	public void reset() throws RemoteException {
		this.unused.clear();
		this.used.clear();
		this.used.clear();
		this.range.clear();
		this.validity = "true";
		this.resetCombinations();

		this.unused.addAll(Arrays.asList(this.project.getDynamicParams()));
	}

	/**
	 * Reset combination values and flags
	 */
	private void resetCombinations() {
		this.combination.clear();
		this.createCombinations();

		this.valid.clear();
		for (int k = 0; k < this.combination.size(); k++) {
			this.valid.add(true);
		}

		this.selected.clear();
		for (int k = 0; k < this.combination.size(); k++) {
			this.selected.add(this.valid.get(k));
		}
	}

	/**
	 * Validate combinations
	 * 
	 * @throws RemoteException
	 * @throws TDataException
	 */
	private void valitateCombination() throws RemoteException, TDataException {
		Parser parser = null;
		Map<String, TData> values;
		TData tmp;
		boolean valid;
		double[] comb;

		try {
			parser = new Parser(this.validity);
		} catch (Exception e1) {
			System.err.println(e1.getMessage());
			e1.printStackTrace();
		}
		values = new Hashtable<String, TData>();

		for (int k = 0; k < this.combination.size(); k++) {
			comb = this.combination.get(k);

			// get values
			values.clear();
			for (int n = 0; n < this.used.size(); n++) {
				values.put(this.used.get(n).getName(), new TNumeric(comb[n]));
			}

			// eval formula
			valid = false;
			if (parser != null) {
				try {
					tmp = parser.eval(values);
					if ((tmp != null) && (tmp instanceof TBoolean)) {
						valid = ((TBoolean) tmp).getBoolean();
					}
				} catch (Exception e) {
					System.err.println(e.getMessage());
					e.printStackTrace();
				}
			}
			this.valid.set(k, valid);
			this.selected.set(k, valid);
		}
	}

	/**
	 * Add parameter to used list
	 * 
	 * @param param
	 *            Parameter to select
	 * @return Selection finished
	 * @throws RemoteException
	 */
	public boolean selectUsed(IParameter param) throws RemoteException {
		boolean modified;

		modified = this.unused.remove(param);
		if (modified) {
			this.used.add(param);
			this.range.add(param.getData().toString());
			this.resetCombinations();
		}

		return modified;
	}

	/**
	 * Remove parameter to used list
	 * 
	 * @param param
	 *            Parameter to deselect
	 * @return Selection finished
	 */
	public boolean deselectUsed(IParameter param) {
		boolean modified = false;
		int index;

		index = this.used.indexOf(param);
		if (index >= 0) {
			modified = this.used.remove(param);
			this.unused.add(param);
			this.range.remove(index);
			this.resetCombinations();
		}

		return modified;
	}

	/**
	 * Get number of used parameters
	 * 
	 * @return Size
	 */
	public int getUsedSize() {
		return this.used.size();
	}

	/**
	 * Get used parameter
	 * 
	 * @param index
	 *            Index of list
	 * @return Parameter
	 */
	public IParameter getUsed(int index) {
		return this.used.get(index);
	}

	/**
	 * Get number of unused parameters
	 * 
	 * @return Size
	 */
	public int getUnusedSize() {
		return this.unused.size();
	}

	/**
	 * Get unused parameter
	 * 
	 * @param index
	 *            Index of list
	 * @return Parameter
	 */
	public IParameter getUnused(int index) {
		return this.unused.get(index);
	}

	/**
	 * Get number of range-strings
	 * 
	 * @return Size
	 */
	public int getRangeSize() {
		return this.range.size();
	}

	/**
	 * Get range-string for used parameter
	 * 
	 * @param index
	 *            Index of list
	 * @return Range-string
	 */
	public String getRange(int index) {
		return this.range.get(index);
	}

	/**
	 * Set range-string for used parameter
	 * 
	 * @param index
	 *            Index of list
	 * @param str
	 *            Range string
	 */
	public void setRange(int index, String str) {
		this.range.set(index, str);
		this.resetCombinations();
	}

	/**
	 * Get evaluated range values
	 * 
	 * @param index
	 *            Index of list
	 * @return Range-values
	 */
	public double[] getRangeValues(int index) {
		TNumeric num;
		double[] values = null;
		String rangeString;
		String[] parts;
		double start, end, step;

		rangeString = this.range.get(index);

		if (!rangeString.isEmpty()) {
			try {
				try {
					num = ((TNumeric) new Parser(rangeString).eval(null));
					values = new double[num.getNumColumns()];
					for (int k = 0; k < values.length; k++) {
						values[k] = num.getReal(0, k);
					}
				} catch (ParserException e) {
					rangeString = rangeString.trim();
					if (rangeString.startsWith("[")
							&& rangeString.endsWith("]")) {
						rangeString = rangeString.substring(1,
								rangeString.length() - 1);
					}
					parts = rangeString.split(":");
					if (parts.length == 3) {
						start = Double.parseDouble(parts[0]);
						step = Double.parseDouble(parts[1]);
						end = Double.parseDouble(parts[2]);
					} else if (parts.length == 2) {
						start = Double.parseDouble(parts[0]);
						step = 1.0;
						end = Double.parseDouble(parts[1]);
					} else {
						throw e;
					}

					values = new double[((int) ((end - start) / step)) + 1];
					for (int k = 0; k < values.length; k++) {
						values[k] = start + (k * step);
					}
				}
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}

		return values;
	}

	/**
	 * Get validity string for combinations
	 * 
	 * @return Validity-string
	 */
	public String getValidityString() {
		return this.validity;
	}

	/**
	 * Set validity string for combinations
	 * 
	 * @param str
	 *            Validity-string
	 * @throws RemoteException
	 * @throws TDataException
	 */
	public void setValidityString(String str)
			throws RemoteException, TDataException {
		this.validity = str;
		this.valitateCombination();
	}

	/**
	 * Get number of combinations
	 * 
	 * @return Size
	 */
	public int getCombinationSize() {
		return this.combination.size();
	}

	/**
	 * Get cobminations-values of used parameters
	 * 
	 * @param index
	 *            Index of combination
	 * @return Values for used parameters
	 */
	public double[] getCombination(int index) {
		return this.combination.get(index);
	}

	/**
	 * Get number of valid combinations
	 * 
	 * @return Valid size
	 */
	public int getValidSize() {
		int size = 0;

		for (boolean v : this.valid) {
			if (v) {
				size++;
			}
		}

		return size;
	}

	/**
	 * Get valid flag of combination
	 * 
	 * @param index
	 *            Index of combination
	 * @return Valid flag
	 */
	public boolean getValid(int index) {
		return this.valid.get(index);
	}

	/**
	 * Get number of selected combinations
	 * 
	 * @return Valid size
	 */
	public int getSelectedSize() {
		int size = 0;

		for (boolean v : this.selected) {
			if (v) {
				size++;
			}
		}

		return size;
	}

	/**
	 * Get selected flag of combination
	 * 
	 * @param index
	 *            Index of combination
	 * @return Selected flag
	 */
	public boolean getSelected(int index) {
		return this.selected.get(index);
	}

	/**
	 * Set selected flag of combination
	 * 
	 * @param index
	 *            Index of combination
	 * @param sel
	 *            Selected flag
	 */
	public void setSelected(int index, boolean sel) {
		this.selected.set(index, sel);
	}

	/**
	 * Get short identifier without project name
	 * 
	 * @param param
	 *            Parameter
	 * @return Short identifier
	 * @throws RemoteException
	 */
	private String getShortIdentifier(IParameter param) throws RemoteException {
		String ident;
		int index;

		ident = param.getIdentifier();
		index = ident.indexOf('.');
		if (index == -1) {
			ident = ".";
		} else {
			ident = ident.substring(index + 1, ident.length());
		}

		return ident;
	}

	/**
	 * Write data to xml
	 * 
	 * @param xmlWriter
	 *            XMLWriter
	 * @throws XMLStreamException
	 * @throws RemoteException
	 */
	void obj2xml(XMLStreamWriter xmlWriter)
			throws XMLStreamException, RemoteException {

		xmlWriter.writeStartElement("ProjectVariants");
		xmlWriter.writeAttribute("object", "ProjectVariants");

		xmlWriter.writeStartElement("Used");
		for (IParameter p : this.used) {
			xmlWriter.writeStartElement("item");
			xmlWriter.writeCharacters(this.getShortIdentifier(p));
			xmlWriter.writeEndElement();
		}
		xmlWriter.writeEndElement();

		xmlWriter.writeStartElement("RangeString");
		for (String s : this.range) {
			xmlWriter.writeStartElement("item");
			xmlWriter.writeCharacters(s);
			xmlWriter.writeEndElement();
		}
		xmlWriter.writeEndElement();

		xmlWriter.writeStartElement("ValidityString");
		for (String s : this.validity.split("\n")) {
			xmlWriter.writeStartElement("item");
			xmlWriter.writeCharacters("valid = " + s);
			xmlWriter.writeEndElement();
		}
		xmlWriter.writeEndElement();

		xmlWriter.writeStartElement("Valid");
		for (boolean v : this.valid) {
			xmlWriter.writeStartElement("item");
			if (v) {
				xmlWriter.writeCharacters("1");
			} else {
				xmlWriter.writeCharacters("0");
			}
			xmlWriter.writeEndElement();
		}
		xmlWriter.writeEndElement();

		xmlWriter.writeStartElement("Selected");
		for (boolean s : this.selected) {
			xmlWriter.writeStartElement("item");
			if (s) {
				xmlWriter.writeCharacters("1");
			} else {
				xmlWriter.writeCharacters("0");
			}
			xmlWriter.writeEndElement();
		}
		xmlWriter.writeEndElement();

		xmlWriter.writeEndElement();

	}

	/**
	 * Read data from xml
	 * 
	 * @param xmlReader
	 *            XMLReader
	 * @throws XMLStreamException
	 * @throws RemoteException
	 */
	void xml2obj(XMLStreamReader xmlReader)
			throws XMLStreamException, RemoteException {
		String str;
		IParameter param;
		boolean resetCombination = false;
		int k = 0;

		xmlReader.next();
		while (xmlReader.getEventType() == XMLStreamConstants.START_ELEMENT) {
			if (xmlReader.getLocalName().equals("Used")) {
				xmlReader.next();
				while (xmlReader
						.getEventType() == XMLStreamConstants.START_ELEMENT) {
					xmlReader.next();
					str = "";
					while (xmlReader
							.getEventType() != XMLStreamConstants.END_ELEMENT) {
						str += xmlReader.getText();
						xmlReader.next();
					}
					param = this.getParameter(str);
					if (param != null) {
						this.selectUsed(param);
					} else {
						resetCombination = true;
					}
					xmlReader.next();
				}
				xmlReader.next();
			} else if (xmlReader.getLocalName().equals("RangeString")) {
				xmlReader.next();
				while (xmlReader
						.getEventType() == XMLStreamConstants.START_ELEMENT) {
					xmlReader.next();
					str = "";
					while (xmlReader
							.getEventType() != XMLStreamConstants.END_ELEMENT) {
						str += xmlReader.getText();
						xmlReader.next();
					}
					this.range.set(k++, str);
					xmlReader.next();
				}
				xmlReader.next();
			} else if (xmlReader.getLocalName().equals("ValidityString")) {
				xmlReader.next();
				this.validity = "";
				while (xmlReader
						.getEventType() == XMLStreamConstants.START_ELEMENT) {
					xmlReader.next();
					str = "";
					while (xmlReader
							.getEventType() != XMLStreamConstants.END_ELEMENT) {
						str += xmlReader.getText();
						xmlReader.next();
					}
					if (!this.validity.isEmpty()) {
						this.validity += "\n";
					}
					this.validity += str;
					if (!this.validity.isEmpty()
							&& this.validity.contains("=")) {
						this.validity = this.validity
								.substring(this.validity.indexOf("=") + 1)
								.trim();
					}
					xmlReader.next();
				}
				xmlReader.next();
			} else if (xmlReader.getLocalName().equals("Valid")) {
				this.valid.clear();
				xmlReader.next();
				while (xmlReader
						.getEventType() == XMLStreamConstants.START_ELEMENT) {
					xmlReader.next();
					str = xmlReader.getText();
					if ("1".equals(str)) {
						this.valid.add(true);
					} else {
						this.valid.add(false);
					}
					xmlReader.next();
					xmlReader.next();
				}
				xmlReader.next();
			} else if (xmlReader.getLocalName().equals("Selected")) {
				this.selected.clear();
				xmlReader.next();
				while (xmlReader
						.getEventType() == XMLStreamConstants.START_ELEMENT) {
					xmlReader.next();
					str = xmlReader.getText();
					if ("1".equals(str)) {
						this.selected.add(true);
					} else {
						this.selected.add(false);
					}
					xmlReader.next();
					xmlReader.next();
				}
				xmlReader.next();
			} else {
				throw new XMLStreamException(
						"Unrecognized property: " + xmlReader.getLocalName());
			}
		}

		if (resetCombination) {
			this.resetCombinations();
		} else {
			this.createCombinations();
		}
	}

	/**
	 * Get parameter by name
	 * 
	 * @param shortIdentifier
	 *            Short identifier (without project name)
	 * @return Parameter from project. If not found null
	 * @throws RemoteException
	 */
	private IParameter getParameter(String shortIdentifier)
			throws RemoteException {
		IParameter p = null;
		int k = 0;

		while ((k < this.unused.size()) && (p == null)) {
			if (this.getShortIdentifier(this.unused.get(k))
					.equals(shortIdentifier)) {
				p = this.unused.get(k);
			}
			k++;
		}

		return p;
	}

	/**
	 * Create combinations
	 */
	private void createCombinations() {
		int size = 1;
		double[][] values;
		int indices[];
		double[] comb;

		// initialize
		values = new double[this.used.size()][];
		indices = new int[values.length];
		for (int k = 0; k < values.length; k++) {
			values[k] = this.getRangeValues(k);
			if (values[k] != null) {
				size *= values[k].length;
			} else {
				size = 0;
			}
			indices[k] = 0;
		}
		this.combination.clear();

		if ((size > 0) && (this.used.size() > 0)) {
			// combine
			for (int k = 0; k < size; k++) {

				// create combination
				comb = new double[values.length];
				for (int i = 0; i < values.length; i++) {
					comb[i] = values[i][indices[i]];
				}
				this.combination.add(comb);

				// calc indices
				indices[indices.length - 1]++;
				for (int i = indices.length - 1; i > 0; i--) {
					if (indices[i] >= values[i].length) {
						indices[i] = 0;
						indices[i - 1]++;
					}
				}
			}
		}
	}

}
