/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.rmi;

import java.io.File;
import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.security.NoSuchAlgorithmException;
import java.util.List;

import at.lcm.bu21.general.dtypes.TData;
import at.lcm.bu21.magopt.structure.DocFile;
import at.lcm.bu21.magopt.structure.IParamContainer;
import at.lcm.bu21.magopt.structure.IParameter;
import at.lcm.bu21.magopt.structure.ParamContainer;
import at.lcm.bu21.magopt.structure.Parameter;

/**
 * RMI adapter for parameter
 * 
 * @author Werner Koppelsttter
 */
public abstract class RParameter extends UnicastRemoteObject implements IParameter {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Parameter class */
	private Parameter param;

	/**
	 * Create rmi adapter for parameter
	 * 
	 * @param param
	 *            Parameter
	 * @throws RemoteException
	 */
	public RParameter(Parameter param) throws RemoteException {
		this.param = param;
	}

	@Override
	public String getPId() {
		return this.param.getPId();
	}

	@Override
	public String getName() {
		return this.param.getName();
	}

	@Override
	public void setName(String name) throws Exception {
		this.param.setName(name);
	}

	@Override
	public String getDescription() {
		return this.param.getDescription();
	}

	@Override
	public void setDescription(String description) {
		this.param.setDescription(description);
	}

	@Override
	public String getComment() {
		return this.param.getComment();
	}

	@Override
	public void setComment(String comment) {
		this.param.setComment(comment);
	}

	@Override
	public String getIdentifier() {
		return this.param.getIdentifier();
	}

	@Override
	public String getShortIdentifier() {
		return this.param.getShortIdentifier();
	}

	@Override
	public int getIndex() {
		return this.param.getIndex();
	}

	@Override
	public void setIndex(int index) {
		this.param.setIndex(index);
	}

	@Override
	public int getLength() {
		return this.param.getLength();
	}

	@Override
	public TData getData() {
		return this.param.getData();
	}

	@Override
	public TData getData(int index) {
		return this.param.getData(index);
	}

	@Override
	public void setData(TData data) throws Exception {
		this.param.setData(data);
	}

	@Override
	public boolean getPrint() {
		return this.param.getPrint();
	}

	@Override
	public void setPrint(boolean print) {
		this.param.setPrint(print);
	}

	@Override
	public boolean getConstant() {
		return this.param.getConstant();
	}

	@Override
	public void setConstant(boolean constant) {
		this.param.setConstant(constant);
	}

	@Override
	public boolean getUsed() {
		return this.param.getUsed();
	}

	@Override
	public boolean getReadOnly() {
		return this.param.getReadOnly();
	}

	@Override
	public boolean getModified() {
		return this.param.getModified();
	}

	@Override
	public void setModified(boolean modified) {
		this.param.setModified(modified);
	}

	@Override
	public IParameter[] getUsedBy() throws RemoteException {
		Parameter[] params;
		IParameter[] rmiParams;

		params = this.param.getUsedBy();
		rmiParams = new IParameter[params.length];
		for (int k = 0; k < params.length; k++) {
			rmiParams[k] = params[k].getRMIImpl();
		}

		return rmiParams;
	}

	@Override
	public IParameter[] getSetBy() throws RemoteException {
		Parameter[] params;
		IParameter[] rmiParams;

		params = this.param.getSetBy();
		rmiParams = new IParameter[params.length];
		for (int k = 0; k < params.length; k++) {
			rmiParams[k] = params[k].getRMIImpl();
		}

		return rmiParams;
	}

	@Override
	public IParameter removeParam() throws RemoteException {
		return this.param.removeParam().getRMIImpl();
	}

	@Override
	public void moveUp() {
		this.param.moveUp();
	}

	@Override
	public void moveDown() {
		this.param.moveDown();
	}

	@Override
	public int getIndexOfParameter() {
		return this.param.getIndexOfParameter();
	}

	@Override
	public boolean getUncertain() {
		return this.param.getUncertain();
	}

	@Override
	public boolean getError() {
		return this.param.getError();
	}

	@Override
	public boolean[] getAllError() {
		return this.param.getAllError();
	}

	@Override
	public void setError(boolean error) {
		this.param.setError(error);
	}

	@Override
	public void refresh() {
		this.param.refresh();
	}

	@Override
	public IParamContainer getParentParameter() {
		ParamContainer cont;
		IParamContainer rmiCont = null;

		cont = this.param.getParent();
		if (cont != null) {
			try {
				rmiCont = cont.getRMIImpl();
			} catch (RemoteException e) {
			}
		}

		return rmiCont;
	}

	@Override
	public IParameter getUniqueObject(String uniqueIdentifier) throws RemoteException {
		return this.param.getUniqueObject(uniqueIdentifier).getRMIImpl();
	}

	@Override
	public IParameter getObject(String identifier) throws RemoteException {
		return this.param.getObject(identifier).getRMIImpl();
	}

	@Override
	public IParameter getParamByPid(String pid) throws RemoteException {
		return this.param.getParamByPid(pid).getRMIImpl();
	}

	@Override
	public IParameter[] getParameters(String c) throws RemoteException {
		Parameter[] params;
		IParameter[] rmiParams;

		params = this.param.getParameters(c);
		rmiParams = new IParameter[params.length];
		for (int k = 0; k < params.length; k++) {
			rmiParams[k] = params[k].getRMIImpl();
		}

		return rmiParams;
	}

	@Override
	public IParameter[] getAnalysisParams() throws RemoteException {
		Parameter[] params;
		IParameter[] rmiParams;

		params = this.param.getAnalysisParams();
		rmiParams = new IParameter[params.length];
		for (int k = 0; k < params.length; k++) {
			rmiParams[k] = params[k].getRMIImpl();
		}

		return rmiParams;
	}

	@Override
	public IParameter[] getDynamicParams() throws RemoteException {
		Parameter[] params;
		IParameter[] rmiParams;

		params = this.param.getDynamicParams();
		rmiParams = new IParameter[params.length];
		for (int k = 0; k < params.length; k++) {
			rmiParams[k] = params[k].getRMIImpl();
		}

		return rmiParams;
	}

	@Override
	public void info(String message) {
		this.param.info(message);
	}

	@Override
	public void info(String message, StackTraceElement[] stack) {
		this.param.info(message, stack);
	}

	@Override
	public void info(String identifier, String message, StackTraceElement[] stack) {
		this.param.info(identifier, message, stack);
	}

	@Override
	public void warning(String message) {
		this.param.warning(message);
	}

	@Override
	public void warning(String message, StackTraceElement[] stack) {
		this.param.warning(message, stack);
	}

	@Override
	public void warning(String identifier, String message, StackTraceElement[] stack) {
		this.param.warning(identifier, message, stack);
	}

	@Override
	public void error(String message) {
		this.param.error(message);
	}

	@Override
	public void error(String message, StackTraceElement[] stack) throws RemoteException {
		this.param.error(message, stack);
	}

	@Override
	public void error(String identifier, String message, StackTraceElement[] stack) {
		this.param.error(identifier, message, stack);
	}

	@Override
	public DocFile[] getDocFiles() {
		return this.param.getDocFiles();
	}

	@Override
	public DocFile getDocFile(String name) {
		return this.param.getDocFile(name);
	}

	@Override
	public File getFileFromDocFile(DocFile doc) {
		return this.param.getFileFromDocFile(doc);
	}

	@Override
	public void registerDocFile(String name, String strFile, String description)
			throws NoSuchAlgorithmException, IOException {
		this.param.registerDocFile(name, strFile, description);
	}

	@Override
	public void unregisterDocFiles() throws RemoteException {
		this.param.unregisterDocFiles();
	}

	@Override
	public void unregisterDocFile(String name) throws RemoteException {
		this.param.unregisterDocFile(name);
	}

	@Override
	public void resetDocFiles() throws RemoteException {
		this.param.resetDocFiles();
	}

	@Override
	public void resetDocFiles(int index) throws RemoteException {
		this.param.resetDocFiles(index);
	}

	@Override
	public void deleteDocFiles() throws RemoteException {
		this.param.deleteDocFiles();
	}

	@Override
	public void deleteDocFiles(int index) throws RemoteException {
		this.param.deleteDocFiles(index);
	}

	@Override
	public boolean checkDocFiles() throws RemoteException {
		return this.param.checkDocFiles();
	}

	@Override
	public boolean checkDocFiles(int index) throws RemoteException {
		return this.param.checkDocFiles(index);
	}

	@Override
	public boolean checkDocFile(String docfile) {
		return this.param.checkDocFile(docfile);
	}

	@Override
	public boolean checkDocFile(int index, String docfile) throws RemoteException {
		return this.param.checkDocFile(index, docfile);
	}

	@Override
	public String getDsId() {
		return this.param.getDsId();
	}

	@Override
	public File getParamDirectory() {
		return this.param.getParamDirectory();
	}

	@Override
	public String getUniqueDocFileName(String file) {
		return this.param.getUniqueDocFileName(file);
	}

	@Override
	public File getUniqueDocFileName(File file) {
		return this.param.getUniqueDocFileName(file);
	}

	@Override
	public String toStringI() {
		return this.param.toString();
	}

	@Override
	public List<String> listTabNames() {
		return this.param.listTabNames();
	}

	@Override
	public IParameter getRoot() throws RemoteException {
		return this.param.getRoot().getRMIImpl();
	}

	// @Override
	// public IParameter cloneParam() throws RemoteException, Exception {
	// // TODO Auto-generated method stub
	// return null;
	// }
	//
	// @Override
	// public IParameter cloneParam(IParameter clone, int index)
	// throws RemoteException, Exception {
	// return this.param.cloneParam(clone, index).getRMIImpl();
	// }
	//
	// @Override
	// public IParameter cloneParam(IParameter clone, int index, boolean
	// docFiles)
	// throws RemoteException {
	// return this.param.cloneParam(clone, index, docFiles);
	// }
	//
	// @Override
	// public IParameter deepClone() throws RemoteException, CloneException {
	// // TODO Auto-generated method stub
	// return null;
	// }
}
