/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;

/**
 * MagOpt setup
 * 
 * @author Werner Koppelsttter
 */
public class MagOptConfig {

	/** MagOpt class-loader */
	private static URLClassLoader cl = null;

	/** List of java-classpath */
	private static File[] classes = null;

	/** List of library */
	private static File[] pyPath = null;

	/** List of library */
	private static File[] libs = null;

	/** List of Matlab-path */
	private static File[] mPath = null;

	{
		MagOptConfig.collectPath();
	}

	/**
	 * Test Setup
	 * 
	 * @param args
	 *            Command line parameter
	 */
	public static void main(String[] args) {
		System.out.println(System.getProperty("java.library.path"));
		System.out.println(Arrays.toString(MagOptConfig.getClassLoader().getURLs()));
		System.out.println("Matlab: " + Arrays.toString(MagOptConfig.getMPath()));
		System.out.println("Python: " + Arrays.toString(MagOptConfig.getPyPath()));
	}

	/**
	 * Get install-path
	 * 
	 * @return Install-path
	 */
	public static String getHome() {
		String path = null;

		try {
			path = new File(MagOptConfig.class.getProtectionDomain().getCodeSource().getLocation().toURI())
					.getParentFile().getParentFile().getParent();
		} catch (URISyntaxException e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}

		return path;
	}

	/**
	 * Get MagOpt config-entry
	 * 
	 * @param name
	 *            Name of config-entry
	 * @return Config value
	 */
	public static String getConfig(String name) {
		File file;
		InputStream in;
		Properties prop, defaultProp;
		String value;

		// load default properties
		defaultProp = new Properties();
		file = new File(MagOptConfig.getHome(), "config");
		if (file.exists()) {
			try {
				in = new FileInputStream(file);
				defaultProp.load(in);
				in.close();
			} catch (IOException e) {
			}
		}

		// load local properties
		prop = new Properties();
		file = new File(MagOptConfig.getHome(), "config.local");
		if (file.exists()) {
			try {
				in = new FileInputStream(file);
				prop.load(in);
				in.close();
			} catch (IOException e) {
			}
		}

		value = prop.getProperty(name, defaultProp.getProperty(name));
		if (value != null) {
			value.trim();
		}
		return value;
	}

	/**
	 * Get class-loader with all jars of MagOpt
	 * 
	 * @return MagOpt class-loader
	 */
	public static URLClassLoader getClassLoader() {
		if ((MagOptConfig.classes == null) || (MagOptConfig.libs == null) || (MagOptConfig.mPath == null)) {
			MagOptConfig.collectPath();
		}

		MagOptConfig.cl = MagOptConfig.createClassLoader();

		return MagOptConfig.cl;
	}

	/**
	 * Get java class-path
	 * 
	 * @return List of class-path
	 */
	public static File[] getJavaPath() {
		if (MagOptConfig.classes == null) {
			MagOptConfig.collectPath();
		}

		return MagOptConfig.classes;
	}

	/**
	 * Get library-path
	 * 
	 * @return List of library-path
	 */
	public static File[] getLibPath() {
		if (MagOptConfig.libs == null) {
			MagOptConfig.collectPath();
		}

		return MagOptConfig.libs;
	}

	/**
	 * Get Matlab-path
	 * 
	 * @return List of Matlab-path
	 */
	public static File[] getMPath() {
		if (MagOptConfig.mPath == null) {
			MagOptConfig.collectPath();
		}

		return MagOptConfig.mPath;
	}

	/**
	 * Get Python-path
	 * 
	 * @return List of Pyhton-path
	 */
	public static File[] getPyPath() {
		if (MagOptConfig.pyPath == null) {
			MagOptConfig.collectPath();
		}

		return MagOptConfig.pyPath;
	}

	/**
	 * Update java-path of java-system
	 */
	public static void updateJava() {
		URLClassLoader cl;
		Method m;

		cl = (URLClassLoader) ClassLoader.getSystemClassLoader();

		try {
			m = URLClassLoader.class.getDeclaredMethod("addURL", URL.class);
			m.setAccessible(true);
			for (File f : MagOptConfig.getJavaPath()) {
				m.invoke(cl, f.toURI().toURL());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Update library-path of java-system
	 */
	public static void updateLib() {
		StringBuffer sb;
		String libPath;
		Field field;

		if (MagOptConfig.getLibPath().length != 0) {
			libPath = File.pathSeparator + System.getProperty("java.library.path").toLowerCase() + File.pathSeparator;
			sb = new StringBuffer();
			for (File f : MagOptConfig.getLibPath()) {
				if (!libPath.contains(File.pathSeparator + f.getPath().toLowerCase() + File.pathSeparator)) {
					sb.append(f.getPath() + File.pathSeparator);
				}
			}

			System.setProperty("java.library.path", sb.toString() + System.getProperty("java.library.path"));
			// update library-path
			try {
				field = ClassLoader.class.getDeclaredField("sys_paths");
				field.setAccessible(true);
				field.set(null, null);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Get Jep path
	 * 
	 * @return Directory of jep
	 */
	public static File getJepPath() {
		String config;
		File file, site, jep = null;

		config = MagOptConfig.getConfig("PythonPath");
		if ((config != null) && !config.isEmpty()) {
			file = new File(config, "Lib");
			if (!file.exists()) {
				file = new File(config, "lib");
			}
			if (file.exists()) {
				site = new File(file, "site-packages");
				if (!site.exists()) {
					for (File f : file.listFiles()) {
						if (!site.exists()) {
							if (f.isDirectory() && f.getName().toLowerCase().startsWith("python")) {
								site = new File(f, "site-packages");
							}
						}
					}
				}

				if (site.exists()) {
					file = new File(site, "jep");
					if (file.exists() && file.isDirectory()) {
						jep = file;
					}
				}
			}
		}

		return jep;
	}

	/**
	 * Collect all path (class-path, library-path and Matlab-path
	 */
	private static void collectPath() {
		File magopt;
		String sys;
		LinkedList<File> jars;
		LinkedList<File> libs;
		LinkedList<File> mPath;
		LinkedList<File> pyPath;
		String config;
		String[] tmp;
		File jar, bin;
		File file;
		boolean exist;

		magopt = new File(MagOptConfig.getHome());
		sys = System.getProperty("sun.arch.data.model");

		jars = new LinkedList<File>();
		libs = new LinkedList<File>();
		mPath = new LinkedList<File>();
		pyPath = new LinkedList<File>();

		// add python paths
		config = MagOptConfig.getConfig("PythonPath");
		if ((config != null) && !config.isEmpty()) {
			file = new File(config);
			if (file.exists()) {
				libs.add(file);
				file = new File(new File(file, "Library"), "bin");
				if (file.exists()) {
					libs.add(file);
				}

				// add jep
				file = MagOptConfig.getJepPath();
				if (file != null) {
					for (File f : file.listFiles()) {
						if (f.getName().toLowerCase().endsWith(".jar")) {
							jars.add(f);
						}
					}
					libs.add(file);
				}
			}
		}

		// add path from config
		config = MagOptConfig.getConfig("ExternalClasspath");
		if ((config != null) && !config.isEmpty()) {
			for (String str : config.split(";")) {
				jar = new File(str.trim());
				exist = false;
				for (int j = 0; j < jars.size(); j++) {
					exist |= jar.getPath().compareToIgnoreCase(jars.get(j).getPath()) == 0;
				}
				if (!exist) {
					jars.add(jar);
				}
			}
		}
		config = MagOptConfig.getConfig("ExternalLibraries");
		if ((config != null) && !config.isEmpty()) {
			for (String str : config.split(";")) {
				file = new File(str.trim());
				exist = false;
				for (int f = 0; f < libs.size(); f++) {
					exist |= file.getPath().compareToIgnoreCase(libs.get(f).getPath()) == 0;
				}
				if (!exist) {
					libs.add(file);
				}
			}
		}
		config = MagOptConfig.getConfig("ExternalPythonPath");
		if ((config != null) && !config.isEmpty()) {
			for (String str : config.split(";")) {
				file = new File(str.trim());
				exist = false;
				for (int f = 0; f < pyPath.size(); f++) {
					exist |= file.getPath().compareToIgnoreCase(pyPath.get(f).getPath()) == 0;
				}
				if (!exist) {
					pyPath.add(file);
				}
			}
		}

		// collect paths
		MagOptConfig.collectJavaPath(jars, libs, sys, new File(magopt, "System" + File.separator + "Java"));
		MagOptConfig.collectJavaRec(jars, libs, sys, new File(magopt, "External"));
		MagOptConfig.collectJavaSys(jars, libs, sys, new File(magopt, "System"));

		// gui
		file = new File(magopt, "System" + File.separator + "GuiSwing");
		if (file.exists()) {
			MagOptConfig.collectJavaPath(jars, libs, sys, file);
		}
		file = new File(magopt, "System" + File.separator + "GuiSWT");
		if (file.exists()) {
			MagOptConfig.collectJavaPath(jars, libs, sys, file);
		}
		file = new File(magopt, "System" + File.separator + "GuiFX");
		if (file.exists()) {
			MagOptConfig.collectJavaPath(jars, libs, sys, file);
		}

		// Library
		for (File f : MagOptConfig.getLibraryPath()) {
			MagOptConfig.collectLib(jars, libs, mPath, pyPath, sys, f);
		}

		MagOptConfig.collectJavaPath(jars, libs, sys,
				new File(magopt, "Tools" + File.separator + "JobManager" + File.separator + "Java"));
		mPath.add(new File(magopt, "System" + File.separator + "Matlab" + File.separator + "gui"));
		mPath.add(new File(magopt, "System" + File.separator + "Matlab" + File.separator + "kernel"));
		mPath.add(new File(magopt, "System" + File.separator + "Matlab" + File.separator + "tools"));
		// TODO integrate to function
		pyPath.add(new File(magopt, "System" + File.separator + "Python" + File.separator + "src"));
		pyPath.add(new File(magopt,
				"System" + File.separator + "General" + File.separator + "Python" + File.separator + "src"));

		// replace jar-file to bin-folder
		config = MagOptConfig.getConfig("JavaJar2Bin");
		if ((config != null) && !config.isEmpty()) {
			for (String str : config.split(";")) {
				tmp = str.split("->");
				jar = new File(tmp[0].trim());
				bin = new File(tmp[1].trim());
				for (int j = 0; j < jars.size(); j++) {
					if (jar.getPath().compareToIgnoreCase(jars.get(j).getPath()) == 0) {
						jars.set(j, bin);
					}
				}
			}
		}

		// add path from config
		config = MagOptConfig.getConfig("ExternalMatlabPath");
		if ((config != null) && !config.isEmpty()) {
			for (String str : config.split(";")) {
				file = new File(str.trim());
				exist = false;
				for (int f = 0; f < mPath.size(); f++) {
					exist |= file.getPath().compareToIgnoreCase(mPath.get(f).getPath()) == 0;
				}
				if (!exist) {
					mPath.add(file);
				}
			}
		}

		MagOptConfig.classes = jars.toArray(new File[0]);
		MagOptConfig.libs = libs.toArray(new File[0]);
		MagOptConfig.mPath = mPath.toArray(new File[0]);
		MagOptConfig.pyPath = pyPath.toArray(new File[0]);
	}

	/**
	 * Get list of library-path
	 * 
	 * @return Libraries
	 */
	public static List<File> getLibraryPath() {
		ArrayList<File> libs;
		String path;
		File file;

		libs = new ArrayList<File>();
		libs.add(new File(MagOptConfig.getHome(), "Library"));

		// add external library paths
		path = MagOptConfig.getConfig("ExternalLibraryPath");
		if (path != null) {
			for (String p : path.split(File.pathSeparator)) {
				file = new File(p.trim());
				if (file.exists() && file.isDirectory()) {
					libs.add(file);
				}
			}
		}

		return libs;
	}

	/**
	 * Create classloader with all jars of MagOpt
	 * 
	 * @return MagOpt class-loader
	 */
	private static URLClassLoader createClassLoader() {
		URL[] urls;
		// modify library-path
		MagOptConfig.updateLib();

		urls = new URL[MagOptConfig.classes.length];
		for (int k = 0; k < urls.length; k++) {
			try {
				urls[k] = MagOptConfig.classes[k].toURI().toURL();
			} catch (MalformedURLException e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}

		return URLClassLoader.newInstance(urls, ClassLoader.getSystemClassLoader());
	}

	/**
	 * Collect jars and library-path from system sub-directory
	 * 
	 * @param jars
	 *            List of jars
	 * @param libs
	 *            List library-path
	 * @param sys
	 *            Type of system
	 * @param path
	 *            System-path
	 */
	private static void collectJavaSys(LinkedList<File> jars, LinkedList<File> libs, String sys, File path) {
		File dir;
		for (File f : path.listFiles()) {
			if (f.isDirectory()) {
				dir = new File(f, "Java");
				if (dir.exists()) {
					MagOptConfig.collectJavaPath(jars, libs, sys, dir);
				}

				dir = new File(f, "External");
				if (dir.exists()) {
					MagOptConfig.collectJavaRec(jars, libs, sys, dir);
				}
			}
		}
	}

	/**
	 * Collect jars and library-path only from this path
	 * 
	 * @param jars
	 *            List of jars
	 * @param libs
	 *            List library-path
	 * @param sys
	 *            Type of system
	 * @param path
	 *            Library-path
	 */
	private static void collectJavaPath(LinkedList<File> jars, LinkedList<File> libs, String sys, File path) {
		for (File f : path.listFiles()) {
			if (f.isDirectory()) {
				if ((f.getName().compareToIgnoreCase("win" + sys) == 0)
						|| (f.getName().compareToIgnoreCase("win") == 0)) {
					libs.add(f);
				}
			} else if (f.getName().toLowerCase().endsWith(".jar")) {
				jars.add(f);
			}
		}
	}

	/**
	 * Collect jars and library-path from library
	 * 
	 * @param jars
	 *            List of jars
	 * @param libs
	 *            List of library-path
	 * @param mPath
	 *            List of Matlab-path
	 * @param pyPath
	 *            List of Python-path
	 * @param sys
	 *            Type of system
	 * @param path
	 *            Library-path
	 */
	private static void collectLib(LinkedList<File> jars, LinkedList<File> libs, LinkedList<File> mPath,
			LinkedList<File> pyPath, String sys, File path) {
		File dir;

		for (File f : path.listFiles()) {
			if (f.isDirectory()) {
				if (f.getName().startsWith("_")) {
					MagOptConfig.collectLib(jars, libs, mPath, pyPath, sys, f);
				} else {
					dir = new File(f, "Functions" + File.separator + "Java");
					if (dir.exists()) {
						MagOptConfig.collectJavaPath(jars, libs, sys, dir);
					}

					dir = new File(f, "Functions" + File.separator + "Matlab");
					if (dir.exists()) {
						mPath.add(dir);
					}

					dir = new File(f, "Functions" + File.separator + "Python" + File.separator + "src");
					if (dir.exists()) {
						pyPath.add(dir);
					}
				}
			}
		}

	}

	/**
	 * Collect jars and library-path recursive
	 * 
	 * @param jars
	 *            List of jars
	 * @param libs
	 *            List library-path
	 * @param sys
	 *            Type of system
	 * @param path
	 *            Library-path
	 */
	private static void collectJavaRec(LinkedList<File> jars, LinkedList<File> libs, String sys, File path) {
		for (File f : path.listFiles()) {
			if (f.isDirectory()) {
				MagOptConfig.collectJavaPath(jars, libs, sys, f);
				MagOptConfig.collectJavaRec(jars, libs, sys, f);
			}
		}
	}

}
