/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.utils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipOutputStream;

import at.lcm.bu21.general.utils.FileUtils;

/**
 * Create zip-archiv and add files and directories
 * 
 * @author Werner Koppelsttter
 */
public class WriteZip {

	/** Zip-archive */
	ZipOutputStream zip;

	/** File to append */
	File appendFile = null;

	/** Remove Entry */
	String rmEntry = null;

	/**
	 * Remove entry from zip-file
	 * 
	 * @param file
	 *            zip-file
	 * @param entry
	 *            Entry
	 * @throws IOException
	 */
	public static void remove(String file, String entry) throws IOException {
		WriteZip.remove(new File(file), entry);
	}

	/**
	 * Remove entry from zip-file
	 * 
	 * @param file
	 *            zip-file
	 * @param entry
	 *            Entry
	 * @throws IOException
	 */
	public static void remove(File file, String entry) throws IOException {
		WriteZip zip;

		zip = new WriteZip(file, true);
		zip.rmEntry = entry;
		zip.close();
	}

	/**
	 * Create class with zip-archive
	 * 
	 * @param file
	 *            Name of zip-file
	 * @throws FileNotFoundException
	 */
	public WriteZip(String file) throws FileNotFoundException {
		this(file, false);
	}

	/**
	 * Create class with zip-archive
	 * 
	 * @param file
	 *            Zip-file
	 * @throws FileNotFoundException
	 */
	public WriteZip(File file) throws FileNotFoundException {
		this(file, false);
	}

	/**
	 * Create class with zip-archive
	 * 
	 * @param file
	 *            Name of zip-file
	 * @param append
	 *            Open to append to file
	 * @throws FileNotFoundException
	 */
	public WriteZip(String file, boolean append) throws FileNotFoundException {
		this(new File(file), append);
	}

	/**
	 * Create class with zip-archive
	 * 
	 * @param file
	 *            Zip-file
	 * @param append
	 *            Open to append to file
	 * @throws FileNotFoundException
	 */
	public WriteZip(File file, boolean append) throws FileNotFoundException {
		int i;
		if (append && file.exists()) {
			this.appendFile = new File(file.getPath() + "~");
			i = 1;
			while (this.appendFile.exists()) {
				this.appendFile = new File(file.getPath() + "~(" + i++ + ")");
			}
			file.renameTo(this.appendFile);
		}
		this.zip = new ZipOutputStream(new BufferedOutputStream(
				new FileOutputStream(file)));
	}

	/**
	 * Add file to archive
	 * 
	 * @param file
	 *            File/Directory to add
	 * @throws IOException
	 */
	public void add(String file) throws IOException {
		this.add("", file);
	}

	/**
	 * Add file to archive
	 * 
	 * @param file
	 *            File/Directory to add
	 * @throws IOException
	 */
	public void add(File file) throws IOException {
		this.add("", file);
	}

	/**
	 * Add files to archive
	 * 
	 * @param files
	 *            Files/Directories to add
	 * @throws IOException
	 */
	public void add(String... files) throws IOException {
		this.add("", files);
	}

	/**
	 * Add files to archive
	 * 
	 * @param files
	 *            Files/Directories to add
	 * @throws IOException
	 */
	public void add(File... files) throws IOException {
		this.add("", files);
	}

	/**
	 * Add file to archive
	 * 
	 * @param entryDir
	 *            Sub-directory in archive
	 * @param file
	 *            File/Directory to add
	 * @throws IOException
	 */
	public void add(String entryDir, String file) throws IOException {
		this.add(entryDir, new String[] { file });
	}

	/**
	 * Add file to archive
	 * 
	 * @param entryDir
	 *            Sub-directory in archive
	 * @param file
	 *            File/Directory to add
	 * @throws IOException
	 */
	public void add(String entryDir, File file) throws IOException {
		this.add(entryDir, new File[] { file });
	}

	/**
	 * Add files to archive
	 * 
	 * @param entryDir
	 *            Sub-directory in archive
	 * @param files
	 *            Files/Directories to add
	 * @throws IOException
	 */
	public void add(String entryDir, String... files) throws IOException {
		File[] list;

		list = new File[files.length];
		for (int i = 0; i < files.length; i++) {
			list[i] = new File(files[i]);
		}
		this.add(entryDir, list);
	}

	/**
	 * Add files to archive
	 * 
	 * @param entryDir
	 *            Sub-directory in archive
	 * @param files
	 *            Files/Directories to add
	 * @throws IOException
	 */
	public void add(String entryDir, File... files) throws IOException {
		if ((entryDir == null) || (entryDir.length() == 0)) {
			entryDir = "";
		} else {
			entryDir += "/";
		}

		for (File f : files) {
			this.addFile(entryDir, f);
		}
	}

	/**
	 * Add file/directory to zip-archive
	 * 
	 * @param entryDir
	 *            Entry-directory for zip-archive
	 * @param file
	 *            File/directory to add
	 * @throws IOException
	 */
	protected void addFile(String entryDir, File file) throws IOException {

		if (file.isDirectory()) {
			entryDir += file.getName() + "/";
			this.addFileEntry(entryDir, null);
			for (File f : file.listFiles()) {
				this.addFile(entryDir, f);
			}
		} else {
			this.addFileEntry(entryDir + file.getName(), file);
		}

	}

	/**
	 * Add file to archive
	 * 
	 * @param entry
	 *            Entry of file
	 * @param file
	 *            File to add
	 * @throws IOException
	 */
	protected void addFileEntry(String entry, File file) throws IOException {
		BufferedOutputStream os;
		BufferedInputStream is;
		ZipEntry ze;

		ze = new ZipEntry(entry);
		this.zip.putNextEntry(ze);
		if (file != null) {
			os = new BufferedOutputStream(this.zip);
			is = new BufferedInputStream(new FileInputStream(file));
			FileUtils.copy(is, os);
			os.flush();
			is.close();
		}
		this.zip.closeEntry();
	}

	/**
	 * Add OutputStream to entry
	 * 
	 * @param entry
	 *            Entry to add
	 * @return OutpoutStream to add
	 * @throws IOException
	 */
	public OutputStream addStream(String entry) throws IOException {
		OutputStream out = null;
		ZipEntry ze;

		ze = new ZipEntry(entry);
		this.zip.putNextEntry(ze);

		out = new BufferedOutputStream(this.zip);

		return out;
	}

	/**
	 * Close zip-archive
	 * 
	 * @throws IOException
	 */
	public void close() throws IOException {
		ReadZip app;
		InputStream in;
		OutputStream out;

		if (this.appendFile != null) {
			app = new ReadZip(this.appendFile);
			for (String entry : app.listEntries()) {
				if ((this.rmEntry == null)
						|| (this.rmEntry.compareToIgnoreCase(entry) != 0)) {
					try {
						in = app.unzipStream(entry);
						out = this.addStream(entry);
						FileUtils.copy(in, out);
						in.close();
						out.flush();
					} catch (ZipException ex) {
					}
				}
			}
			app.close();
			this.appendFile.delete();
		}
		this.zip.close();
	}
}
