/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.component.browser;

import java.awt.BorderLayout;
import java.awt.Desktop;
import java.awt.FlowLayout;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.WindowConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import magopt.gui.web.WebPanel;
import magopt.utils.ComponentInfo;

/**
 * Tree view panel for component-browser
 * 
 * @author Werner Koppelsttter
 */
public class ComponentBrowserTreeView extends JDialog {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	// /** Header panel */
	// private JPanel header;

	/** Split pane for tree and browser */
	private JSplitPane pane;

	/** Tree to display component structure */
	private JTree tree;

	/** Panel for web-browser */
	private WebPanel webPanel;

	/** Footer panel */
	private JPanel footer;

	/** Smart insert button */
	private JButton smart;

	/** Insert button */
	private JButton insert;

	/** Cancel button */
	private JButton cancel;

	/** Selected item */
	private ComponentInfo selected = null;

	/** Smart insert flag */
	private boolean smartInsert = false;

	/**
	 * Create a panel to display a project tree
	 * 
	 * @param components
	 *            Component-infos to display
	 * @param parent
	 *            Parent frame
	 */
	public ComponentBrowserTreeView(ComponentInfo[] components, JFrame parent) {
		super(parent, true);

		this.setName("Component browser");
		this.setLayout(new BorderLayout());
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		// // header panel
		// this.header = new JPanel();
		// this.add(this.header, BorderLayout.NORTH);

		// add tree
		MutableTreeNode node = this.createNode(components);
		this.tree = new JTree(node);
		this.tree.setBorder(new EmptyBorder(5, 5, 5, 5));
		this.tree.setCellRenderer(new ComponentBrowserTreeCellRenderer());

		this.tree.setRootVisible(false);
		this.tree.setAutoscrolls(true);
		this.tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		this.tree.setRowHeight(0);
		// this.tree.expandPath(new TreePath(this.model));
		// ToolTipManager.sharedInstance().registerComponent(this.tree);
		this.tree.addTreeSelectionListener(new TreeSelectionListener() {

			@Override
			public void valueChanged(TreeSelectionEvent e) {
				final Object item;

				item = ((DefaultMutableTreeNode) e.getPath().getLastPathComponent()).getUserObject();
				if (item instanceof ComponentInfo) {
					ComponentBrowserTreeView.this.smart.setEnabled(true);
					ComponentBrowserTreeView.this.insert.setEnabled(true);
					try {
						if ((((ComponentInfo) item).getHelp() != null) && ((ComponentInfo) item).getHelp().exists()) {
							ComponentBrowserTreeView.this.webPanel
									.load(((ComponentInfo) item).getHelp().toURI().toURL().toString());
						}
					} catch (MalformedURLException e1) {
						e1.printStackTrace();
					}
				} else {
					ComponentBrowserTreeView.this.smart.setEnabled(false);
					ComponentBrowserTreeView.this.insert.setEnabled(false);
					ComponentBrowserTreeView.this.webPanel.load(null);
				}
			}
		});
		this.tree.addMouseListener(new MouseAdapter() {

			@Override
			public void mouseClicked(MouseEvent e) {
				TreePath path;

				path = ComponentBrowserTreeView.this.tree.getPathForLocation(e.getX(), e.getY());
				if (path != null) {
					if (ComponentBrowserTreeView.this.tree.isExpanded(path)) {
						ComponentBrowserTreeView.this.tree.collapsePath(path);
					} else {
						ComponentBrowserTreeView.this.tree.expandPath(path);
					}
				}
			}
		});
		JScrollPane scrollPane = new JScrollPane(this.tree);

		// add browser
		this.webPanel = new WebPanel();
		this.webPanel.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent event) {
				File help;

				if (event.getClickCount() == 2) {
					help = ((ComponentInfo) ((DefaultMutableTreeNode) ComponentBrowserTreeView.this.tree
							.getSelectionPath().getLastPathComponent()).getUserObject()).getHelp();
					if ((help != null) && help.exists() && (Desktop.isDesktopSupported())) {
						try {
							Desktop.getDesktop().browse(help.toURI());
						} catch (IOException e) {
						}
					}
				}
			}
		});

		// split pane with tree and browser
		this.pane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, scrollPane, this.webPanel);
		this.pane.setDividerLocation(300);
		// this.pane.setEnabled(false);
		this.pane.setBorder(new EmptyBorder(10, 10, 10, 10));
		this.add(this.pane, BorderLayout.CENTER);

		// footer with buttons
		this.footer = new JPanel();
		((FlowLayout) this.footer.getLayout()).setAlignment(FlowLayout.RIGHT);
		this.add(this.footer, BorderLayout.SOUTH);

		this.smart = new JButton("Smart Insert");
		this.smart.setEnabled(false);
		this.footer.add(this.smart);
		this.smart.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ComponentBrowserTreeView.this.smartInsert = true;
				ComponentBrowserTreeView.this.selected = (ComponentInfo) ((DefaultMutableTreeNode) ComponentBrowserTreeView.this.tree
						.getSelectionPath().getLastPathComponent()).getUserObject();
				ComponentBrowserTreeView.this.dispose();
			}
		});

		this.insert = new JButton("Insert at position");
		this.insert.setEnabled(false);
		this.footer.add(this.insert);
		this.insert.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ComponentBrowserTreeView.this.smartInsert = false;
				ComponentBrowserTreeView.this.selected = (ComponentInfo) ((DefaultMutableTreeNode) ComponentBrowserTreeView.this.tree
						.getSelectionPath().getLastPathComponent()).getUserObject();
				ComponentBrowserTreeView.this.dispose();
			}
		});

		this.cancel = new JButton("Cancel");
		this.footer.add(this.cancel);
		this.cancel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ComponentBrowserTreeView.this.dispose();
			}
		});

		// set size
		int width = 1250;
		int height = 1000;
		Rectangle rec = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
		width = rec.width < width ? rec.width : width;
		height = rec.height < height ? rec.height : height;
		this.setSize(width, height);
		this.setLocation(rec.x + ((rec.width - width) / 2), rec.y + ((rec.height - height) / 2));

		// // expand all nodes
		// for (int k = 0; k < this.tree.getRowCount(); k++) {
		// this.tree.expandRow(k);
		// }

		// // select first leaf node
		// int k = 0;
		// while ((k < this.tree.getRowCount())
		// && !((TreeNode) this.tree.getPathForRow(k)
		// .getLastPathComponent()).isLeaf()) {
		// k++;
		// }
		// if (k < this.tree.getRowCount()) {
		// this.tree.setSelectionRow(k);
		// }

	}

	/**
	 * Create tree-node structure of components
	 * 
	 * @param comps
	 *            Components
	 * @return Tree-nodes
	 */
	private MutableTreeNode createNode(ComponentInfo[] comps) {
		DefaultMutableTreeNode itemNode, node, root, tmp;
		int index;

		root = new DefaultMutableTreeNode("Components");

		for (ComponentInfo item : comps) {
			itemNode = new DefaultMutableTreeNode(item);
			node = root;
			for (String p : item.getPackageName().split("\\.")) {
				if (!p.isEmpty()) {
					index = -1;
					for (int k = 0; k < node.getChildCount(); k++) {
						if (((DefaultMutableTreeNode) node.getChildAt(k)).getUserObject().equals(p)) {
							index = k;
						}
					}
					if (index == -1) {
						tmp = new DefaultMutableTreeNode(p);
						node.add(tmp);
						node = tmp;
					} else {
						node = (DefaultMutableTreeNode) node.getChildAt(index);
					}
				}
			}
			node.add(itemNode);
		}

		return root;
	}

	/**
	 * Display dialog and return selected item
	 * 
	 * @return Selected item (null if cancel)
	 */
	public ComponentInfo getSelectedItem() {
		this.setVisible(true);

		return this.selected;
	}

	/**
	 * Is smart insert selected
	 * 
	 * @return Smart insert
	 */
	public boolean isSmartInsert() {
		return this.smartInsert;
	}
}
