/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.tree;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.SystemColor;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.Autoscroll;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.Timer;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import at.lcm.bu21.magopt.structure.ParamContainer;
import at.lcm.bu21.magopt.structure.Parameter;

/**
 * JTree with drag and drop (DnD) support for project
 * 
 * @author Markus Priglinger
 */
public class ProjectDnDTree extends JTree implements DragSourceListener,
		DragGestureListener, Autoscroll, TreeModelListener {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** time (in ms) to expand/collapse a node after mouse was moved over it */
	private static final int HOVER_TIME = 2000;

	// Fields...
	/** The path being dragged */
	private TreePath _pathSource;

	/** The 'drag image' */
	private BufferedImage _imgGhost;

	/** Where, in the drag image, the mouse was clicked */
	private Point _ptOffset = new Point();

	/** true if dragged node should put dropped into closed folder */
	public boolean _dropIntoFolder = false;

	/**
	 * Constructor
	 */
	public ProjectDnDTree() {

		this.putClientProperty("JTree.lineStyle", "Angled");

		// Make this JTree a drag source
		DragSource dragSource = DragSource.getDefaultDragSource();
		dragSource.createDefaultDragGestureRecognizer(this,
				DnDConstants.ACTION_COPY_OR_MOVE, this);

		// Also, make this JTree a drag target
		DropTarget dropTarget = new DropTarget(this,
				new ProjectDropTargetListener(this));
		dropTarget.setDefaultActions(DnDConstants.ACTION_COPY_OR_MOVE);
	}

	// Interface: DragGestureListener
	@Override
	public void dragGestureRecognized(DragGestureEvent e) {

		Point ptDragOrigin = e.getDragOrigin();
		TreePath path = this.getPathForLocation(ptDragOrigin.x, ptDragOrigin.y);
		if (path == null) {
			return;
		}
		if (this.isRootPath(path)) {
			return; // Ignore user trying to drag the root node
		}

		// Work out the offset of the drag point from the TreePath bounding
		// rectangle origin
		Rectangle raPath = this.getPathBounds(path);
		this._ptOffset.setLocation(ptDragOrigin.x - raPath.x, ptDragOrigin.y
				- raPath.y);

		// Get the cell renderer (which is a JLabel) for the path being dragged
		// JLabel lbl = (JLabel) getCellRenderer().getTreeCellRendererComponent

		JPanel pnl = (JPanel) this.getCellRenderer()
				.getTreeCellRendererComponent(this, // tree
						path.getLastPathComponent(), // value
						false, // isSelected (dont want a colored background)
						this.isExpanded(path), // isExpanded
						this.getModel().isLeaf(path.getLastPathComponent()), // isLeaf
						0, // row (not important for rendering)
						false // hasFocus (dont want a focus rectangle)
				);

		// lbl.setSize((int)raPath.getWidth(), (int)raPath.getHeight()); // <--
		// The layout manager would normally do this
		pnl.setSize((int) raPath.getWidth(), (int) raPath.getHeight()); // <--
																		// The
																		// layout
																		// manager
																		// would
																		// normally
																		// do
																		// this

		// we need a frame to render the component onto
		JFrame picFrame = new JFrame();
		picFrame.add(pnl);
		picFrame.pack(); // this calculates the bounding

		// Get a buffered image of the selection for dragging a ghost image
		// _imgGhost = new BufferedImage((int) raPath.getWidth(),
		// (int) raPath.getHeight(), BufferedImage.TYPE_INT_ARGB_PRE);

		int pnl_w = pnl.getWidth();
		int pnl_h = pnl.getHeight();

		this._imgGhost = new BufferedImage(pnl_w, pnl_h,
				BufferedImage.TYPE_INT_ARGB);
		// _imgGhost = new BufferedImage(pnl_w, pnl_h,
		// BufferedImage.TYPE_INT_RGB);

		Graphics2D g2 = this._imgGhost.createGraphics();

		// Ask the cell renderer to paint itself into the BufferedImage
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC, 0.8f)); // Make
																				// the
																				// image
																				// ghostlike
		// pnl.repaint();
		pnl.paint(g2);

		// Now paint a gradient UNDER the ghosted JLabel text (but not under the
		// icon if any)
		// Note: this will need tweaking if your icon is not positioned to the
		// left of the text
		// Icon icon = lbl.getIcon();
		// int nStartOfText = (icon == null) ? 0 :
		// icon.getIconWidth()+lbl.getIconTextGap();

		// Make the gradient ghostlike
		// g2.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_OVER,
		// 05f));

		// ORI g2.setPaint(new GradientPaint(nStartOfText, 0,
		// SystemColor.controlShadow, getWidth(), 0, new Color(255,255,255,0)));
		// g2.setPaint(new GradientPaint(0, 0, SystemColor.controlShadow, pnl_w,
		// 0, new Color(255, 255, 255, 0)));
		// ORI g2.fillRect(nStartOfText, 0, getWidth(), _imgGhost.getHeight());
		// g2.fillRect(0, 0, pnl_w, pnl_h);

		g2.dispose();
		picFrame.dispose();

		this.setSelectionPath(path); // Select this path in the tree

		// Wrap the path being transferred into a Transferable object
		Transferable transferable = new ProjectTransferableTreePath(path);

		// Remember the path being dragged (because if it is being moved, we
		// will have to delete it later)
		this._pathSource = path;

		// We pass our drag image just in case it IS supported by the platform
		e.startDrag(null, this._imgGhost, new Point(5, 5), transferable, this);
	}

	// /////////////////////////////////////////////
	//
	// Interface: DragSourceListener
	//
	// /////////////////////////////////////////////
	@Override
	public void dragEnter(DragSourceDragEvent e) {
	}

	@Override
	public void dragOver(DragSourceDragEvent e) {
	}

	@Override
	public void dragExit(DragSourceEvent e) {
	}

	@Override
	public void dropActionChanged(DragSourceDragEvent e) {
	}

	@Override
	public void dragDropEnd(DragSourceDropEvent e) {
		if (e.getDropSuccess()) {
			int nAction = e.getDropAction();
			if (nAction == DnDConstants.ACTION_MOVE) { // The dragged item
														// (_pathSource) has
														// been inserted at
														// the target selected
														// by the user.
														// Now it is time to
														// delete it from
														// its original
														// location.
				// .
				// .. ask your TreeModel to delete the node
				// .

				ProjectDnDTree.this._pathSource = null;
			}
		}
	}

	/**
	 * Interface: DropTargetListener
	 * 
	 */
	public class ProjectDropTargetListener implements DropTargetListener {
		// Fields...
		/** last path to node */
		private TreePath _pathLast = null;
		/** rectangle for cue geometry */
		private Rectangle2D _raCueGeometry = new Rectangle2D.Float();
		/** rectangle for ghost geometry */
		private Rectangle2D _raGhost = new Rectangle2D.Float();
		/** color for cue line */
		private Color _colorCueLine;
		/** color for cue box */
		private Color _colorCueBox;
		/** used to check if mouse is moving */
		private Point _ptLast = new Point();
		/** Timer for automatically expanding/collapsing tree paths */
		private Timer _timerHover;
		/** corresponding tree */
		private ProjectDnDTree _tree;

		// /
		/**
		 * Constructor
		 * 
		 * @param tree
		 *            corresponding tree
		 */
		public ProjectDropTargetListener(ProjectDnDTree tree) {

			this._tree = tree;
			this._colorCueLine = new Color(SystemColor.controlShadow.getRed(),
					SystemColor.controlShadow.getGreen(),
					SystemColor.controlShadow.getBlue(), 64);
			this._colorCueBox = new Color(0,
					SystemColor.controlShadow.getGreen(), 0, 32);

			// Set up a hover timer, so that a node will be automatically
			// expanded or collapsed
			// if the user lingers on it for more than a short time

			this._timerHover = new Timer(ProjectDnDTree.HOVER_TIME,
					new ActionListener() {

						@Override
						public void actionPerformed(ActionEvent e) {

							if (ProjectDnDTree.this
									.isRootPath(ProjectDropTargetListener.this._pathLast)) {
								return; // Do nothing if we are hovering over
										// the root node
							}
							if (ProjectDnDTree.this
									.isExpanded(ProjectDropTargetListener.this._pathLast)) {
								ProjectDnDTree.this
										.collapsePath(ProjectDropTargetListener.this._pathLast);
							} else {
								ProjectDnDTree.this
										.expandPath(ProjectDropTargetListener.this._pathLast);
							}
						}
					});
			this._timerHover.setRepeats(false); // Set timer to one-shot mode
		}

		// DropTargetListener interface
		@Override
		public void dragEnter(DropTargetDragEvent e) {

			if (!this.isDragAcceptable(e)) {
				e.rejectDrag();
			} else {
				e.acceptDrag(e.getDropAction());
			}
		}

		@Override
		public void dragExit(DropTargetEvent e) {

			ProjectDnDTree.this.repaint(this._raGhost.getBounds());
			// if (!DragSource.isDragImageSupported()) {
			// repaint(_raGhost.getBounds());
			// }
		}

		/**
		 * This is where the ghost image is drawn
		 */
		@Override
		public void dragOver(DropTargetDragEvent e) {

			// Even if the mouse is not moving, this method is still invoked 10
			// times per second
			Point pt = e.getLocation();
			if (pt.equals(this._ptLast)) {
				return;
			}

			this._ptLast = pt;

			Graphics2D g2 = (Graphics2D) ProjectDnDTree.this.getGraphics();

			// If a drag image is not supported by the platform, then draw my
			// own drag image
			if (!DragSource.isDragImageSupported()) {
				// Rub out the last ghost image and cue line
				ProjectDnDTree.this.paintImmediately(this._raGhost.getBounds());
				// And remember where we are about to draw the new ghost image
				this._raGhost.setRect(pt.x - ProjectDnDTree.this._ptOffset.x,
						pt.y - ProjectDnDTree.this._ptOffset.y,
						ProjectDnDTree.this._imgGhost.getWidth(),
						ProjectDnDTree.this._imgGhost.getHeight());
				g2.drawImage(
						ProjectDnDTree.this._imgGhost,
						AffineTransform.getTranslateInstance(
								this._raGhost.getX(), this._raGhost.getY()),
						null);
			} else {
				// Just rub out the last cue line
				ProjectDnDTree.this.paintImmediately(this._raCueGeometry
						.getBounds());
			}

			TreePath path = ProjectDnDTree.this.getClosestPathForLocation(pt.x,
					pt.y);
			// System.out.println("DNDTree: TreePath: "+path);

			if (!(path == this._pathLast)) {
				this._pathLast = path;
				this._timerHover.restart();
			}

			Rectangle raPath = ProjectDnDTree.this.getPathBounds(path);

			// check if node should be dropped into a closed folder
			ProjectDnDTree.this._dropIntoFolder = false;
			Object nodeObject = path.getLastPathComponent();
			if (nodeObject instanceof ParamContainer) {
				ParamContainer paramContainer = (ParamContainer) nodeObject;
				boolean folderIsEmpty = paramContainer.getLength() == 0;
				if (ProjectDnDTree.this.isCollapsed(path) || folderIsEmpty) {
					float dy = (float) pt.y - raPath.y;
					float areaRatio = (float) (dy / raPath.getHeight());
					ProjectDnDTree.this._dropIntoFolder = areaRatio < 0.6f;
				}
			}

			// In any case draw (over the ghost image if necessary) a cue
			// geometry
			// indicating where a drop will occur
			if (ProjectDnDTree.this._dropIntoFolder) {
				this._raCueGeometry.setRect(0, raPath.y,
						ProjectDnDTree.this.getWidth(), raPath.getHeight());
				g2.setColor(this._colorCueBox);
				g2.fill(this._raCueGeometry);
			} else {
				this._raCueGeometry.setRect(0,
						raPath.y + (int) raPath.getHeight(),
						ProjectDnDTree.this.getWidth(), 2);
				g2.setColor(this._colorCueLine);
				g2.fill(this._raCueGeometry);
			}
			// And include the cue geometry in the area to be rubbed out next
			// time
			this._raGhost = this._raGhost.createUnion(this._raCueGeometry);

			// prohibit dropping onto the drag source
			if (path.equals(ProjectDnDTree.this._pathSource)) {
				e.rejectDrag();
			} else {
				e.acceptDrag(e.getDropAction());
			}
		}

		@Override
		public void dropActionChanged(DropTargetDragEvent e) {

			if (!this.isDragAcceptable(e)) {
				e.rejectDrag();
			} else {
				e.acceptDrag(e.getDropAction());
			}
		}

		@Override
		public void drop(DropTargetDropEvent e) {

			this._timerHover.stop(); // Prevent hover timer from doing an
										// unwanted
			// expandPath or collapsePath

			if (!this.isDropAcceptable(e)) {
				ProjectDnDTree.this.repaint(this._raGhost.getBounds()); // clear
																		// cue
																		// image
				e.rejectDrop();
				return;
			}

			Transferable transferable = e.getTransferable();

			DataFlavor[] flavors = transferable.getTransferDataFlavors();
			for (int i = 0; i < flavors.length; i++) {
				DataFlavor flavor = flavors[i];
				if (flavor
						.isMimeTypeEqual(DataFlavor.javaJVMLocalObjectMimeType)) {
					try {

						Point pt = e.getLocation();
						TreePath pathTarget = ProjectDnDTree.this
								.getClosestPathForLocation(pt.x, pt.y);
						TreePath pathSource = (TreePath) transferable
								.getTransferData(flavor);

						if (pathSource.isDescendant(pathTarget)) {
							ProjectDnDTree.this.repaint(this._raGhost
									.getBounds()); // clear cue image
							e.rejectDrop();
							return;
						}
						Rectangle nodeBoundsRectangle = ProjectDnDTree.this
								.getPathBounds(pathTarget);

						TreeModel model = ProjectDnDTree.this.getModel();

						Parameter targetObject = (Parameter) pathTarget
								.getLastPathComponent();

						int idxTarget = 0;

						boolean appendNodeAtEnd = false;
						boolean inTargetNodeBoundaries = ((pt.y > nodeBoundsRectangle.y) && (pt.y < (nodeBoundsRectangle.y + nodeBoundsRectangle.height)));
						if (!inTargetNodeBoundaries) {
							// node was dragged below the last node

							targetObject = (Parameter) model.getRoot();
							pathTarget = new TreePath(targetObject);
							idxTarget = -1;
							appendNodeAtEnd = true;
						}
						if (pathTarget.equals(pathSource)) {
							e.rejectDrop();
							return;
						}
						e.acceptDrop(e.getDropAction());

						TreePath pathNewChild = null;

						// check if target is a collapsed branch

						ParamContainer targetParent = targetObject.getParent();

						Parameter draggedObject = (Parameter) pathSource
								.getLastPathComponent();

						if (targetObject.equals(model.getRoot())) {
							// dragged on root
							targetParent = null;
						} else {
							idxTarget = model.getIndexOfChild(targetParent,
									targetObject);
						}

						try {
							if ((DnDConstants.ACTION_COPY & e.getDropAction()) != 0) {
								draggedObject = draggedObject.cloneParam();
							}

							if (model.isLeaf(targetObject)) {
								// target is a LEAF node
								targetParent.addParam(draggedObject,
										idxTarget + 1);
							} else {
								// target is a BRANCH node
								if (appendNodeAtEnd) {
									// very last position in the tree
									int rootChildrenNumber = model
											.getChildCount(model.getRoot());
									((ParamContainer) targetObject).addParam(
											draggedObject, rootChildrenNumber);
								} else {
									if (this._tree._dropIntoFolder) {
										// drop into a closed folder at the last
										// position
										ParamContainer parCont = (ParamContainer) targetObject;
										parCont.addParam(draggedObject,
												parCont.getParamSize());

									} else {
										if (this._tree.isExpanded(pathTarget)) {
											// drop on first position into open
											// folder
											((ParamContainer) targetObject)
													.addParam(draggedObject, 0);
										} else {
											// drop between closed folder and
											// other node
											int idxOfParam = targetObject
													.getIndexOfParameter();
											targetParent.addParam(
													draggedObject,
													idxOfParam + 1);
										}
									}
								}
							}
						} catch (Exception eh) {
							System.out.println("-------- !! add param failed");
						}

						// _tree.validate();
						this._tree.updateUI();

						TreePath pathTargetParent = pathTarget.getParentPath();

						if (pathTargetParent == null) {
							pathTargetParent = pathTarget;
						}
						pathNewChild = pathTargetParent
								.pathByAddingChild(draggedObject);

						if (pathNewChild != null) {
							ProjectDnDTree.this.setSelectionPath(pathNewChild); // Mark
																				// this
																				// as
																				// the
						}
						// selected path in
						// the tree
						break; // No need to check remaining flavors
					} catch (UnsupportedFlavorException ufe) {
						System.out.println(ufe);
						e.dropComplete(false);
						return;
					} catch (IOException ioe) {
						System.out.println(ioe);
						e.dropComplete(false);
						return;
					}
				}
			}

			e.dropComplete(true);
		}

		/**
		 * Refer to DropTargetListener
		 * 
		 * @param e
		 *            DropTargetDragEvent
		 * @return <CODE>true<\CODE> if drag is acceptable
		 */
		public boolean isDragAcceptable(DropTargetDragEvent e) {

			// Only accept COPY or MOVE gestures (ie LINK is not supported)
			if ((e.getDropAction() & DnDConstants.ACTION_COPY_OR_MOVE) == 0) {
				return false;
			}

			// Only accept this particular flavor
			if (!e.isDataFlavorSupported(ProjectTransferableTreePath.TREEPATH_FLAVOR)) {
				return false;
			}

			// prohibit dropping onto the drag source
			Point pt = e.getLocation();
			TreePath path = ProjectDnDTree.this.getClosestPathForLocation(pt.x,
					pt.y);
			if (path.equals(ProjectDnDTree.this._pathSource)) {
				;
				// chkbug return false;
			}

			/*
			 * /* // Do this if you want to select the best flavor on offer...
			 * DataFlavor[] flavors = e.getCurrentDataFlavors(); for (int i = 0;
			 * i < flavors.length; i++ ) { DataFlavor flavor = flavors[i]; if
			 * (flavor.isMimeTypeEqual(DataFlavor.javaJVMLocalObjectMimeType))
			 * return true; }
			 */
			return true;
		}

		/**
		 * Refer to DropTargetListener
		 * 
		 * @param e
		 *            DropTargetDropEvent
		 * @return <CODE>true<\CODE> if drop is acceptable
		 */
		public boolean isDropAcceptable(DropTargetDropEvent e) {

			// Only accept COPY or MOVE gestures (ie LINK is not supported)
			if ((e.getDropAction() & DnDConstants.ACTION_COPY_OR_MOVE) == 0) {
				return false;
			}

			// Only accept this particular flavor
			if (!e.isDataFlavorSupported(ProjectTransferableTreePath.TREEPATH_FLAVOR)) {
				return false;
			}

			// prohibit dropping onto the drag source
			Point pt = e.getLocation();
			TreePath path = ProjectDnDTree.this.getClosestPathForLocation(pt.x,
					pt.y);
			if (path.equals(ProjectDnDTree.this._pathSource)) {
				;
				// chkbug return false;
			}

			/*
			 * // Do this if you want to select the best flavor on offer...
			 * DataFlavor[] flavors = e.getCurrentDataFlavors(); for (int i = 0;
			 * i < flavors.length; i++ ) { DataFlavor flavor = flavors[i]; if
			 * (flavor.isMimeTypeEqual(DataFlavor.javaJVMLocalObjectMimeType))
			 * return true; }
			 */
			return true;
		}
	}

	// //////////////////////////////////////////////
	//
	// Interface: Autoscroll
	//
	// /////////////////////////////////////////////

	// Autoscroll Interface...
	// The following code was borrowed from the book:
	// Java Swing
	// By Robert Eckstein, Marc Loy & Dave Wood
	// Paperback - 1221 pages 1 Ed edition (September 1998)
	// O'Reilly & Associates; ISBN: 156592455X
	//
	// The relevant chapter of which can be found at:
	// http://www.oreilly.com/catalog/jswing/chapter/dnd.beta.pdf

	/** margin breath in pixels */
	private static final int AUTOSCROLL_MARGIN = 12;

	// Ok, weve been told to scroll because the mouse cursor is in our
	// scroll zone.
	@Override
	public void autoscroll(Point pt) {
		// Figure out which row were on.
		int nRow = this.getRowForLocation(pt.x, pt.y);

		// If we are not on a row then ignore this autoscroll request
		if (nRow < 0) {
			return;
		}

		Rectangle raOuter = this.getBounds();
		// Now decide if the row is at the top of the screen or at the
		// bottom. We do this to make the previous row (or the next
		// row) visible as appropriate. If were at the absolute top or
		// bottom, just return the first or last row respectively.

		nRow = ((pt.y + raOuter.y) <= ProjectDnDTree.AUTOSCROLL_MARGIN) // Is
																		// row
																		// at
																		// top
																		// of
		// screen?
		? (nRow <= 0 ? 0 : nRow - 1) // Yes, scroll up one row
				: (nRow < (this.getRowCount() - 1) ? nRow + 1 : nRow); // No,
																		// scroll
		// down one row

		this.scrollRowToVisible(nRow);
	}

	// Calculate the insets for the *JTREE*, not the viewport
	// the tree is in. This makes it a bit messy.
	@Override
	public Insets getAutoscrollInsets() {
		Rectangle raOuter = this.getBounds();
		Rectangle raInner = this.getParent().getBounds();
		return new Insets((raInner.y - raOuter.y)
				+ ProjectDnDTree.AUTOSCROLL_MARGIN, (raInner.x - raOuter.x)
				+ ProjectDnDTree.AUTOSCROLL_MARGIN, (raOuter.height
				- raInner.height - raInner.y)
				+ raOuter.y + ProjectDnDTree.AUTOSCROLL_MARGIN, (raOuter.width
				- raInner.width - raInner.x)
				+ raOuter.x + ProjectDnDTree.AUTOSCROLL_MARGIN);
	}

	/*
	 * // Use this method if you want to see the boundaries of the // autoscroll
	 * active region. Toss it out, otherwise. public void
	 * paintComponent(Graphics g) { super.paintComponent(g); Rectangle raOuter =
	 * getBounds(); Rectangle raInner = getParent().getBounds();
	 * g.setColor(Color.red); g.drawRect(-raOuter.x + 12, -raOuter.y + 12,
	 * raInner.width - 24, raInner.height - 24); }
	 */

	// //////////////////////////////////////////////
	//
	// Interface: TreeModelListener
	//
	// /////////////////////////////////////////////

	@Override
	public void treeNodesChanged(TreeModelEvent e) {
	}

	@Override
	public void treeNodesInserted(TreeModelEvent e) {

		// We need to reset the selection path to the node just inserted
		int nChildIndex = e.getChildIndices()[0];
		TreePath pathParent = e.getTreePath();
		this.setSelectionPath(ProjectDnDTree.this.getChildPath(pathParent,
				nChildIndex));
	}

	@Override
	public void treeNodesRemoved(TreeModelEvent e) {
	}

	@Override
	public void treeStructureChanged(TreeModelEvent e) {
	}

	/**
	 * Creates a new TreePath consisting of the parent path and the child
	 * 
	 * @param pathParent
	 *            TreePath to the parent
	 * @param nChildIndex
	 *            index of the child to add to the new TreePath
	 * @return TreePath to the child
	 */
	private TreePath getChildPath(TreePath pathParent, int nChildIndex) {
		TreeModel model = this.getModel();
		return pathParent.pathByAddingChild(model.getChild(
				pathParent.getLastPathComponent(), nChildIndex));
	}

	/**
	 * Checks if the path is the path to the root node
	 * 
	 * @param path
	 *            to check
	 * @return <CODE>true<\CODE> if root
	 */
	public boolean isRootPath(TreePath path) {
		return this.isRootVisible() && (this.getRowForPath(path) == 0);
	}
}
