/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.docfiles;

import java.awt.BorderLayout;
import java.awt.Desktop;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.rmi.RemoteException;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;

import at.lcm.bu21.magopt.structure.DocFile;
import at.lcm.bu21.magopt.structure.IParameter;
import magopt.gui.view.ParamBaseScroll;

/**
 * Display DocFiles of parameter
 * 
 * @author Werner Koppelsttter
 */
public class DocFilesView extends ParamBaseScroll {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Parameter to display */
	private IParameter param = null;

	/** Table to display DocFiles */
	protected JTable table;

	/** Doc-files model */
	protected DocFilesModel model;

	/** Main panel */
	private JPanel mainPanel;

	/**
	 * Create and initialize view
	 */
	public DocFilesView() {
		JScrollPane panelTable;

		this.mainPanel = new JPanel();
		this.add(this.mainPanel);
		this.mainPanel.setOpaque(true);
		this.mainPanel.setVisible(false);
		this.mainPanel.setLayout(new BorderLayout());

		this.mainPanel.setPreferredSize(this.getPreferredSize());
		this.mainPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		panelTable = new JScrollPane();
		this.mainPanel.add(panelTable);
		this.table = new JTable();
		this.table.getTableHeader().setReorderingAllowed(false);
		panelTable.setViewportView(this.table);
		this.model = new DocFilesModel();
		this.table.setModel(this.model);

		// Cell renderer
		this.table.setDefaultRenderer(Object.class, new DocFilesRenderer());
		this.table.getColumnModel().getColumn(DocFilesColumn.CHECKSUM.getValue())
				.setCellRenderer(new DocFilesChecksumRenderer());

		/* set max width for checkbox column */
		this.table.getColumnModel().getColumn(DocFilesColumn.ICON.getValue()).setPreferredWidth(40);
		this.table.getColumnModel().getColumn(DocFilesColumn.ICON.getValue()).setMaxWidth(40);
		this.table.getColumnModel().getColumn(DocFilesColumn.NAME.getValue()).setPreferredWidth(30);
		this.table.getColumnModel().getColumn(DocFilesColumn.DESCRIPTION.getValue()).setPreferredWidth(70);
		this.table.getColumnModel().getColumn(DocFilesColumn.CHECKSUM.getValue()).setPreferredWidth(40);
		this.table.getColumnModel().getColumn(DocFilesColumn.CHECKSUM.getValue()).setMaxWidth(40);

		// mouse-listener
		this.table.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent event) {
				if (event.getClickCount() == 2) {
					DocFilesView.this.openDocFile();
				}
			}

			@Override
			public void mouseReleased(MouseEvent event) {
				this.showPopup(event);
			}

			@Override
			public void mousePressed(MouseEvent event) {
				this.showPopup(event);
			}

			private void showPopup(MouseEvent event) {
				JPopupMenu popup;
				JMenuItem item;

				if (event.isPopupTrigger()) {
					// select row
					int row = DocFilesView.this.table.rowAtPoint(event.getPoint());
					DocFilesView.this.table.getSelectionModel().setSelectionInterval(row, row);

					// menu
					popup = new JPopupMenu();

					item = new JMenuItem("Open");
					item.addActionListener(new ActionListener() {
						@Override
						public void actionPerformed(ActionEvent event) {
							DocFilesView.this.openDocFile();
						}
					});
					popup.add(item);

					item = new JMenuItem("Open Folder");
					item.addActionListener(new ActionListener() {
						@Override
						public void actionPerformed(ActionEvent event) {
							DocFilesView.this.openDocFileFolder();
						}
					});
					popup.add(item);

					popup.show(event.getComponent(), event.getX(), event.getY());
				}
			}
		});

		// key-action
		this.table.getInputMap().put(KeyStroke.getKeyStroke("ENTER"), "enter");
		this.table.getActionMap().put("enter", new AbstractAction() {
			private static final long serialVersionUID = 1L;

			@Override
			public void actionPerformed(ActionEvent e) {
				DocFilesView.this.openDocFile();
			}
		});

		// this.table.setDragEnabled(true);
		// this.table.setTransferHandler(newHandler);

	}

	/**
	 * Open doc-file
	 */
	private void openDocFile() {
		File file;
		DocFile doc;

		try {
			doc = DocFilesView.this.param.getDocFiles()[DocFilesView.this.table.getSelectedRow()];

			file = this.param.getFileFromDocFile(doc);
			if (file.exists()) {
				try {
					if (Desktop.isDesktopSupported()) {
						Desktop.getDesktop().open(file);
					} else {
						throw new Exception();
					}
				} catch (Exception ex) {
					DocFilesView.this.param.info("Unable to open: " + file.getPath());
				}
			} else {
				DocFilesView.this.param.info("File does not exists: " + file.getPath());
			}
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Open doc-file
	 */
	private void openDocFileFolder() {
		File file;
		DocFile doc;

		try {
			doc = DocFilesView.this.param.getDocFiles()[DocFilesView.this.table.getSelectedRow()];

			file = this.param.getFileFromDocFile(doc).getParentFile();
			if (file.exists()) {
				try {
					if (Desktop.isDesktopSupported()) {
						Desktop.getDesktop().open(file);
					} else {
						throw new Exception();
					}
				} catch (Exception ex) {
					DocFilesView.this.param.info("Unable to open Folder: " + file.getPath());
				}
			} else {
				DocFilesView.this.param.info("Folder does not exists: " + file.getPath());
			}
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	@Override
	protected void update() {

		this.model.setParameter(this.param);

	}

	/**
	 * Sets new ParamFunction.
	 * 
	 * @param param
	 *            Parameter
	 */
	public void setParameter(IParameter param) {
		this.param = param;
		this.mainPanel.setVisible(this.param != null);
		this.update();
	}

}
