/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.utils;

import java.io.File;
import java.util.Collections;
import java.util.LinkedList;
import java.util.Properties;

import at.lcm.bu21.general.utils.FileUtils;
import at.lcm.bu21.magopt.utils.MagOptConfig;

/**
 * Component information
 *
 * @author Werner Koppelsttter
 */
public class ComponentInfo implements Comparable<ComponentInfo> {

	/** Comparator for natural string order */
	private static NaturalStringComparator comp = new NaturalStringComparator();

	/**
	 * List info about all component in the library
	 * 
	 * @return List of component-infos
	 */
	public static ComponentInfo[] listComponents() {
		LinkedList<ComponentInfo> list;

		list = new LinkedList<ComponentInfo>();
		for (File f : MagOptConfig.getLibraryPath()) {
			ComponentInfo.listComponents(list, f);
		}

		Collections.sort(list);

		return list.toArray(new ComponentInfo[0]);
	}

	/**
	 * Collect all components
	 * 
	 * @param list
	 *            List of components
	 * @param path
	 *            Folder to search
	 */
	private static void listComponents(LinkedList<ComponentInfo> list, File path) {
		File folder;
		MopFile mop;

		for (File file : path.listFiles()) {
			if (file.isDirectory()) {
				if (file.getName().startsWith("_")) {
					ComponentInfo.listComponents(list, file);
				} else {
					folder = new File(file, "Components");
					if (folder.exists()) {
						for (File f : folder.listFiles()) {
							if (f.isFile()) {
								mop = new MopFile(f);
								if (mop.isComponent()) {
									try {
										mop.expandComponent();
									} catch (Exception e) {
									}
									list.add(mop.getComponentInfo());
								}
							}
						}
					}
				}
			}
		}
	}

	/** List of properties */
	private Properties info;

	/** Depending mop-file */
	private MopFile mop;

	/**
	 * Create ComponentInfo for mop-file
	 * 
	 * @param mop
	 *            Mop-file
	 */
	public ComponentInfo(MopFile mop) {
		this.mop = mop;
		this.info = new Properties();
	}

	/**
	 * Get list of properties
	 * 
	 * @return List of properties
	 */
	public Properties getProperties() {
		return this.info;
	}

	// /**
	// * Set list of properties
	// *
	// * @param info
	// * List of properties
	// */
	// void setProperties(Properties info) {
	// this.info = info;
	// }

	/**
	 * Get depended mop-file
	 * 
	 * @return Mop-file
	 */
	public MopFile getMop() {
		return this.mop;
	}

	/**
	 * Get property-value for name
	 * 
	 * @param name
	 *            Property name
	 * @return Property value
	 */
	public String getProperty(String name) {
		return this.info.getProperty(name);
	}

	/**
	 * Set property for name
	 * 
	 * @param name
	 *            Property name
	 * @param value
	 *            Property value
	 */
	public void setProperty(String name, String value) {
		if (name == null) {
			this.info.remove(name);
		} else {
			this.info.setProperty(name, value);
		}
	}

	/**
	 * Get name of component
	 * 
	 * @return Name
	 */
	public String getName() {
		String name;

		name = this.info.getProperty("name");
		if (name == null) {
			name = this.mop.getName();
			name = name.substring(0, name.length() - 5);
		}

		return name;
	}

	/**
	 * Set name of component
	 * 
	 * @param name
	 *            Name
	 */
	public void setName(String name) {
		this.setProperty("name", name);
	}

	/**
	 * Get tags of component
	 * 
	 * @return Tag string
	 */
	public String getTags() {
		return this.info.getProperty("tags");
	}

	/**
	 * Set tags of component
	 * 
	 * @param tags
	 *            Tag string
	 */
	public void setTags(String tags) {
		this.setProperty("tags", tags);
	}

	/**
	 * Get image of component
	 * 
	 * @return Image
	 */
	public File getImage() {
		String path;
		File image = null;

		path = this.info.getProperty("image");
		if (path != null) {
			image = FileUtils.trimPath(new File(this.mop.getFilesDirectrory(), path));
		}

		return image;

	}

	/**
	 * Set image of component
	 * 
	 * @param image
	 *            Image
	 */
	public void setImage(File image) {
		String path = null;

		if (image != null) {
			path = FileUtils.getRelativePath(this.mop.getFilesDirectrory().getPath(), image.getPath());
		}
		this.setProperty("image", path);
	}

	/**
	 * Get help file
	 * 
	 * @return Help file
	 */
	public File getHelp() {
		String path;
		File help = null;

		path = this.info.getProperty("help");
		if (path != null) {
			help = FileUtils.trimPath(new File(this.mop.getFilesDirectrory(), path));
		}

		return help;
	}

	/**
	 * Set help file
	 * 
	 * @param help
	 *            Help file
	 */
	public void setHelp(File help) {
		String path = null;

		if (help != null) {
			path = FileUtils.getRelativePath(this.mop.getFilesDirectrory().getPath(), help.getPath());
		}
		this.setProperty("help", path);
	}

	/**
	 * Get simple name of component
	 * 
	 * @return Simple name
	 */
	public String getSimpleName() {
		String name;
		int index;

		name = this.getName();
		index = name.lastIndexOf(".");
		if (index >= 0) {
			name = name.substring(index + 1);
		}

		return name;
	}

	/**
	 * Get package name
	 * 
	 * @return Package name
	 */
	public String getPackageName() {
		String name;
		int index;

		name = this.getName();
		index = name.lastIndexOf(".");
		if (index >= 0) {
			name = name.substring(0, index);
		} else {
			name = "";
		}

		return name;
	}

	@Override
	public String toString() {
		return this.getSimpleName();
	}

	@Override
	public int compareTo(ComponentInfo other) {
		String[] tNames, oNames;
		int k = 0, cmp = 0;

		// split to parts
		tNames = this.getName().split("\\.");
		oNames = other.getName().split("\\.");

		// compare parts
		while ((k < tNames.length) && (k < oNames.length) && (cmp == 0)) {
			// compare names (folder first then components)
			if ((k == (tNames.length - 1)) && (tNames.length < oNames.length)) {
				cmp = 1;
			} else if ((k == (oNames.length - 1)) && (oNames.length < tNames.length)) {
				cmp = -1;
			} else {
				cmp = ComponentInfo.comp.compare(tNames[k], oNames[k]);
			}
			k++;
		}

		// different length
		if ((cmp == 0) && (tNames.length != oNames.length)) {
			cmp = tNames.length - oNames.length;
		}

		return cmp;
	}
}
