/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.utils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import at.lcm.bu21.general.utils.FileUtils;

/**
 * Class to access MOP-Archive (read/write files)
 * 
 * @author gweidenholzer
 * @author Werner Koppelsttter
 */
public class MopFile extends File {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Files extensions */
	private static final String filesExtention = ".Files";
	// FIXME data-directory
	/** Data extensions */
	private static final String dataExtention = ".Data";

	/** Tag for optimization files */
	public static final String optimizationTag = "GeneticOptimization/";

	/** Tag for component files */
	public static final String componentTag = "ComponentFiles/";

	/**
	 * Constructor
	 * 
	 * @param file
	 *            Path and Filename
	 */
	public MopFile(String file) {
		super(new File(file.toLowerCase().matches(".+\\.mop[tc]?") ? FileUtils.trimPath(file)
				: FileUtils.trimPath(file + ".mop")).getAbsolutePath());
	}

	/**
	 * Constructor
	 * 
	 * @param file
	 *            Path and Filename
	 */
	public MopFile(File file) {
		this(file.getPath());
	}

	/**
	 * Get list of internal files
	 * 
	 * @return list of files
	 * @throws Exception
	 */
	public String[] getFileNames() throws Exception {

		ArrayList<String> list = new ArrayList<String>();

		ZipFile zf = new ZipFile(this);
		Enumeration<? extends ZipEntry> entries = zf.entries();
		while (entries.hasMoreElements()) {
			list.add(entries.nextElement().getName());
		}
		zf.close();

		return list.toArray(new String[0]);
	}

	/**
	 * 
	 * 
	 * @param ma
	 *            Implementing MopAccess object
	 * @param filename
	 *            Name of file in zip
	 * @throws Exception
	 */
	public void readStream(MopAccess ma, String filename) throws Exception {
		ZipFile zf;
		Enumeration<? extends ZipEntry> entries;
		ZipEntry ze;
		BufferedInputStream is = null;
		OutputStream outputStream;
		InputStream inputStream;
		File file;

		// TODO vielleicht zipinputstream mit buffered inputstream

		zf = new ZipFile(this);
		entries = zf.entries();

		while (entries.hasMoreElements()) {
			ze = entries.nextElement();
			if (ze.getName().startsWith("Files/")) {
				file = new File(this.getFilesDirectrory(), ze.getName().substring("Files/".length()));
				file.getParentFile().mkdirs();
				outputStream = new BufferedOutputStream(new FileOutputStream(file));

				inputStream = new BufferedInputStream(zf.getInputStream(ze));
				FileUtils.copy(inputStream, outputStream);
				inputStream.close();
				outputStream.close();
			}
		}

		ze = zf.getEntry(filename);
		if (ze != null) {
			is = new BufferedInputStream(zf.getInputStream(ze));
		} else {
			zf.close();
			throw new Exception("Invalid file!");
		}

		try {

			ma.readStream(is);

		} catch (Exception e) {
			if (is != null) {
				is.close();
			}
			zf.close();
			throw e;
		}

		if (is != null) {
			is.close();
		}

		zf.close();
	}

	/**
	 * 
	 * 
	 * @param ma
	 *            Implementing MopAccess object
	 * @param filename
	 *            Name of file in zip
	 * @throws Exception
	 */
	public void writeStream(MopAccess ma, String filename) throws Exception {
		ZipFile zf;
		File bakFile;
		ZipOutputStream zipOutputStream = null;
		BufferedOutputStream zipEntryOutputStream = null, fileOutputStream = null;
		Enumeration<? extends ZipEntry> entries;

		int i = 0;
		if (this.exists()) {

			// create backup-file
			do {
				bakFile = new File(this.getPath() + "#" + i++ + "#");
			} while (bakFile.exists());
			if (!this.renameTo(bakFile)) {
				throw new Exception("Unable to create backup-file!");
			}

			// get output Stream
			fileOutputStream = new BufferedOutputStream(new FileOutputStream(this));
			zipOutputStream = new ZipOutputStream(fileOutputStream);

			// get input Stream
			zf = new ZipFile(bakFile);
			entries = zf.entries();

			while (entries.hasMoreElements()) {
				ZipEntry ze = entries.nextElement();
				if ((ze.getName().compareToIgnoreCase(filename) != 0) && !ze.getName().startsWith("Files")) {

					// get input Stream
					zipOutputStream.putNextEntry(new ZipEntry(ze.getName()));

					InputStream inputStream = new BufferedInputStream(zf.getInputStream(ze));
					FileUtils.copy(inputStream, zipOutputStream);
					inputStream.close();
				}
			}
			zf.close();

		} else {
			fileOutputStream = new BufferedOutputStream(new FileOutputStream(this));
			zipOutputStream = new ZipOutputStream(fileOutputStream);
		}

		if (ma != null) {

			// write actual file
			zipOutputStream.putNextEntry(new ZipEntry(filename));
			zipEntryOutputStream = new BufferedOutputStream(zipOutputStream);

			try {
				ma.writeStream(zipEntryOutputStream);
			} catch (Exception e) {
				if (zipEntryOutputStream != null) {
					zipEntryOutputStream.close();
				}
				if (zipOutputStream != null) {
					zipOutputStream.close();
				}
				if (fileOutputStream != null) {
					fileOutputStream.close();
				}
				throw e;
			}
		}

		// FIXME File-Version
		// add files-directory to mop
//		this.zipFiles(zipOutputStream, this.getFilesDirectrory());

		if (zipEntryOutputStream != null) {
			zipEntryOutputStream.close();
		}
		if (zipOutputStream != null) {
			zipOutputStream.close();
		}
		if (fileOutputStream != null) {
			fileOutputStream.close();
		}

		// delete backup-files
		File[] fileList = this.getParentFile().listFiles();
		for (File f : fileList) {
			if (f.getName().indexOf(this.getName()) == 0) {
				if (f.getName().substring(this.getName().length()).matches("#\\d+#")) {
					f.delete();
				}
			}
		}
	}

	/**
	 * Zip Files-directory to mop
	 * 
	 * @param zipOutputStream
	 *            Outputstream
	 * @param directory
	 *            Directory to add
	 * @throws IOException
	 */
	private void zipFiles(ZipOutputStream zipOutputStream, File directory) throws IOException {
		File rel;
		String name;

		for (File file : directory.listFiles()) {
			if (file.isDirectory()) {
				this.zipFiles(zipOutputStream, file);
			} else {
				rel = FileUtils.getRelativePath(this.getFilesDirectrory(), file);
				name = "Files/" + rel.getPath().replace(File.separator, "/");

				zipOutputStream.putNextEntry(new ZipEntry(name));

				InputStream inputStream = new BufferedInputStream(new FileInputStream(file));
				FileUtils.copy(inputStream, zipOutputStream);
				inputStream.close();
			}
		}
	}

	/**
	 * Test if file is a valid mop-file
	 * 
	 * @return Is valid mop-file
	 */
	public boolean isValid() {
		boolean found = false;

		if (this.exists()) {
			ZipFile zf;
			try {

				zf = new ZipFile(this);
				Enumeration<? extends ZipEntry> entries = zf.entries();
				while (entries.hasMoreElements()) {
					if (entries.nextElement().getName().compareToIgnoreCase("project.xml") == 0) {
						found = true;
					}
				}
				zf.close();

			} catch (Exception e) {
			}
		}

		return found;
	}

	/**
	 * MopAccess class for file operations (write/read file)
	 */
	private class FileAccess implements MopAccess {

		/**
		 * File for operation (read/write)
		 */
		File file = null;

		/**
		 * @param file_
		 */
		FileAccess(File file_) {
			this.file = file_;
		}

		/**
		 * Read from stream and write to file
		 * 
		 * @param is
		 *            InputStream
		 * @throws Exception
		 */
		@Override
		public void readStream(InputStream is) throws Exception {
			OutputStream os = new BufferedOutputStream(new FileOutputStream(this.file));
			FileUtils.copy(is, os);
			os.close();
		}

		/**
		 * Read from file and write to stream
		 * 
		 * @param os
		 *            OutputStream
		 * @throws Exception
		 */
		@Override
		public void writeStream(OutputStream os) throws Exception {
			InputStream is = new BufferedInputStream(new FileInputStream(this.file));
			FileUtils.copy(is, os);
			is.close();
		}
	};

	/**
	 * Copy a local file into zip
	 * 
	 * @param file
	 *            Local File
	 * @param name
	 *            Filename in Archive
	 * @throws Exception
	 */
	public void writeFile(File file, String name) throws Exception {
		this.writeStream(new FileAccess(file), name);
	}

	/**
	 * Read to local file
	 * 
	 * @param file
	 *            Local File
	 * @param name
	 *            Filename in Archive
	 * @throws Exception
	 */
	public void readFile(File file, String name) throws Exception {
		this.readStream(new FileAccess(file), name);
	}

	/**
	 * Remove from zip
	 * 
	 * @param filename
	 * @throws Exception
	 */
	public void remove(String filename) throws Exception {
		this.writeStream(null, filename);
	}

	/**
	 * Get information about the component
	 * 
	 * @return Component information
	 */
	public ComponentInfo getComponentInfo() {
		final ComponentInfo info;

		if (this.isComponent()) {
			info = new ComponentInfo(this);
			try {
				this.readStream(new MopAccess() {

					@Override
					public void writeStream(OutputStream os) throws Exception {

					}

					@Override
					public void readStream(InputStream is) throws Exception {
						info.getProperties().load(is);
					}
				}, "Component.info");
			} catch (Exception e) {
				// e.printStackTrace();
			}
		} else {
			info = null;
		}
		return info;
	}

	@Override
	public boolean delete() {
		File dir;
		boolean del = false;

		// files
		dir = this.getFilesDirectrory();
		if (dir.exists()) {
			FileUtils.delete(dir);
		}

		// dataset
		dir = this.getDataDirectrory();
		if (dir.exists()) {
			FileUtils.delete(dir);
		}

		// mop
		if (this.exists()) {
			del = super.delete();
		}

		return del;
	}

	/**
	 * Get files-directory
	 * 
	 * @return Files-directory
	 */
	public File getFilesDirectrory() {
		return new File(this.getPath() + MopFile.filesExtention);

	}

	/**
	 * Get dataset directory
	 * 
	 * @return Dataset directory
	 */
	public File getDataDirectrory() {
		return new File(this.getPath() + MopFile.dataExtention);
	}

	/**
	 * Is MopFile a component
	 * 
	 * @return Is component
	 */
	public boolean isComponent() {
		return this.getName().toLowerCase().endsWith(".mopc");
	}

	/**
	 * Unzip all component files
	 * 
	 * @throws Exception
	 */
	public void expandComponent() throws Exception {
		File folder, f;
		String relPath;

		if (this.isComponent()) {
			folder = this.getFilesDirectrory();
			if (folder.exists() && (folder.lastModified() < this.lastModified())) {
				FileUtils.delete(folder);
			}

			if (!folder.exists()) {
				for (String path : this.getFileNames()) {
					// component files
					if (path.startsWith(MopFile.componentTag)) {
						relPath = path.substring(MopFile.componentTag.length());
						f = new File(folder, relPath);
						f.getParentFile().mkdirs();
						this.readFile(f, path);
					}
				}
			}

		}
	}
}
