/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 247 $
 * $Date: 2020-03-19 10:13:52 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.optim.gui.setting;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.io.File;
import java.util.Arrays;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import at.lcm.bu21.optim.core.Archive;
import at.lcm.bu21.optim.core.Problem;
import magopt.utils.MopFile;

/**
 * Panel to select problem
 * 
 * @author Gnther Weidenholzer
 * @author Werner Koppelsttter
 */
public class ChooseProblem extends SettingsPanel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Path of selected project */
	private String projectPath = "";

	/** Name of selected archive */
	private String archiveName = "";

	/** Name of selected problem */
	private String problemName = "";

	/** Selected archive */
	private Archive archive = null;

	/** Selected problem */
	private Problem problem = null;

	/** Selected mop-file */
	private MopFile mop = null;

	/** Initial path for selecting mop */
	private File initialPath = null;

	/** Refresh panel */
	private PanelRefresh refresh = null;

	/** Panel enabled */
	private boolean enable = true;

	/** File-filter for mop-files */
	FileFilter mopFilter = new FileNameExtensionFilter("Project Files (*.mop)", "mop");

	/** File-filter for settings-files */
	FileFilter settingFilter = new FileNameExtensionFilter("Settings Files (*.xml)", "xml");

	/** Textfield for selected mop-file */
	private JTextField textField_mop;

	/** Button to select mop-file */
	private JButton button_chooseMop;

	/** Panel for archives */
	private JPanel panelArchives;

	/** List archives */
	private JList<String> listArchive;

	/** Button to add archive */
	private JButton buttonAddArchive;

	/** Button to remove archive */
	private JButton buttonRmArchive;

	/** Button to duplicate archive */
	private JButton buttonDuplicateArchive;

	/** Button to rename archive */
	private JButton buttonRenameArchive;

	/** Panel for problems */
	private JPanel panelProblems;

	/** List problems */
	private JList<String> listProblem;

	/** Button to add problem */
	private JButton buttonAddProblem;

	/** Button to remove problem */
	private JButton buttonRmProblem;

	/** Button to duplicate problem */
	private JButton buttonDuplicateProblem;

	/** Button to rename problem */
	private JButton buttonRenameProblem;

	/** Button to export problem settings */
	private JButton buttonExportSettings;

	/** Button to import problem settings */
	private JButton buttonImportSettings;

	/** Button to reduce archive */
	private JButton buttonReduceArchive;

	/** Unused button */
	private JButton buttonNewButton;

	/**
	 * Validate problem
	 * 
	 * @param prob
	 *            Problem
	 * 
	 * @return Message if not valid
	 */
	public static String validate(Problem prob) {
		String message = null;

		if (prob == null) {
			message = "Select a problem";
		}

		return message;
	}

	/**
	 * Create panel. Used by WindowsBuilder
	 */
	private ChooseProblem() {
		this.setBorder(new EmptyBorder(5, 5, 5, 5));
		this.initComponents();

		this.buttonAddArchive.setEnabled(false);
		this.buttonRmArchive.setEnabled(false);
		this.buttonDuplicateArchive.setEnabled(false);
		this.buttonAddProblem.setEnabled(false);
		this.buttonRmProblem.setEnabled(false);
		this.buttonDuplicateProblem.setEnabled(false);

		this.listArchive.setModel(new DefaultListModel<String>());
		this.listProblem.setModel(new DefaultListModel<String>());

	}

	/**
	 * Create panel to choose problem
	 * 
	 * @param path
	 *            Initial path
	 * @param refresh
	 *            Refresh panel
	 */
	public ChooseProblem(String path, PanelRefresh refresh) {
		this(path);
		this.refresh = refresh;
	}

	/**
	 * Create panel to choose problem
	 * 
	 * @param path
	 *            Initial path
	 */
	public ChooseProblem(String path) {
		this(new File(path));
	}

	/**
	 * Create panel to choose problem
	 * 
	 * @param mop
	 *            Defined mop-file
	 * @param refresh
	 *            Refresh panel
	 */
	public ChooseProblem(MopFile mop, PanelRefresh refresh) {
		this(mop);
		this.refresh = refresh;
	}

	/**
	 * Create panel to choose problem
	 * 
	 * @param mop
	 *            Defined mop-file
	 */
	public ChooseProblem(MopFile mop) {
		this(mop.getParentFile());
		this.setMopFile(mop.getPath());
	}

	/**
	 * Create panel to choose problem
	 * 
	 * @param path
	 *            Initial path
	 * @param refresh
	 *            Refresh panel
	 */
	public ChooseProblem(File path, PanelRefresh refresh) {
		this(path);
		this.refresh = refresh;
	}

	/**
	 * Create panel to choose problem
	 * 
	 * @param path
	 *            Initial path
	 */
	public ChooseProblem(File path) {
		this();

		this.initialPath = path;
		this.refresh();
	}

	/**
	 * Create panel to choose problem
	 * 
	 * @param problem
	 *            Initial problem
	 * @param refresh
	 *            Refresh panel
	 */
	public ChooseProblem(Problem problem, PanelRefresh refresh) {
		this(problem);
		this.refresh = refresh;
	}

	/**
	 * Create panel to choose problem
	 * 
	 * @param problem
	 *            Initial problem
	 */
	public ChooseProblem(Problem problem) {
		this(problem.getArchive().getMop().getParentFile());

		if (problem != null) {
			this.projectPath = problem.getArchive().getMop().getPath();
			this.mop = problem.getArchive().getMop();
			this.archiveName = problem.getArchive().getName();
			this.archive = problem.getArchive();
			this.problemName = problem.getName();
			this.problem = problem;
		}

		this.refresh();
	}

	/**
	 * Refresh panel
	 */
	protected void refresh() {
		if (this.refresh != null) {
			this.refresh.refresh();
		}

		this.refreshMop();
		this.refreshArchive();
		this.refreshProblem();

		this.refreshEnabled();
	}

	/** Refresh selected project */
	private void refreshMop() {
		DefaultListModel<String> model;
		String[] list;

		ChooseProblem.this.textField_mop.setText(this.projectPath);

		model = (DefaultListModel<String>) this.listArchive.getModel();
		model.clear();
		if (this.mop != null) {
			try {
				list = Archive.listArchives(this.mop);
				Arrays.sort(list);
				for (String archive : list) {
					model.addElement(archive);
				}
			} catch (Exception e) {
			}
		}

	}

	/** Refresh selected archive */
	private void refreshArchive() {
		DefaultListModel<String> model;
		String[] list;

		this.listArchive.clearSelection();
		this.listArchive.setSelectedValue(this.archiveName, true);

		model = (DefaultListModel<String>) this.listProblem.getModel();
		model.clear();
		if (this.archive != null) {
			try {
				list = this.archive.listProblems();
				Arrays.sort(list);
				for (String archive : list) {
					model.addElement(archive);
				}
			} catch (Exception e) {
			}
		}
	}

	/** Refresh selected problem */
	private void refreshProblem() {
		this.listProblem.clearSelection();
		this.listProblem.setSelectedValue(this.problemName, true);
	}

	@Override
	public String validateContents() {
		String message = null;

		if (this.getMopFile() == null) {
			message = "Specify a valid project file!";
		} else if (this.getArchive() == null) {
			message = "Select a archive";
		} else if (this.getProblem() == null) {
			message = ChooseProblem.validate(this.getProblem());
		}

		return message;
	}

	/**
	 * Are settings enabled
	 * 
	 * @return Setting enable
	 */
	public boolean isSettingEnabled() {
		return this.enable;
	}

	/**
	 * Set settings enabled
	 * 
	 * @param enable
	 *            Settings enable
	 */
	public void setSettingEnabled(boolean enable) {
		this.enable = enable;
		this.refreshEnabled();
	}

	/**
	 * Refresh enable state of components
	 */
	private void refreshEnabled() {
		boolean en;

		this.textField_mop.setEnabled(this.enable);
		this.button_chooseMop.setEnabled(this.enable);

		this.listArchive.setEnabled(this.enable);
		en = this.enable && (this.mop != null);
		this.buttonAddArchive.setEnabled(en);
		en = en && (this.archive != null);
		this.buttonRmArchive.setEnabled(en);
		this.buttonDuplicateArchive.setEnabled(en);
		this.buttonDuplicateArchive.setEnabled(en);
		this.buttonRenameArchive.setEnabled(en);

		this.listProblem.setEnabled(this.enable);
		this.buttonAddProblem.setEnabled(en);
		this.buttonImportSettings.setEnabled(en);
		en = en && (this.problem != null);
		this.buttonRmProblem.setEnabled(en);
		this.buttonDuplicateProblem.setEnabled(en);
		this.buttonRenameProblem.setEnabled(en);
		this.buttonExportSettings.setEnabled(en);
		this.buttonReduceArchive.setEnabled(en);

		this.buttonNewButton.setEnabled(false);
	}

	/**
	 * Get selected mop-file
	 * 
	 * @return Mop-file
	 */
	public MopFile getMopFile() {
		return this.mop;
	}

	/**
	 * Get selected archive
	 * 
	 * @return Archive
	 */
	public Archive getArchive() {
		return this.archive;
	}

	/**
	 * Get selected problem
	 * 
	 * @return Problem
	 */
	public Problem getProblem() {
		return this.problem;
	}

	/**
	 * Initialize components
	 */
	private void initComponents() {
		JScrollPane jScrollArchive, jScrollProblem;
		JLabel jLabelProject, labArchive, labProblem;
		JPanel panelMop, panelArchiveButtons, panelProblemButtons, panelCenter;

		this.setLayout(new BorderLayout(5, 5));

		panelMop = new JPanel();
		this.add(panelMop, BorderLayout.NORTH);
		panelMop.setLayout(new BorderLayout(5, 5));
		panelMop.setBorder(new EmptyBorder(5, 5, 5, 5));

		jLabelProject = new JLabel();
		panelMop.add(jLabelProject, BorderLayout.WEST);
		jLabelProject.setText("Project File:");

		this.textField_mop = new JTextField();
		panelMop.add(this.textField_mop);
		this.textField_mop.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				ChooseProblem.this.setMopFile(ChooseProblem.this.textField_mop.getText());
			}
		});
		this.textField_mop.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent evt) {
				ChooseProblem.this.setMopFile(ChooseProblem.this.textField_mop.getText());
			}
		});

		this.button_chooseMop = new JButton();
		panelMop.add(this.button_chooseMop, BorderLayout.EAST);
		this.button_chooseMop.setText("...");
		this.button_chooseMop.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				JFileChooser chooser;
				int returnVal;

				chooser = new JFileChooser();
				if (ChooseProblem.this.initialPath != null) {
					chooser.setCurrentDirectory(ChooseProblem.this.initialPath);
				}
				chooser.setFileFilter(ChooseProblem.this.mopFilter);

				returnVal = chooser.showOpenDialog(ChooseProblem.this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					ChooseProblem.this.setMopFile(chooser.getSelectedFile().getAbsolutePath());
				}
			}
		});

		panelCenter = new JPanel();
		this.add(panelCenter, BorderLayout.CENTER);
		panelCenter.setLayout(new GridLayout(0, 2, 0, 0));

		this.panelArchives = new JPanel();
		panelCenter.add(this.panelArchives);
		this.panelArchives.setBorder(new EmptyBorder(5, 5, 5, 3));
		this.panelArchives.setLayout(new BorderLayout(5, 5));

		labArchive = new JLabel();
		labArchive.setText("Archives");
		this.panelArchives.add(labArchive, BorderLayout.NORTH);

		jScrollArchive = new JScrollPane();
		this.panelArchives.add(jScrollArchive, BorderLayout.CENTER);
		jScrollArchive.setPreferredSize(new Dimension(0, 0));

		this.listArchive = new JList<String>();
		jScrollArchive.setViewportView(this.listArchive);
		this.listArchive.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.listArchive.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent evt) {
				String name = null;

				if (!ChooseProblem.this.listArchive.isSelectionEmpty()) {
					name = ChooseProblem.this.listArchive.getSelectedValue();
					ChooseProblem.this.setArchive(name);
				}
			}
		});

		panelArchiveButtons = new JPanel();
		this.panelArchives.add(panelArchiveButtons, BorderLayout.SOUTH);
		panelArchiveButtons.setLayout(new GridLayout(3, 2, 5, 5));

		this.buttonAddArchive = new JButton("Add");
		panelArchiveButtons.add(this.buttonAddArchive);
		this.buttonAddArchive.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				String name;

				name = JOptionPane.showInputDialog(ChooseProblem.this, "Choose name of new archive:", "New archive",
						JOptionPane.QUESTION_MESSAGE);
				if ((name != null) && !name.isEmpty()) {
					try {
						if (!Arrays.asList(Archive.listArchives(ChooseProblem.this.mop)).contains(name)) {
							new Archive(ChooseProblem.this.mop, name).save();
							ChooseProblem.this.setArchive(name);
						}
					} catch (Exception e) {
					}
				}
			}
		});

		this.buttonRmArchive = new JButton("Remove");
		panelArchiveButtons.add(this.buttonRmArchive);
		this.buttonRmArchive.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				if (JOptionPane.showConfirmDialog(ChooseProblem.this,
						"Do you really want to remove archive '" + ChooseProblem.this.archiveName + "'?",
						"Remove archive", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == 0) {
					try {
						Archive.delete(ChooseProblem.this.mop, ChooseProblem.this.archiveName);
						ChooseProblem.this.setArchive(null);
					} catch (Exception e) {
					}
				}
				ChooseProblem.this.listArchive.requestFocus();
			}
		});

		this.buttonDuplicateArchive = new JButton("Duplicate");
		panelArchiveButtons.add(this.buttonDuplicateArchive);
		this.buttonDuplicateArchive.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				String name;

				name = JOptionPane.showInputDialog(ChooseProblem.this, "Choose name of new archive:",
						"Duplicate archive", JOptionPane.QUESTION_MESSAGE);
				if ((name != null) && !name.isEmpty()) {
					try {
						ChooseProblem.this.archive.duplicate(name);
						ChooseProblem.this.setArchive(name);
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}
		});

		this.buttonRenameArchive = new JButton("Rename");
		panelArchiveButtons.add(this.buttonRenameArchive);
		this.buttonRenameArchive.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				String name;

				name = JOptionPane.showInputDialog(ChooseProblem.this, "Choose new name of archive:", "Rename archive",
						JOptionPane.QUESTION_MESSAGE);
				if ((name != null) && !name.isEmpty()) {
					try {
						ChooseProblem.this.archive.rename(name);
						ChooseProblem.this.setArchive(ChooseProblem.this.archive.getName());
					} catch (Exception e) {
					}
				}
			}
		});

		this.buttonNewButton = new JButton("New button");
		panelArchiveButtons.add(this.buttonNewButton);
		this.buttonNewButton.setVisible(false);
		this.buttonNewButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent event) {
			}
		});

		this.buttonReduceArchive = new JButton("Reduce Archive");
		panelArchiveButtons.add(this.buttonReduceArchive);
		this.buttonReduceArchive.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				String name, prob, str;
				int num = -1;

				name = JOptionPane.showInputDialog(ChooseProblem.this, "Choose name of new archive:", "Reduce archive",
						JOptionPane.QUESTION_MESSAGE);
				if ((name != null) && !name.isEmpty()) {
					str = JOptionPane.showInputDialog(ChooseProblem.this, "Choose number of pareto-fronts:",
							"Reduce archive", JOptionPane.QUESTION_MESSAGE);
					try {
						num = Integer.parseInt(str);
					} catch (Exception ex) {
						ex.printStackTrace();
					}
					if (num != -1) {
						prob = ChooseProblem.this.problem.getName();
						try {
							ChooseProblem.this.problem.reduceArchive(name, num);
							ChooseProblem.this.setArchive(name);
							ChooseProblem.this.setProblem(prob);
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
				}
			}
		});

		this.panelProblems = new JPanel();
		panelCenter.add(this.panelProblems);
		this.panelProblems.setLayout(new BorderLayout(5, 5));
		this.panelProblems.setBorder(new EmptyBorder(5, 3, 5, 5));

		labProblem = new JLabel();
		this.panelProblems.add(labProblem, BorderLayout.NORTH);
		labProblem.setText("Problems");

		jScrollProblem = new JScrollPane();
		this.panelProblems.add(jScrollProblem, BorderLayout.CENTER);
		jScrollProblem.setPreferredSize(new Dimension(0, 0));

		this.listProblem = new JList<String>();
		jScrollProblem.setViewportView(this.listProblem);
		this.listProblem.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.listProblem.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent evt) {
				String name = null;

				if (!ChooseProblem.this.listProblem.isSelectionEmpty()) {
					name = ChooseProblem.this.listProblem.getSelectedValue();
					ChooseProblem.this.setProblem(name);
				}
			}
		});

		panelProblemButtons = new JPanel();
		this.panelProblems.add(panelProblemButtons, BorderLayout.SOUTH);
		panelProblemButtons.setLayout(new GridLayout(3, 2, 5, 5));

		this.buttonAddProblem = new JButton("Add");
		panelProblemButtons.add(this.buttonAddProblem);
		this.buttonAddProblem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				String name;

				name = JOptionPane.showInputDialog(ChooseProblem.this, "Choose name of new problem:", "New problem",
						JOptionPane.QUESTION_MESSAGE);
				if ((name != null) && !name.isEmpty()) {
					try {
						ChooseProblem.this.archive.createProblem(name);
						// save changes
						ChooseProblem.this.archive.save();
						ChooseProblem.this.setProblem(name);
					} catch (Exception e) {
					}
				}
			}
		});

		this.buttonRmProblem = new JButton("Remove");
		panelProblemButtons.add(this.buttonRmProblem);
		this.buttonRmProblem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				if (JOptionPane.showConfirmDialog(ChooseProblem.this,
						"Do you really want to remove problem '" + ChooseProblem.this.problemName + "'?",
						"Remove problem", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == 0) {
					try {
						ChooseProblem.this.archive.deleteProblem(ChooseProblem.this.problemName);
						// save changes
						ChooseProblem.this.archive.save();
						ChooseProblem.this.setProblem(null);
					} catch (Exception e) {
					}
				}
				ChooseProblem.this.listProblem.requestFocus();
			}
		});

		this.buttonDuplicateProblem = new JButton("Duplicate");
		panelProblemButtons.add(this.buttonDuplicateProblem);
		this.buttonDuplicateProblem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				String name;

				name = JOptionPane.showInputDialog(ChooseProblem.this, "Choose name of new problem:",
						"Duplicate problem", JOptionPane.QUESTION_MESSAGE);
				if ((name != null) && !name.isEmpty()) {
					try {
						if (ChooseProblem.this.problem != null) {
							ChooseProblem.this.problem.duplicate(name);
							// save changes
							ChooseProblem.this.problem.getArchive().save();
							ChooseProblem.this.setProblem(name);
						}
					} catch (Exception e) {
					}
				}
			}
		});

		this.buttonRenameProblem = new JButton("Rename");
		panelProblemButtons.add(this.buttonRenameProblem);
		this.buttonRenameProblem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				String name;

				name = JOptionPane.showInputDialog(ChooseProblem.this, "Choose new name of problem:", "Rename problem",
						JOptionPane.QUESTION_MESSAGE);
				if ((name != null) && !name.isEmpty()) {
					try {
						ChooseProblem.this.problem.rename(name);
						// save changes
						ChooseProblem.this.problem.getArchive().save();
						ChooseProblem.this.setProblem(ChooseProblem.this.problem.getName());
					} catch (Exception e) {
						name = null;
					}
				}
			}
		});

		this.buttonExportSettings = new JButton("Export Settings");
		panelProblemButtons.add(this.buttonExportSettings);
		this.buttonExportSettings.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				JFileChooser chooser;
				int returnVal;

				chooser = new JFileChooser();
				chooser.setCurrentDirectory(ChooseProblem.this.mop.getParentFile());
				chooser.setFileFilter(ChooseProblem.this.settingFilter);

				returnVal = chooser.showSaveDialog(ChooseProblem.this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {

					if (ChooseProblem.this.problem != null) {
						ChooseProblem.this.problem.exportSettings(chooser.getSelectedFile());
					}
				}
			}
		});

		this.buttonImportSettings = new JButton("Import Settings");
		panelProblemButtons.add(this.buttonImportSettings);
		this.buttonImportSettings.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				JFileChooser chooser;
				int returnVal;
				String name, message = null;

				// choose problem name
				name = JOptionPane.showInputDialog(ChooseProblem.this, "Choose name of new problem:", "Import settings",
						JOptionPane.QUESTION_MESSAGE);
				if ((name != null) && !name.isEmpty()) {
					// choose file
					chooser = new JFileChooser();
					chooser.setCurrentDirectory(ChooseProblem.this.mop.getParentFile());
					chooser.setFileFilter(ChooseProblem.this.settingFilter);
					returnVal = chooser.showOpenDialog(ChooseProblem.this);
					if (returnVal == JFileChooser.APPROVE_OPTION) {

						try {
							message = ChooseProblem.this.archive.importProblemSettings(name, chooser.getSelectedFile());
							ChooseProblem.this.setProblem(name);
						} catch (Exception ex) {
							ex.printStackTrace();
							JOptionPane.showMessageDialog(ChooseProblem.this, ex.getMessage(), "Import error",
									JOptionPane.ERROR_MESSAGE);
						}

						if (message != null) {
							JOptionPane.showMessageDialog(ChooseProblem.this, message, "Import warning",
									JOptionPane.WARNING_MESSAGE);
						}
					}
				}
			}
		});
	}

	/**
	 * Set path of project
	 * 
	 * @param path
	 *            Project path
	 */
	protected void setMopFile(String path) {
		MopFile mop;

		if (path == null) {
			path = "";
		}

		if (this.projectPath.compareToIgnoreCase(path) != 0) {
			this.projectPath = path;
			this.archiveName = "";
			this.problemName = "";
			this.mop = null;
			this.archive = null;
			this.problem = null;

			mop = new MopFile(this.projectPath);
			if (mop.exists() && mop.isFile() && this.mopFilter.accept(mop) && mop.isValid()) {
				this.mop = mop;
			}

			this.refresh();
		}
	}

	/**
	 * Set name of archive
	 * 
	 * @param name
	 *            Archive name
	 */
	protected void setArchive(String name) {
		if (name == null) {
			name = "";
		}

		if ((this.mop != null) && (this.archiveName.compareToIgnoreCase(name) != 0)) {
			this.archiveName = name;
			this.problemName = "";
			this.archive = null;
			this.problem = null;

			try {
				this.archive = Archive.load(this.mop, name);
			} catch (Exception e) {
			}

			this.refresh();
		}
	}

	/**
	 * Set name of problem
	 * 
	 * @param name
	 *            Problem name
	 */
	protected void setProblem(String name) {
		if (name == null) {
			name = "";
		}

		if ((this.archive != null) && (this.problemName.compareToIgnoreCase(name) != 0)) {
			this.problemName = name;
			this.problem = null;

			try {
				this.problem = this.archive.readProblem(name);
			} catch (Exception e) {
			}

			this.refresh();
		}
	}

}
