/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 247 $
 * $Date: 2020-03-19 10:13:52 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.optim.gui.setting;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;

import javax.swing.DefaultBoundedRangeModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.LayoutStyle;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import at.lcm.bu21.optim.algorithm.GenerationBased;
import at.lcm.bu21.optim.algorithm.SolverList;
import at.lcm.bu21.optim.core.Problem;

/**
 * Panel to change general-settings
 * 
 * @author Gnther Weidenholzer
 * @author Werner Koppelsttter
 */
public class GeneralSettings extends SettingsPanel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Problem of the setting */
	private Problem problem = null;

	/** Refresh panel */
	private PanelRefresh refresh = null;

	/** Panel enabled */
	private boolean enable = true;

	/** Slider for poolsize */
	private JSlider sliderPoolsize;

	/** Text-field for poolsize */
	private JTextField txtPoolsize;

	/** Slider for function-timeout */
	private JSlider sliderFunctionTimeout;

	/** Text-field for function-timeout */
	private JTextField txtFunctionTimeout;

	/** Combo-box to select solver */
	private JComboBox<SolverList> comboSolver;

	/** Check-box for initialize solver with memory */
	private JCheckBox cbInitSolver;

	/** Check-box for reattempt individuals with errors */
	private JCheckBox cbReattemptError;

	/** Solver for autosave-interval */
	private JSlider sliderAutosaveInterval;

	/** Text-field for autosave-interval */
	private JTextField txtAutosaveInterval;

	/** Check-box to enable autosave-time */
	private JCheckBox cbAutosaveTime;

	/** Slider for autosave-time */
	private JSlider sliderAutosaveTime;

	/** Text-field for autosave-time */
	private JTextField txtAutosaveTime;

	/** Label for unit of autosave-time */
	private JLabel labAutosaveTimeUnit;

	/**
	 * Change-event for slider with type of changes
	 * 
	 * @author Werner Koppelsttter
	 */
	private class SliderChangeEvent extends ChangeEvent {

		/** Serialization version number */
		private static final long serialVersionUID = 1L;

		/** Is value changed */
		boolean _valueChanged = false;

		/**
		 * Constructs a ChangeEvent object
		 * 
		 * @param source
		 *            Source of the event (typically this)
		 */
		public SliderChangeEvent(Object source) {
			super(source);
		}

		/**
		 * Constructs a ChangeEvent object
		 * 
		 * @param source
		 *            Source of the event (typically this)
		 * @param valueChanged
		 *            Is value of slide is changed
		 */
		public SliderChangeEvent(Object source, boolean valueChanged) {
			super(source);
			this._valueChanged = valueChanged;
		}

		/**
		 * Check if value is changed
		 * 
		 * @return Is value of slide is changed
		 */
		public boolean isValueChanged() {
			return this._valueChanged;
		}
	}

	/**
	 * Bounded-range-model for slider with change-event with the type of changes
	 * 
	 * @author Werner Koppelsttter
	 */
	private class SliderDefaultBoundedRangeModel extends DefaultBoundedRangeModel {

		/** Serialization version number */
		private static final long serialVersionUID = 1L;

		@Override
		public void setRangeProperties(int newValue, int newExtent, int newMin, int newMax, boolean adjusting) {
			// generate new change event with change information
			this.changeEvent = new SliderChangeEvent(this, this.getValue() != newValue);
			super.setRangeProperties(newValue, newExtent, newMin, newMax, adjusting);
		}
	}

	/**
	 * Validate problem
	 * 
	 * @param prob
	 *            Problem
	 * 
	 * @return Message if not valid
	 */
	public static String validate(Problem prob) {
		String message = null;

		if (prob.getSolver() == null) {
			message = "Please complete settings";
		} else {
			message = prob.getSolver().validateProblem();
		}

		return message;
	}

	/**
	 * Create panel. Used by WindowsBuilder
	 */
	private GeneralSettings() {
		this.initComponents();
	}

	/**
	 * Creates new form general-settings
	 * 
	 * @param problem
	 *            Problem to set general-settings
	 * @param refresh
	 *            Refresh panel
	 */
	public GeneralSettings(Problem problem, PanelRefresh refresh) {
		this(problem);
		this.refresh = refresh;
	}

	/**
	 * Creates new form general-settings
	 * 
	 * @param problem
	 *            Problem to set general-settings
	 */
	public GeneralSettings(Problem problem) {
		this();
		this.problem = problem;

		// poolsize
		this.refreshPoolSize();

		// function timeout
		this.refreshFunctionTimeout();

		// solver
		this.comboSolver.setSelectedItem(SolverList.getSolverListEntry(this.problem.getSolver().getOptimizerName()));
		this.setSolver();

		// init solver
		this.cbInitSolver.setSelected(problem.getInitSolver());

		// reattempt error
		this.cbReattemptError.setSelected(problem.getReattemptError());

		// autosave interval
		this.refreshAutosaveInterval();

		// autosave time
		this.cbAutosaveTime.setSelected(this.problem.getAutosaveTimeEnabled());
		this.refreshAutosaveTime();

		this.refreshEnabled();
	}

	/**
	 * Refresh panel
	 */
	protected void refresh() {
		if (this.refresh != null) {
			this.refresh.refresh();
		}
	}

	@Override
	public String validateContents() {
		return GeneralSettings.validate(this.getProblem());
	}

	/**
	 * Are settings enabled
	 * 
	 * @return Setting enable
	 */
	public boolean isSettingEnabled() {
		return this.enable;
	}

	/**
	 * Set settings enabled
	 * 
	 * @param enable
	 *            Settings enable
	 */
	public void setSettingEnabled(boolean enable) {
		this.enable = enable;
		this.refreshEnabled();
	}

	/**
	 * Refresh enable state of components
	 */
	private void refreshEnabled() {
		boolean en;

		// pool size
		en = this.enable;
		this.sliderPoolsize.setEnabled(en);
		this.txtPoolsize.setEditable(en);

		// solver
		en = this.enable && !this.problem.isLocked();
		this.comboSolver.setEnabled(en);

		// init solver
		en = this.enable && !this.problem.isLocked();
		this.cbInitSolver.setEnabled(en);

		// autosave interval
		en = (this.problem.getSolver() != null) && !(this.problem.getSolver() instanceof GenerationBased);
		this.sliderAutosaveInterval.setEnabled(en);
		this.txtAutosaveInterval.setEnabled(en);

		// autosave time
		en = this.problem.getAutosaveTimeEnabled();
		this.sliderAutosaveTime.setEnabled(en);
		this.txtAutosaveTime.setEnabled(en);
		this.labAutosaveTimeUnit.setEnabled(en);
	}

	/**
	 * Set value of pool-size and refresh display
	 */
	private void takePoolSizeTxt() {
		try {
			int value = Integer.parseInt(GeneralSettings.this.txtPoolsize.getText());
			if (value != GeneralSettings.this.problem.getPoolSize()) {
				GeneralSettings.this.problem.setPoolSize(value);
				GeneralSettings.this.refreshPoolSize();
			}
		} catch (Exception e) {
		}
		GeneralSettings.this.refreshPoolSize();
	}

	/**
	 * Refresh value of pool-size
	 */
	private void refreshPoolSize() {
		this.txtPoolsize.setText(Integer.toString(this.problem.getPoolSize()));
		// adjust upper boundary of slider (maximum)
		if (this.sliderPoolsize.getMaximum() < this.problem.getPoolSize()) {
			this.sliderPoolsize.setMaximum((((this.problem.getPoolSize() - 1) / 50) + 1) * 50);
		}
		this.sliderPoolsize.setValue(this.problem.getPoolSize());
	}

	/**
	 * Set value of function-timeout and refresh display
	 */
	private void takeFunctionTimeoutTxt() {
		try {
			int value = Integer.parseInt(GeneralSettings.this.txtFunctionTimeout.getText());
			if (value != GeneralSettings.this.problem.getFunctionTimeout()) {
				GeneralSettings.this.problem.setFunctionTimeout(value);
				GeneralSettings.this.refreshFunctionTimeout();
			}
		} catch (Exception e) {
		}
	}

	/**
	 * Refresh value of function-timeout
	 */
	private void refreshFunctionTimeout() {
		this.txtFunctionTimeout.setText(Integer.toString(this.problem.getFunctionTimeout()));
		// adjust upper boundary of slider (maximum)
		if (this.sliderFunctionTimeout.getMaximum() < this.problem.getFunctionTimeout()) {
			this.sliderFunctionTimeout.setMaximum((((this.problem.getFunctionTimeout() - 1) / 60) + 1) * 60);
		}
		this.sliderFunctionTimeout.setValue(this.problem.getFunctionTimeout());
	}

	/**
	 * Set value of solver
	 */
	private void setSolver() {
		SolverList solver;

		solver = (SolverList) this.comboSolver.getSelectedItem();
		if (this.problem.getSolver().getOptimizerName().compareTo(solver.getName()) != 0) {
			try {
				this.problem.setSolver(solver.getSolver(this.problem));
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}
		this.setSettingEnabled(this.enable);

		this.refresh();
	}

	/**
	 * Set value of autosave-interval and refresh display
	 */
	private void takeAutosaveIntervalTxt() {
		try {
			int value = Integer.parseInt(GeneralSettings.this.txtAutosaveInterval.getText());
			if (value != GeneralSettings.this.problem.getAutosaveInterval()) {
				GeneralSettings.this.problem.setAutosaveInterval(value);
				GeneralSettings.this.refreshAutosaveInterval();
			}
		} catch (Exception e) {
		}
	}

	/**
	 * Refresh value of autosave-interval
	 */
	private void refreshAutosaveInterval() {
		this.txtAutosaveInterval.setText(Integer.toString(this.problem.getAutosaveInterval()));
		// adjust upper boundary of slider (maximum)
		if (this.sliderAutosaveInterval.getMaximum() < this.problem.getAutosaveInterval()) {
			this.sliderAutosaveInterval.setMaximum((((this.problem.getAutosaveInterval() - 1) / 50) + 1) * 50);
		}
		this.sliderAutosaveInterval.setValue(this.problem.getAutosaveInterval());
	}

	/**
	 * Set value of init solver
	 */
	private void setInitSolver() {
		this.problem.setInitSolver(this.cbInitSolver.isSelected());
		this.cbInitSolver.setSelected(this.problem.getInitSolver());
	}

	/**
	 * Set value of reattempt on error
	 */
	private void setReattemptError() {
		this.problem.setReattemptError(this.cbReattemptError.isSelected());
		this.cbReattemptError.setSelected(this.problem.getReattemptError());
	}

	/**
	 * Set value of autosave time enabled
	 */
	private void setAutosaveTimeEnabled() {
		this.problem.setAutosaveTimeEnabled(this.cbAutosaveTime.isSelected());
		this.cbAutosaveTime.setSelected(this.problem.getAutosaveTimeEnabled());
		this.setSettingEnabled(this.enable);
	}

	/**
	 * Set value of autosave-time and refresh display
	 */
	private void takeAutosaveTimeTxt() {
		try {
			int value = Integer.parseInt(GeneralSettings.this.txtAutosaveTime.getText());
			if (value != GeneralSettings.this.problem.getAutosaveTime()) {
				GeneralSettings.this.problem.setAutosaveTime(value);
				GeneralSettings.this.refreshAutosaveTime();
			}
		} catch (Exception e) {
		}
	}

	/**
	 * Refresh value of autosave-time
	 */
	private void refreshAutosaveTime() {
		this.txtAutosaveTime.setText(Integer.toString(this.problem.getAutosaveTime()));
		// adjust upper boundary of slider (maximum)
		if (this.sliderAutosaveTime.getMaximum() < this.problem.getAutosaveTime()) {
			this.sliderAutosaveTime.setMaximum((((this.problem.getAutosaveTime() - 1) / 60) + 1) * 60);
		}
		this.sliderAutosaveTime.setValue(this.problem.getAutosaveTime());
	}

	/**
	 * Get problem of panel
	 * 
	 * @return Problem
	 */
	public Problem getProblem() {
		return this.problem;
	}

	/**
	 * Initialize components
	 */
	private void initComponents() {
		JLabel labPoolsizeLabel, labPoolsizeUnit;
		JLabel labFunctionTimeoutLabel, labFunctionTimeoutUnit;
		JLabel labSolverLabel;
		JLabel labAutosaveIntervalLabel, labAutosaveIntervalUnit;

		// poolsize
		labPoolsizeLabel = new JLabel("Pool Size:");

		this.sliderPoolsize = new JSlider(new SliderDefaultBoundedRangeModel());
		this.sliderPoolsize.setMaximum(200);
		this.sliderPoolsize.setMinimum(1);
		this.sliderPoolsize.setValue(50);
		this.sliderPoolsize.getModel().addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent evt) {
				if ((evt instanceof SliderChangeEvent) && ((SliderChangeEvent) evt).isValueChanged()) {
					int value = GeneralSettings.this.sliderPoolsize.getValue();
					GeneralSettings.this.problem.setPoolSize(value);
					GeneralSettings.this.refreshPoolSize();
				}
			}
		});

		this.txtPoolsize = new JTextField("50");
		this.txtPoolsize.setPreferredSize(new Dimension(60, 20));
		this.txtPoolsize.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				GeneralSettings.this.takePoolSizeTxt();
			}
		});
		this.txtPoolsize.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent evt) {
				GeneralSettings.this.takePoolSizeTxt();
			}
		});

		labPoolsizeUnit = new JLabel("Individuals");

		// function timeout
		labFunctionTimeoutLabel = new JLabel("Function Timeout:");

		this.sliderFunctionTimeout = new JSlider(new SliderDefaultBoundedRangeModel());
		this.sliderFunctionTimeout.setMaximum(120);
		this.sliderFunctionTimeout.setMinimum(1);
		this.sliderFunctionTimeout.setValue(20);
		this.sliderFunctionTimeout.getModel().addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent evt) {
				if ((evt instanceof SliderChangeEvent) && ((SliderChangeEvent) evt).isValueChanged()) {
					int value = GeneralSettings.this.sliderFunctionTimeout.getValue();
					GeneralSettings.this.problem.setFunctionTimeout(value);
					GeneralSettings.this.refreshFunctionTimeout();
				}
			}
		});

		this.txtFunctionTimeout = new JTextField("20");
		this.txtFunctionTimeout.setPreferredSize(new Dimension(60, 20));
		this.txtFunctionTimeout.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				GeneralSettings.this.takeFunctionTimeoutTxt();
			}
		});
		this.txtFunctionTimeout.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent evt) {
				GeneralSettings.this.takeFunctionTimeoutTxt();
			}
		});

		labFunctionTimeoutUnit = new JLabel("min.");

		// optimizer
		labSolverLabel = new JLabel("Select Optimizer:");

		this.comboSolver = new JComboBox<SolverList>(new DefaultComboBoxModel<SolverList>(SolverList.getValues()));
		this.comboSolver.setMinimumSize(new Dimension(60, 20));
		this.comboSolver.setPreferredSize(new Dimension(60, 22));
		this.comboSolver.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				GeneralSettings.this.setSolver();
			}
		});

		// initialize solver
		this.cbInitSolver = new JCheckBox();
		this.cbInitSolver.setSelected(true);
		this.cbInitSolver.setText("Initialize empty solver");
		this.cbInitSolver.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				GeneralSettings.this.setInitSolver();
			}
		});

		// reattempt
		this.cbReattemptError = new JCheckBox("Reattempt individuals with error");
		this.cbReattemptError.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				GeneralSettings.this.setReattemptError();
			}
		});

		// autosave interval
		labAutosaveIntervalLabel = new JLabel("Autosave Interval:");

		this.sliderAutosaveInterval = new JSlider(new SliderDefaultBoundedRangeModel());
		this.sliderAutosaveInterval.setMaximum(500);
		this.sliderAutosaveInterval.setMinimum(1);
		this.sliderAutosaveInterval.setValue(100);
		this.sliderAutosaveInterval.getModel().addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent evt) {
				if ((evt instanceof SliderChangeEvent) && ((SliderChangeEvent) evt).isValueChanged()) {
					int value = GeneralSettings.this.sliderAutosaveInterval.getValue();
					GeneralSettings.this.problem.setAutosaveInterval(value);
					GeneralSettings.this.refreshAutosaveInterval();
				}
			}
		});

		this.txtAutosaveInterval = new JTextField("100");
		this.txtAutosaveInterval.setPreferredSize(new Dimension(60, 20));
		this.txtAutosaveInterval.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				GeneralSettings.this.takeAutosaveIntervalTxt();
			}
		});
		this.txtAutosaveInterval.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent evt) {
				GeneralSettings.this.takeAutosaveIntervalTxt();
			}
		});

		labAutosaveIntervalUnit = new JLabel("Individuals");

		// autosave time
		this.cbAutosaveTime = new JCheckBox("Periodic Autosave");
		this.cbAutosaveTime.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				GeneralSettings.this.setAutosaveTimeEnabled();
			}
		});

		this.sliderAutosaveTime = new JSlider(new SliderDefaultBoundedRangeModel());
		this.sliderAutosaveTime.setMaximum(300);
		this.sliderAutosaveTime.setMinimum(1);
		this.sliderAutosaveTime.setValue(60);
		this.sliderAutosaveTime.getModel().addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent evt) {
				if ((evt instanceof SliderChangeEvent) && ((SliderChangeEvent) evt).isValueChanged()) {
					int value = GeneralSettings.this.sliderAutosaveTime.getValue();
					GeneralSettings.this.problem.setAutosaveTime(value);
					GeneralSettings.this.refreshAutosaveTime();
				}
			}
		});

		this.txtAutosaveTime = new JTextField("60");
		this.txtAutosaveTime.setPreferredSize(new Dimension(60, 20));
		this.txtAutosaveTime.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				GeneralSettings.this.takeAutosaveTimeTxt();
			}
		});
		this.txtAutosaveTime.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent evt) {
				GeneralSettings.this.takeAutosaveTimeTxt();
			}
		});

		this.labAutosaveTimeUnit = new JLabel("min.");

		// layout
		GroupLayout layout = new GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createSequentialGroup().addContainerGap().addGroup(layout
				.createParallelGroup(GroupLayout.Alignment.LEADING).addComponent(labPoolsizeLabel)
				.addGroup(layout.createSequentialGroup()
						.addComponent(this.sliderPoolsize, GroupLayout.PREFERRED_SIZE, 241, GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
						.addComponent(this.txtPoolsize, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
								GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED).addComponent(labPoolsizeUnit))
				.addComponent(labFunctionTimeoutLabel)
				.addGroup(
						layout.createSequentialGroup()
								.addComponent(this.sliderFunctionTimeout, GroupLayout.PREFERRED_SIZE, 241,
										GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.txtFunctionTimeout, GroupLayout.PREFERRED_SIZE,
										GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED)
								.addComponent(labFunctionTimeoutUnit))
				.addGroup(layout.createSequentialGroup().addComponent(labSolverLabel).addGap(18, 18, 18)
						.addComponent(this.comboSolver, GroupLayout.PREFERRED_SIZE, 198, GroupLayout.PREFERRED_SIZE))
				.addComponent(this.cbInitSolver).addComponent(this.cbReattemptError)
				.addComponent(labAutosaveIntervalLabel)
				.addGroup(
						layout.createSequentialGroup()
								.addComponent(this.sliderAutosaveInterval, GroupLayout.PREFERRED_SIZE, 241,
										GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.txtAutosaveInterval, GroupLayout.PREFERRED_SIZE,
										GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED)
								.addComponent(labAutosaveIntervalUnit))
				.addComponent(this.cbAutosaveTime).addGroup(
						layout.createSequentialGroup()
								.addComponent(this.sliderAutosaveTime, GroupLayout.PREFERRED_SIZE, 241,
										GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
								.addComponent(this.txtAutosaveTime, GroupLayout.PREFERRED_SIZE,
										GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
								.addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED)
								.addComponent(this.labAutosaveTimeUnit)))
				.addContainerGap(GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createSequentialGroup().addContainerGap().addComponent(labPoolsizeLabel)
				.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(this.sliderPoolsize, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
								GroupLayout.PREFERRED_SIZE)
						.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
								.addComponent(this.txtPoolsize, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
										GroupLayout.PREFERRED_SIZE)
								.addComponent(labPoolsizeUnit)))
				.addGap(11, 11, 11).addComponent(labFunctionTimeoutLabel)
				.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(this.sliderFunctionTimeout, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
								GroupLayout.PREFERRED_SIZE)
						.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
								.addComponent(this.txtFunctionTimeout, GroupLayout.PREFERRED_SIZE,
										GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
								.addComponent(labFunctionTimeoutUnit)))
				.addGap(18, 18, 18)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING).addComponent(labSolverLabel)
						.addComponent(this.comboSolver, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
								GroupLayout.PREFERRED_SIZE))
				.addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED).addComponent(this.cbInitSolver)
				.addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED).addComponent(this.cbReattemptError)
				.addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED).addComponent(labAutosaveIntervalLabel)
				.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(this.sliderAutosaveInterval, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
								GroupLayout.PREFERRED_SIZE)
						.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
								.addComponent(this.txtAutosaveInterval, GroupLayout.PREFERRED_SIZE,
										GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
								.addComponent(labAutosaveIntervalUnit)))
				.addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED).addComponent(this.cbAutosaveTime)
				.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED)
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(this.sliderAutosaveTime, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
								GroupLayout.PREFERRED_SIZE)
						.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
								.addComponent(this.txtAutosaveTime, GroupLayout.PREFERRED_SIZE,
										GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
								.addComponent(this.labAutosaveTimeUnit)))
				.addContainerGap(33, Short.MAX_VALUE));
	}
}
