# -*- coding: utf-8 -*-
"""
Created on Thu Feb 25 19:07:27 2018

@author: mkitzberger
"""

import os

try:
    import java
    from jobmanager import JobDesc
    from jobmanager import JobType
    from jep import jarray

except Exception as e:
    print(e)


class SymJobDesc:
    """
    Python wrapper class for |sym|'s java class JobDesc.
    This wrapper provides a Python interface to the SymSpace
    Jobmanager.
    """

    @staticmethod
    def addFile(jobID, filename):
        """
        Add a file to job ID

        Parameters
        ----------
        jobID : str
            ID of the job
        filename : str
            Filename of the file to add

        """
        if not isinstance(jobID, str):
            raise ValueError('jobID must be a string')
        JobDesc.addFile(jobID, java.io.File(os.path.normpath(filename)))

    @staticmethod
    def linkFile(jobID, filename):
        """
        Link a file to job.

        Parameters
        ----------
        jobID : str
            ID of the job
        filename : str
            Filename of the file to add

        """
        JobDesc.linkFile(jobID, filename)

    @staticmethod
    def createJob(project, dataset, param, name, *args):
        """
        Create a job for submission to JobManager.

        Parameters
        ----------
        project : str
            A string visible in the project tab of |sym|'s jobmanager GUI.
        dataset : str
            A string visible in the dataset tab of |sym|'s jobmanager GUI.
        param : str
            A string visible in the param tab of |sym|'s  jobmanager GUI.
        name : str
            A string visible in the name tab of |sym|'s jobmanager GUI.
        args : List of strings
            Additional parameters. These parameters are forwarded to the Java
            starter class which is submitted to to JobManager for launching local
            jobs.

        """

        args = args[0]
        jarr = jarray(len(args), java.lang.String)
        for i in range(len(args)):
            if not isinstance(args[i], str):
                raise ValueError('All args must be strings')
            jarr[i] = java.lang.String(args[i])

        return str(JobDesc.createJob(project, dataset, param, name, jarr))

    @staticmethod
    def createJavaJob(project, dataset, param, name, *args):
        """
        Create a job for submission to JobManager.

        Parameters
        ----------
        project : str
            A string visible in the project tab of |sym|'s jobmanager GUI.
        dataset : str
            A string visible in the dataset tab of |sym|'s jobmanager GUI.
        param : str
            A string visible in the param tab of |sym|'s  jobmanager GUI.
        name : str
            A string visible in the name tab of |sym|'s jobmanager GUI.
        args : List of strings
            Additional parameters. These parameters are forwarded to the Java
            starter class which is submitted to to JobManager for launching local
            jobs.

        """

        args = args[0]
        jarr = jarray(len(args), java.lang.String)
        for i in range(len(args)):
            if not isinstance(args[i], str):
                raise ValueError('All args must be strings')
            jarr[i] = java.lang.String(args[i])

        return str(JobDesc.createJavaJob(project, dataset, param, name, jarr))

    @staticmethod
    def createDockerJob(project, dataset, param, name, docker_image, command, *args):
        """
        Create a job for submission to JobManager.

        Parameters
        ----------
        project : str
            A string visible in the project tab of |sym|'s jobmanager GUI.
        dataset : str
            A string visible in the dataset tab of |sym|'s jobmanager GUI.
        param : str
            A string visible in the param tab of |sym|'s  jobmanager GUI.
        name : str
            A string visible in the name tab of |sym|'s jobmanager GUI.
        docker_image : str
            Name to the docker imager.
        command : str
            Command to execute in docker container.
        args : List of strings
            Additional parameters. These parameters are forwarded to the Java
            starter class which is submitted to to JobManager for launching local
            jobs.

        """

        args = args[0]
        jarr = jarray(len(args), java.lang.String)
        for i in range(len(args)):
            if not isinstance(args[i], str):
                raise ValueError('All args must be strings')
            jarr[i] = java.lang.String(args[i])

        return str(JobDesc.createDockerJob(project, dataset, param, name, docker_image, command, jarr))

    @staticmethod
    def getPath(jobID):
        """
        Get the path of the local job file directory.

        Parameters
        ----------
        jobID : str

        Returns
        -------
        path : str
            Full path to job file directory

        """
        return os.path.normpath(JobDesc.getPath(jobID).getAbsolutePath())

    @staticmethod
    def getFileList(jobID):
        """
        Get list of files contained in local job directory.

        Parameters
        ----------
        jobID : str

        Returns
        -------
        ret : list of path strings
            Full paths to the files contained in local job directory.
        """

        jFileList = JobDesc.getFilelist(jobID)
        pyFileList = []
        for jFile in jFileList:
            path = os.path.normpath(jFile.getAbsolutePath())
            pyFileList.append(path)

        return pyFileList

    @staticmethod
    def getStatus(jobID):
        """
        Get the status of jobID.
        Job Status is one of the following strings.

        .. csv-table:: Defined Status strings
            :header: "Status", "Description"
            :widths: 10, 10

            "'UNDEFINED'", "Job status is undefined"
            "'CREATED'", "Job is created"
            "'SUBMITTED'", "Job is submitted to a execution queue"
            "'RUNNING'", "Job executes"
            "'COMPLETED'", "Job completed execution"
            "'HELD'", "Job is held"
            "'REMOVED'", "Job is removed"
            "'TIMEOUT'", "Job stopped because of timeout"
            "'ERROR'", "An error occured"
            "'STOP'", "Remove from executing queue"
            "'RESTART'", "Restart Job"
            "'COMPLETE'", "Job complete execution"
            "'SUBMIT'", "Job submitting to execute-queue"
            "'SUSPENDED'", "Job suspended execution"

        Parameters
        ----------
        jobID : string
            Job Id for query

        Returns
        -------
        status : string
            Status of the job
        """
        return JobDesc.getStatus(jobID).toString()

    @staticmethod
    def isJobRunning(symProject, index=None):
        """
        Returns True, if one or more jobs of symProject are at the job manager and have submit(ed) or running status.
        If there is no job available, jobs are finnished or have an error, the function returns false.
        If an index is defined, jobs of the defined index are considered, otherwise jobs of the current index are taken
        into account.
        This function is intended to check during script-based simulation control, wether an index requires
        an update (regenerate) or not.

        Parameters
        ----------
        symProject : symspace.wrapper.symwrapper.SymProject
            Python handle of the SyMSpace project
        index : int
            index of dataset (one-based!); if None - current index is used

        Returns
        -------
        ret : bool
        """

        if index is not None:
            curIndex = symProject.getIndex()
            symProject.setIndex(index)

        dataset = symProject.getDSId()

        if index is not None:
            symProject.setIndex(curIndex)

        # JobDesc.allFinished(path_to_project, id_of_dataset, id_of_parameter -> None = all parameters/functions)
        return not JobDesc.allFinished(os.path.join(symProject.getDirectory(), symProject.getFile()), dataset, None)

    @staticmethod
    def setTimeout(jobID, timeout):
        """
        Set the timeout value for the jobID.

        Parameters
        ----------
        jobID : str
        timeout : float

        """

        if not isinstance(jobID, str):
            raise ValueError('jobID must be a string')
        JobDesc.setTimeout(jobID, timeout)

    @staticmethod
    def isFinished(jobID):
        """
        Check if job is finished.  The job is finished if it is completed, removed
        or stopped because of an error or timeout.

        Parameters
        ----------
        jobID : str

        Returns
        -------
        ret : boolean
            Finished status of jobID.
        """
        return JobDesc.isFinished(jobID)

    @staticmethod
    def setRequirement(jobID, requirement_string):
        """
        Set the requirement for jobID.

        Parameters
        ----------
        jobID : str
        requirement_string : str
            Condor compatible requirement string
        """
        if not isinstance(jobID, str):
            raise ValueError('jobID must be a string')
        JobDesc.setRequirement(jobID, requirement_string)

    @staticmethod
    def submit(jobID, **kwargs):
        """
        Submit the job locally or to cluster.

        Parameters
        ----------
        jobID : str
        **kwargs :
            Keyword argument 'destination' can be set to 'local' or 'cluster' depending on where to run job.
        """

        if not isinstance(jobID, str):
            raise ValueError('jobID must be a string')

        if kwargs.get("destination"):
            destination = kwargs["destination"].lower()
            if destination == 'local':
                JobDesc.submit(jobID, JobType.LOCAL)
            elif destination == 'cluster':
                JobDesc.submit(jobID, JobType.CLUSTER)
            else:
                raise ValueError('Unknown keyword argument')
        else:
            raise ValueError('Keyword argument destination required')

    @staticmethod
    def stop(jobID):
        """
        Stop the job in Job Manager
        """
        if not isinstance(jobID, str):
            raise ValueError('jobID must be a string')

        JobDesc.stop(jobID)

    @staticmethod
    def remove(jobID):
        """
        Remove the job from Job Manager
        """
        if not isinstance(jobID, str):
            raise ValueError('jobID must be a string')

        JobDesc.remove(jobID)

    @staticmethod
    def restart(jobID):
        """
        Restart the job
        """
        if not isinstance(jobID, str):
            raise ValueError('jobID must be a string')

        JobDesc.restart(jobID)
