#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 9896 $
$LastChangedDate:: 2020-06-30 08:41:38 +0200#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import sys
import socket
import traceback
import pickle
import shutil
import argparse
import logging


def addLogHandler(log, log_file: str, log_level: int, use_format: bool = True):
    f_handler = logging.FileHandler(log_file)
    f_handler.setLevel(log_level)

    if use_format:
        f_format = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s', '%Y-%m-%d %H:%M:%S')
        f_handler.setFormatter(f_format)

    '''Add handlers to the logger'''
    log.addHandler(f_handler)


def run(logger):
    addLogHandler(logger, 'symstarter_debug.log', logging.DEBUG)
    parser = argparse.ArgumentParser(description='Run a python job')
    parser.add_argument('moduleName', help='')
    parser.add_argument('className', help='')
    parser.add_argument('infile', help='')
    parser.add_argument('outfile', help='')

    args = parser.parse_args()

    moduleName = args.moduleName
    className = args.className
    infile = args.infile
    outfile = args.outfile

    logger.debug('symstarter.py called on machine {}'.format(socket.gethostname()))
    logger.debug('Arguments:')
    for i, arg in enumerate(sys.argv):
        logger.debug('arg[{}] = {}'.format(i, arg))

    '''unpack the package archives'''
    logger.debug(50 * '-')
    logger.debug('Unpacking package archives on local machine')
    shutil.unpack_archive('lcm_package.zip', 'lcm')
    shutil.unpack_archive('symspace_package.zip', 'symspace')
    if os.path.isfile('packages.zip'):
        shutil.unpack_archive('packages.zip')
    logger.debug('Archives unpacked successfully')

    '''read input data'''
    logger.debug(50 * '-')
    logger.debug('Reading input data from file: {}'.format(infile))
    with open(infile, 'rb') as f:
        d = pickle.load(f)
    logger.debug('Input data read succesfully')

    '''retrieve the runargs dictionary'''
    runargs = d['__RunArgs']

    '''replace parampythonfunction by dummy version to avoid jep imports'''
    shutil.copyfile('dummy_parampythonfnc.py', os.path.join('symspace', 'parampythonfnc.py'))

    '''import class'''
    logger.debug(50 * '-')
    module0 = os.path.splitext(moduleName)[0]
    logger.debug('Importing module: {}'.format(module0))
    module = __import__(module0)
    logger.debug('Importing class: {}'.format(className))
    classHandle = getattr(module, className)
    logger.debug('Creating Instance of class {}'.format(className))
    inst = classHandle(None)
    logger.debug('Call function runJob with arguments: d={}, runargs={}'.format('<inputDataDict>', str(runargs)))
    ret = inst.runJob(d, runargs)
    logger.debug('status runJob: {}'.format(str(ret)))

    '''export data'''
    logger.debug('Export data: {}'.format('<exportDataDict>'))
    logger.debug('Writing export data to file {}'.format(outfile))
    with open(outfile, 'wb') as f:
        pickle.dump(d, f)
    logger.debug('Outfile written successfully')

    logger.debug(50 * '-')
    logger.debug('Starter finished successfully!')


if __name__ == '__main__':
    '''Python Starter for Symspace ParamPythonFunction remote job execution'''

    sym_logger = logging.getLogger('symstarter')
    sym_logger.setLevel(logging.DEBUG)

    try:
        run(sym_logger)
    except Exception:
        '''Handle error'''
        sym_logger.exception("Exception occurred")

        error_logger = logging.getLogger('symstarter_error')
        error_logger.setLevel(logging.ERROR)
        addLogHandler(error_logger, 'symstarter_error.log', logging.ERROR, use_format=False)

        exc_type, exc_value, exc_traceback = sys.exc_info()
        error_logger.error(str(exc_type))
        error_logger.error(str(exc_value))
        tb = traceback.extract_tb(exc_traceback)
        for i, item in enumerate(tb):
            error_logger.error('{},{},{},{}'.format(item.name, os.path.basename(item.filename), item.lineno, item.line))
