#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 9896 $
$LastChangedDate:: 2020-06-30 08:41:38 +0200#$

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import optparse
import pkgutil
import nose
import unittest
import xmlrunner
import symspace.osutils.pathutils as pathutils
from symspace.osutils.pathutils import CMDParser


class NoseTestRunner:
    """
    Tests all Python tests found in path and all it's subdirectories using the nose framework

    - writes xunit xml-files with testresults
    - writes coverage xml-files for selected packages

    """

    def __init__(self, start_dir, reports_dir, cov_pkgs):

        self.start_dir = start_dir
        self.reports_dir = reports_dir
        self.cov_pkgs = cov_pkgs
        self.xunit_file = os.path.join(reports_dir, 'nosetests-junit.xml')
        self.cover_xml = True
        self.cover_html = False
        self.runner = nose

    def run(self):
        if not os.path.isdir(self.reports_dir):
            os.makedirs(self.reports_dir, exist_ok=True)

        arguments = ['', '--where={}'.format(self.start_dir),
                     '--with-xunit', '--xunit-file={}'.format(self.xunit_file),
                     '--with-coverage'
                     ]
        if self.cover_xml:
            arguments.extend(['--cover-xml',
                              '--cover-xml-file={}'.format(os.path.join(self.reports_dir, 'coverage.xml')),
                              ])

        if self.cover_html:
            arguments.extend(['--cover-html',
                              '--cover-html-dir={}'.format(os.path.join(self.reports_dir, 'html'))
                              ])

        for pkgs in self.cov_pkgs:
            arguments.append('--cover-package={}'.format(pkgs))

        return self.runner.run(argv=arguments)

    @classmethod
    def fromCMDParser(cls) -> 'NoseTestRunner':
        """
        Commandline parser to create instance of this class

        Commandline Arguments
        ---------------------

        --startDir : str
            The search for tests starts from this directory (Optional).

        Returns
        -------
        runner : NoseTestRunner
        """

        start_dir_default = os.path.join(pathutils.symspace_home, 'System', 'Python', 'tests')
        test_dir_default = pathutils.test_reports_dir
        cov_dir_default = os.path.join(pathutils.symspace_home, 'System', 'Python', 'src')

        parser = optparse.OptionParser("usage: %prog [options]")
        parser.add_option("-d", "--startDir", dest="start_dir", default=start_dir_default,
                          type="string", help="The search for tests starts from this directory.")
        parser.add_option("-r", "--reportsDir", dest="reports_dir", default=test_dir_default,
                          type="string", help="The test reports will be saved to this directory.")
        parser.add_option("-c", "--coverPkgDir", dest="cov_dir", default=cov_dir_default,
                          type="string", help="The packages importable from this directory will be covered.")
        args = CMDParser.jepSysArgs()
        (options, args) = parser.parse_args(args)

        start_dir = CMDParser.jepArgPath(options.start_dir)
        reports_dir = CMDParser.jepArgPath(options.reports_dir)
        cov_dir = CMDParser.jepArgPath(options.cov_dir)

        if start_dir == "":
            raise ValueError("--startDir must be given")
        if reports_dir == "":
            raise ValueError("--reportsDir must be given")
        if cov_dir == "":
            raise ValueError("--coverPkgDir must be given")

        cov_pkgs = [pkgname for _, pkgname, _ in pkgutil.iter_modules([cov_dir])]

        print('Start Path: {}'.format(start_dir))
        print('Report Path: {}'.format(reports_dir))
        print('Coverage: {}'.format(cov_pkgs))

        runner = cls(start_dir, reports_dir=reports_dir, cov_pkgs=cov_pkgs)
        return runner


class XMLTestRunner:
    """
    Tests all Python tests found in path and all it's subdirectories using the unittest framework

    - writes xunit xml-files with testresults

    """

    def __init__(self, start_dir, reports_dir):
        self.reports_dir = reports_dir
        self.suite = unittest.TestSuite()
        self.loader = unittest.TestLoader()

        self.runner = xmlrunner.XMLTestRunner(output=reports_dir, outsuffix='-junit')
        self.suite.addTests(self.loader.discover(start_dir))

    def run(self):
        if not os.path.isdir(self.reports_dir):
            os.makedirs(self.reports_dir, exist_ok=True)

        return self.runner.run(self.suite)
