# -*- coding: utf-8 -*-
"""
Created on Tue October 09 15:23:02 2017

@author: wkoppelstaetter

"""

import os
import glob
import pandas as pd
from enum import Enum
from symspace.wrapper.symwrapper import SymProject
from symspace.wrapper.symplot import SymSubPlot, SymPlot2Scatter, SymPlot2Hist

try:
    # Start of java imports
    # noinspection PyUnresolvedReferences
    import java
    # noinspection PyUnresolvedReferences
    from java.lang import String as JString
    # noinspection PyUnresolvedReferences
    from java.io import File as JFile
    # noinspection PyUnresolvedReferences
    from java.awt import Color as JColor
    # noinspection PyUnresolvedReferences
    from jep import jarray
    # noinspection PyUnresolvedReferences
    from java.lang.reflect import Array as JArray
    # noinspection PyUnresolvedReferences
    from magopt.utils import MopFile as JMopFile
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.optim.core import Archive as JArchive
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.optim.core import Individual as JIndividual
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.optim.core import VariableDesc as JVariableDesc
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.optim.core import ResultDesc as JResultDesc
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.optim.core import ObjectiveType as JObjectiveType
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.optim.core import SavePoint as JSavePoint
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.optim.core.project import XLSWrite as JXLSWrite
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.plot import Marker as JMarker
except Exception as e:
    print('Unable to load Java Libraries!\n{}'.format(e))


class SymArchive:
    """
    Optimization archive
    """

    @staticmethod
    def listArchives(strFile):
        """
        list archive-names from mop-file

        Parameters
        ----------
        strFile : string
            Full path to .mop file

        Returns
        -------
        list[str]
            List of existing archive-names
        """

        norm_path = os.path.normpath(strFile)
        mop = JMopFile(norm_path)
        archs = JArchive.listArchives(mop)
        return [n for n in archs]

    @staticmethod
    def load(strFile, name):
        """
        load archive from mop-file

        Parameters
        ----------
        strFile : string
            Full path to .mop file
        name : string
            Name of archive

        Returns
        -------
        archive : SymArchive
        """

        norm_path = os.path.normpath(strFile)
        mop = JMopFile(norm_path)

        arch = SymArchive(javaObj=JArchive.load(mop, name))
        return arch

    @staticmethod
    def delete(strFile, name):
        """
        remove archive from mop-file

        Parameters
        ----------
        strFile : string
            Full path to .mop file
        name : string
            Name of archive
        """

        norm_path = os.path.normpath(strFile)
        mop = JMopFile(norm_path)
        JArchive.delete(mop, name)

    @classmethod
    def new(cls, strFile, name):
        """
        Create a new archive

        Parameters
        ----------
        strFile : string
            Full path to .mop file
        name : string
            Name of archive

        Returns
        -------
        SymArchive
        """

        obj = cls(strFile, name)
        return obj

    def __init__(self, *args, **kwargs):
        if kwargs.get('javaObj'):
            self.javaObj = kwargs['javaObj']
        else:
            if args:
                nargs = len(args)
                if nargs == 2:
                    norm_path = os.path.normpath(args[0])
                    mop = JMopFile(norm_path)
                    name = args[1]
                    self.javaObj = JArchive(mop, name)
                else:
                    raise ValueError('wrong number of parameters')
            else:
                raise ValueError('missing parameter')

    def __str__(self):
        return 'Sym' + self.javaObj.toString()

    def __repr__(self):
        return self.__str__()

    # methods
    def getProject(self):
        """
        Get project

        Returns
        -------
        SymProject
        """

        return SymProject(javaObj=self.javaObj.getProject())

    def save(self):
        """
        Save archive
        """

        self.javaObj.save()

    def getName(self):
        """
        Get name of archive

        Returns
        -------
        Name of archive
        """

        return self.javaObj.getName()

    def rename(self, name):
        """
        Rename archive

        Parameters
        ----------
        name : string
            Name of archive
        """

        self.javaObj.rename(name)

    def duplicate(self, name):
        """
        Duplicate archive

        Parameters
        ----------
        name : string
            Name of archive

        Returns
        -------
        Archive : SymArchive
        """

        return SymArchive(javaObj=self.javaObj.duplicate(name))

    def getMop(self):
        """
        Get mop-path of project

        Returns
        -------
        Path to .mop
        """

        return self.javaObj.getMop().getAbsolutePath()

    def getBasedIndex(self):
        """
        Get based index of project

        Returns
        -------
        Based index
        """

        return self.javaObj.getBasedIndex()

    def setBasedIndex(self, index):
        """
        Set based index of project

        Parameters
        ----------
        index : integer
            Based index of project
        """

        self.javaObj.setBasedIndex(index)

    def listProblems(self):
        """
        list existing problem-names

        Returns
        -------
        List of existing problem-names
        """

        probs = self.javaObj.listProblems()
        return [n for n in probs]

    def createProblem(self, name):
        """
        Create problem

        Parameters
        ----------
        name : string
            Name of problem

        Returns
        -------
        Problem : SymProblem
        """

        return SymProblem(javaObj=self.javaObj.createProblem(name))

    def readProblem(self, name):
        """
        Read existing problem

        Parameters
        ----------
        name : string
            Name of problem

        Returns
        -------
        Problem : SymProblem
        """

        return SymProblem(javaObj=self.javaObj.readProblem(name))

    def deleteProblem(self, name):
        """
        Delete existing problem

        Parameters
        ----------
        name : string
            Name of problem
        """

        self.javaObj.deleteProblem(name)

    def getVariables(self):
        """
        List existing variables-names

        Returns
        -------
        List of exitsting variable-names
        """

        vars = self.javaObj.getVariables()
        return [n for n in vars]

    def getResults(self):
        """
        List existing result-names

        Returns
        -------
        List of exitsting result-names
        """

        ress = self.javaObj.getResults()
        return [n for n in ress]

    def importProblemSettings(self, name, settingsFile, project):
        """
        Import problem-settings from file

        Parameters
        ----------
        name : string
            Name of new problem
        settingsFile : string
            Full path to settings-file
        project : SymProject
            Based project

        Returns
        -------
        Warning message
        """

        norm_path = os.path.normpath(settingsFile)
        xml = JFile(norm_path)
        return self.javaObj.importProblemSettings(name, xml, project.javaObj)

    def getMemory(self):
        """
        Get Memory with computed individuals

        Returns
        -------
        SymMemory
        """

        return SymMemory(javaObj=self.javaObj.getMemory())


class SymProblem:
    """
    Optimization problem based on archive
    """

    def __init__(self, javaObj):
        self.javaObj = javaObj

    def __str__(self):
        return 'Sym' + self.javaObj.toString()

    def __repr__(self):
        return self.__str__()

    # methods
    def getName(self):
        """
        Get name of problem

        Returns
        -------
        Name
        """

        return self.javaObj.getName()

    def rename(self, name):
        """
        Rename problem

        Parameters
        ----------
        name : string
            New name
        """

        self.javaObj.rename(name)

    def getArchive(self):
        """
        Get based archive

        Returns
        -------
        SymArchive
        """

        return SymArchive(javaObj=self.javaObj.getArchive())

    def createSavePoint(self):
        """
        CreateGet SavePoint
        """

        return self.javaObj.createSavePoint()

    def getNumSavePoints(self):
        """
        Get number of savepoints

        Returns
        -------
        Number of savepoints
        """

        return self.javaObj.getNumSavePoints()

    def getSavePoint(self, index):
        """
        Get SavePoint with index

        Parameters
        ----------
        index : int
            SavePoint

        Returns
        -------
        SymSavepoint
        """

        return SymSavePoint(javaObj=self.javaObj.getSavePoint(index))

    def isLocked(self):
        """
        Is problem locked (savepoints exist)

        Returns
        -------
        Is locked
        """

        return self.javaObj.isLocked()

    def duplicate(self, name):
        """
        Duplicate problem

        Parameters
        ----------
        name : string
            New name

        Returns
        -------
        SymProblem
        """

        return SymProblem(javaObj=self.javaObj.duplicate(name))

    def reduceArchive(self, name, numberOfParetoFront):
        """
        Duplicate and reduce Archive

        Parameters
        ----------
        name : string
            New archive name
        numberOfParetoFront : int
            Number of individuals in new archive from pareto-front
        """

        self.javaObj.reduceArchive(name, numberOfParetoFront)

    def exportSettings(self, settingsFile):
        """
        Export problem-settings to file

        Parameters
        ----------
        settingsFile : string
            Full path for settings-file
        """

        norm_path = os.path.normpath(settingsFile)
        xml = JFile(norm_path)
        return self.javaObj.exportSettings(xml)

    def addVariable(self, name):
        """
        Add variable to archive

        Parameters
        ----------
        name : string
            Name of new variable

        Returns
        -------
        SymVaribleDesc
        """

        return SymVaribleDesc(javaObj=self.javaObj.addVariable(name))

    def rmVariable(self, name):
        """
        Remove variable from archive

        Parameters
        ----------
        name : string
            Name of variable
        """

        return self.javaObj.rmVariable(name)

    def getNumVariables(self):
        """
        Get number of variables

        Returns
        -------
        Number
        """

        return self.javaObj.getNumVariables()

    def getVariable(self, name):
        """
        Get variable description

        Parameters
        ----------
        name : string
            Name

        Returns
        -------
        SymVaribleDesc
        """

        return SymVaribleDesc(javaObj=self.javaObj.getVariable(name))

    def getVariables(self):
        """
        Get variables description

        Returns
        -------
        List of SymVaribleDesc
        """

        return [SymVaribleDesc(javaObj=v) for v in self.javaObj.getVariables()]

    def getNumVariedVariables(self):
        """
        Get number of varied variables

        Returns
        -------
        Number
        """

        return self.javaObj.getNumVariedVariables()

    def getVariedVariables(self):
        """
        Get varied variables description

        Returns
        -------
        List of SymVaribleDesc
        """

        return [SymVaribleDesc(javaObj=v) for v in self.javaObj.getVariedVariables()]

    def addResult(self, name):
        """
        Add result to archive

        Parameters
        ----------
        name : string
            Name of new result

        Returns
        -------
        SymResultDesc
        """

        return SymResultDesc(javaObj=self.javaObj.addResult(name))

    def rmResult(self, name):
        """
        Remove result from archive

        Parameters
        ----------
        name : string
            Name of result
        """

        return self.javaObj.rmResult(name)

    def getNumResults(self):
        """
        Get number of results

        Returns
        -------
        Number
        """

        return self.javaObj.getNumResults()

    def getResult(self, name):
        """
        Get result description

        Parameters
        ----------
        name : string
            Name

        Returns
        -------
        SymResultDesc
        """

        return SymResultDesc(javaObj=self.javaObj.getResult(name))

    def getResults(self):
        """
        Get results description

        Returns
        -------
        List of SymResultDesc
        """

        return [SymResultDesc(javaObj=r) for r in self.javaObj.getResults()]

    def getNumObjectives(self):
        """
        Get number of objectives

        Returns
        -------
        Number
        """

        return self.javaObj.getNumObjectives()

    def getObjectives(self):
        """
        Get results description of objectives

        Returns
        -------
        List of SymResultDesc
        """

        return [SymResultDesc(javaObj=r) for r in self.javaObj.getObjectives()]

    def getNumConstraints(self):
        """
        Get number of contraints

        Returns
        -------
        Number
        """

        return self.javaObj.getNumConstraints()

    def getConstraints(self):
        """
        Get results description of constraints

        Returns
        -------
        List of SymResultDesc
        """

        return [SymResultDesc(javaObj=r) for r in self.javaObj.getConstraints()]

    def clearSolution(self):
        """
        Clear solution of problem (remove save-points)
        """

        self.javaObj.clearSolution()

    def createRandom(self):
        """
        Create random individual

        Returns
        -------
        SymIndividual
        """

        return SymIndividual(javaObj=self.javaObj.createRandom())

    def createInitialDistribution(self, number):
        """
        Create initial individual by latin hyercube-sampling

        Parameters
        ----------
        number : int
            Number of individuals

        Returns
        -------
        List of SymIndividuals
        """

        return SymIndividual(javaObj=self.javaObj.createInitDistribution(number))

    def getPoolSize(self):
        """
        Get size of calculation pool

        Returns
        -------
        Size of pool
        """

        return self.javaObj.getPoolSize()

    def setPoolSize(self, poolSize):
        """
        Set size of calculation pool

        Parameters
        ----------
        poolSize : int
            New size of pool
        """

        self.javaObj.setPoolSize(poolSize)

    def getAutosaveInterval(self):
        """
        Get interval (number of individuals) to create save-point

        Returns
        -------
        Number of individuals between save-points
        """

        return self.javaObj.getAutosaveInterval()

    def setAutosaveInterval(self, autosave):
        """
        Set interval (number of individuals) to create save-point

        Parameters
        ----------
        autosave : int
            Number of individuals between save-point
        """

        self.javaObj.setAutosaveIntervall(autosave)

    def getAutosaveTime(self):
        """
        Get time interval to create save-point

        Returns
        -------
        Time between save-points (seconds)
        """

        return self.javaObj.getAutosaveTime()

    def setAutosaveTime(self, autosave):
        """
        Set time interval to create save-point

        Parameters
        ----------
        autosave : int
            Time between save-points (seconds)
        """

        self.javaObj.setAutosaveTime(autosave)

    def getAutosaveTimeEnabled(self):
        """
        Is time-based auto-save enabled

        Returns
        -------
        Is  enabled
        """

        return self.javaObj.getAutosaveTimeEnabled()

    def setAutosaveTimeEnabled(self, enabled):
        """
        Enable time-based auto-save

        Parameters
        ----------
        enabled : boolean
            Set enable
        """

        self.javaObj.setAutosaveTimeEnabled(enabled)

    def getFunctionTimeout(self):
        """
        Get timeout for external function on jobmanager

        Returns
        -------
        Timeout for external functions (seconds)
        """

        return self.javaObj.getFunctionTimeout()

    def setFunctionTimeout(self, timeout):
        """
        Set timeout for external function on jobmanager

        Parameters
        ----------
        timeout : int
            Timeout for external functions (seconds)
        """

        self.javaObj.setFunctionTimeout(timeout)

    def getInitSolver(self):
        """
        Is initulize empty solver from memory enabled

        Returns
        -------
        Initilize solver
        """

        return self.javaObj.getInitSolver()

    def setInitSolver(self, enabled):
        """
        Enable initialize empty solver from memory

        Parameters
        ----------
        enabled : boolean
            initialize solver
        """

        self.javaObj.setInitSolver(enabled)

    def getReattemptError(self):
        """
        Get reattempt individual if allready calculatet with an error

        Returns
        -------
        Reattempt on error
        """

        return self.javaObj.getReattemptError()

    def setReattemptError(self, enabled):
        """
        Set reattempt individual if allready calculatet with an error

        Parameters
        ----------
        enabled : boolean
            reattemt on error
        """

        self.javaObj.setReattemptError(enabled)

    # TODO getSolver
    # TODO setSolver

    def getOptimization(self):
        """
        Get optimization

        Returns
        -------
        SymOptimization
        """

        optim = self.javaObj.getOptimization()
        if optim is not None:
            optim = SymOptimization(javaObj=optim)

        return optim


class SymMemory:
    """
    Memory of computed individuals
    """

    def __init__(self, javaObj):
        self.javaObj = javaObj

    def __str__(self):
        return 'Sym' + self.javaObj.toString()

    def __repr__(self):
        return self.__str__()

    # methods
    def isEmpty(self):
        """
        Is size of memory zero

        Returns
        -------
        Is empty
        """

        return self.javaObj.isEmpty()

    def getArchive(self):
        """
        Get based archive

        Returns
        -------
        SymArchive
        """

        return SymArchive(javaObj=self.javaObj.getArchive())

    def getNumIndividuals(self):
        """
        Get number of stored individuals

        Returns
        -------
        Number of individuals
        """

        return self.javaObj.getNumIndividuals()

    def getIndividual(self, select):
        """
        Get individual from memory
        1) By index
        2) By pattern with defined variable values

        Parameters
        ----------
        select : int or SymIndividual
            Index or pattern of individual

        Returns
        -------
        SymIndividual
        """

        if isinstance(select, SymIndividual):
            select = self.javaObj.getIndividual(select.javaObj)
        else:
            select = self.javaObj.getIndividual(select)

        if select is not None:
            select = SymIndividual(javaObj=select)

        return select

    def getIndividuals(self):
        """
        Get stored individuals

        Returns
        -------
        SymIndividuals
        """

        return [SymIndividual(javaObj=ind) for ind in self.javaObj.getIndividual()]

    def getParetoIndividuals(self, problem):
        """
        Get individuals from pareto-from

        Parameters
        ----------
        problem : SymProblem
            Problem for filter

        Returns
        -------
        SymIndividuals
        """

        return [SymIndividual(javaObj=ind) for ind in self.javaObj.getParetoIndividual(problem.javaObj)]

    def getValidIndividuals(self, problem):
        """
        Get valid individuals

        Parameters
        ----------
        problem : SymProblem
            Problem for filter

        Returns
        -------
        SymIndividuals
        """

        return [SymIndividual(javaObj=ind) for ind in self.javaObj.getValidIndividual(problem.javaObj)]

    def getNoErrorIndividuals(self):
        """
        Get individuals without error

        Returns
        -------
        SymIndividuals
        """

        return [SymIndividual(javaObj=ind) for ind in self.javaObj.getNoErrorIndividual()]

    def getErrorIndividuals(self):
        """
        Get individuals with error

        Returns
        -------
        SymIndividuals
        """

        return [SymIndividual(javaObj=ind) for ind in self.javaObj.getErrorIndividual()]


class SymIndividual:
    """
    Individuals (one dataset)
    """

    @classmethod
    def new(cls, archive):
        """
        Create a new archive

        Parameters
        ----------
        archive : SymArchive
            Based archive

        Returns
        -------
        SymIndividual
        """

        obj = cls(archive)
        return obj

    def __init__(self, *args, **kwargs):
        if kwargs.get('javaObj'):
            self.javaObj = kwargs['javaObj']
        else:
            if args:
                nargs = len(args)
                if nargs == 1:
                    arch = args[0]
                    self.javaObj = JIndividual(arch.javaObj)
                else:
                    raise ValueError('wrong number of parameters')
            else:
                raise ValueError('missing parameter')

    def __str__(self):
        return self.javaObj.toString()

    def __repr__(self):
        return self.__str__()

    # methods
    def getArchive(self):
        """
        Get based archive

        Returns
        -------
        SymArchive
        """

        return SymArchive(javaObj=self.javaObj.getArchive())

    def isError(self):
        """
        Has individual an error

        Returns
        -------
        Is error
        """

        return self.javaObj.isError()

    def isValid(self, problem):
        """
        Is valid individual for problem

        Parameters
        ----------
        problem : SymProblem
            Problem to check

        Returns
        -------
        Is valid
        """

        return self.javaObj.isValid(problem.javaObj)

    def isValidVar(self, problem):
        """
        Are variable values valid for problem

        Parameters
        ----------
        problem : SymProblem
            Problem to check

        Returns
        -------
        Are variable valid
        """

        return self.javaObj.isValidVar(problem.javaObj)

    def isFromMapping(self):
        """
        Is this indivudal created by mapping

        Returns
        -------
        Is from mapping
        """

        return self.javaObj.isFromMapping()

    def getVariable(self, select):
        """
        Get variable value
        1) By index
        2) By name

        Parameters
        ----------
        select : int or str
            Index or name of variable

        Returns
        -------
        Value of variable
        """
        return self.javaObj.getVariable(select)

    def setVariable(self, select, value):
        """
        Set variable value
        1) By index
        2) By name

        Parameters
        ----------
        select : int or str
            Index or name of variable
        value : double
            Value of variable
        """
        self.javaObj.setVariable(select, value)

    def getVariableValues(self):
        """
        Get list of variable values

        Returns
        -------
        Values of variables
        """
        return [v for v in self.javaObj.getVariableValues()]

    def getResult(self, select):
        """
        Get result value
        1) By index
        2) By name

        Parameters
        ----------
        select : int or str
            Index or name of result

        Returns
        -------
        Value of result
        """
        return self.javaObj.getResult(select)

    def getResultValues(self):
        """
        Get list of result values

        Returns
        -------
        Values of results
        """
        return [v for v in self.javaObj.getResultValues()]

    def getObjectiveValues(self, problem):
        """
        Get list of objective values

        Parameters
        ----------
        problem : SymProblem
            problem with objective description

        Returns
        -------
        Values of objectives
        """
        return [v for v in self.javaObj.getObjectiveValues(problem.javaObj)]


class SymSavePointType(Enum):
    """
    Save-point-types
    """
    USER = 1
    INTERVAL = 2
    TIME = 3

    @staticmethod
    def _getPythonType(javaType):
        """
        get python-type from java-type for SavePoint

        Parameters
        ----------
        javaType : enum
            Java-tpye

        Returns
        -------
        SymSavepoint-type (python)
        """
        if javaType == JSavePoint.SavePointType.USER:
            pythonType = SymSavePointType.USER
        elif javaType == JSavePoint.SavePointType.INTERVAL:
            pythonType = SymSavePointType.INTERVAL
        elif javaType == JSavePoint.SavePointType.TIME:
            pythonType = SymSavePointType.TIME
        else:
            raise ValueError('Undefined SavePoint-type')
        return pythonType

    @staticmethod
    def _getJavaType(pyType):
        """
        get java-type from python-type for SavePoint

        Parameters
        ----------
        pyType : enum
            Python-tpye

        Returns
        -------
        SymSavepoint-type (java)
        """
        if pyType == SymSavePointType.USER:
            javaType = JSavePoint.SavePointType.USER
        elif pyType == SymSavePointType.INTERVAL:
            javaType = JSavePoint.SavePointType.INTERVAL
        elif pyType == SymSavePointType.TIME:
            javaType = JSavePoint.SavePointType.TIME
        else:
            raise ValueError('Undefined SavePoint-type')
        return javaType


class SymSavePoint:
    """
    Save-point of optimization
    """

    def __init__(self, javaObj):
        self.javaObj = javaObj

    def __str__(self):
        return 'Sym' + self.javaObj.toString()

    def __repr__(self):
        return self.__str__()

    # methods
    def getType(self):
        """
        Get type of savepoint

        Returns
        -------
        SymSavePointType
        """

        return SymSavePointType._getPythonType(self.javaObj.getType())

    def getDate(self):
        """
        Get creation date

        Returns
        -------
        Date
        """

        # TODO python date
        return self.javaObj.getDate()

    def getNumErrors(self):
        """
        Get number of error individuals in savepoint

        Returns
        -------
        Number of error individuals
        """

        return self.javaObj.getNError()

    def getNumInvalids(self):
        """
        Get number of invalid individuals in savepoint

        Returns
        -------
        Number of invalid individuals
        """

        return self.javaObj.getNInvalid()

    def getNumValids(self):
        """
        Get number of valid individuals in savepoint

        Returns
        -------
        Number of valid individuals
        """

        return self.javaObj.getNValid()

    def getFullPareto(self):
        """
        Get pareto-front of memory

        Returns
        -------
        Full pareto-front
        """

        return [SymIndividual(javaObj=ind) for ind in self.javaObj.getFullPareto()]

    def getLocalPareto(self):
        """
        Get local pareto-front of selected algorithm

        Returns
        -------
        Local pareto-front of algorithm
        """

        return [SymIndividual(javaObj=ind) for ind in self.javaObj.getLocalPareto()]


class SymVaribleDesc:
    """
    Variable description
    """

    @classmethod
    def new(cls, name):
        """
        Create a new variable description

        Parameters
        ----------
        name : string
            Name of variable (identifier)

        Returns
        -------
        SymVariableDesc
        """

        # TODO optinoal parameters
        obj = cls(name)
        return obj

    def __init__(self, *args, **kwargs):
        if kwargs.get('javaObj'):
            self.javaObj = kwargs['javaObj']
        else:
            if args:
                nargs = len(args)
                if nargs == 1:
                    name = args[0]
                    self.javaObj = JVariableDesc(name)
                elif nargs == 4:
                    name = args[0]
                    lb = args[1]
                    stepSize = args[2]
                    ub = args[3]
                    self.javaObj = JVariableDesc(name, lb, stepSize, ub)
                else:
                    raise ValueError('wrong number of parameters')
            else:
                raise ValueError('missing parameter')

    def __str__(self):
        return self.javaObj.toString()

    def __repr__(self):
        return self.__str__()

    # methods
    def getName(self):
        """
        Get name of variable (identifier)

        Returns
        -------
        Name
        """

        return self.javaObj.getName()

    def getLB(self):
        """
        Get lower boundary of variable

        Returns
        -------
        Lower boundary
        """

        return self.javaObj.getLB()

    def getStepSize(self):
        """
        Get step-size of variable
        Value of zero defines non-discrete value (real)

        Returns
        -------
        Step-size
        """

        return self.javaObj.getStepSize()

    def getUB(self):
        """
        Get uppder boundary of variable

        Returns
        -------
        Upper boundary
        """

        return self.javaObj.getUB()

    def getNumSteps(self):
        """
        Get number of variable steps

        Returns
        -------
        Number steps
        """

        return self.javaObj.getNumSteps()

    def isVaried(self):
        """
        Is variable varied (upper boundary is greater than lower boundary)

        Returns
        -------
        Is varied
        """

        return self.javaObj.isVaried()

    def round(self, value):
        """
        Round value to be on steps. This means that value is valid

        Parameters
        ----------
        value : double
            Value to round

        Returns
        -------
        Round value to be valid
        """

        return self.javaObj.round(value)

    def setBounds(self, lb, step, ub):
        """
        Set the boundaries of variable

        Parameters
        ----------
        lb: float
            Lower boundary
        step : float
            Step-size
        ub : float
            Upper boundary
        """

        self.javaObj.setBounds(lb, step, ub)


class SymResultDesc:
    """
    Result description
    """

    @classmethod
    def new(cls, name):
        """
        Create a new result description

        Parameters
        ----------
        name : string
            Name of variable (identifier)

        Returns
        -------
        SymResultDesc
        """

        # TODO optinoal parameters
        obj = cls(name)
        return obj

    def __init__(self, *args, **kwargs):
        if kwargs.get('javaObj'):
            self.javaObj = kwargs['javaObj']
        else:
            if args:
                nargs = len(args)
                if nargs == 1:
                    name = args[0]
                    self.javaObj = JResultDesc(name)
                elif nargs == 2:
                    name = args[0]
                    type = args[1]
                    self.javaObj = JResultDesc(name, type.javaObj)  # TODO ObjectiveType
                elif nargs == 3:
                    name = args[0]
                    lb = args[1]
                    ub = args[3]
                    self.javaObj = JResultDesc(name, lb, ub)
                elif nargs == 4:
                    name = args[0]
                    type = args[1]
                    lb = args[2]
                    ub = args[3]
                    self.javaObj = JResultDesc(name, type.javaObj, lb, ub)  # TODO ObjectiveType
                else:
                    raise ValueError('wrong number of parameters')
            else:
                raise ValueError('missing parameter')

    def __str__(self):
        return self.javaObj.toString()

    def __repr__(self):
        return self.__str__()

    # methods
    def getName(self):
        """
        Get name of result (identifier)

        Returns
        -------
        Name
        """

        return self.javaObj.getName()

    def getLB(self):
        """
        Get lower boundary of constraint

        Returns
        -------
        Lower boundary
        """

        return self.javaObj.getLB()

    def getUB(self):
        """
        Get upper boundary of constraint

        Returns
        -------
        Upper boundary
        """

        return self.javaObj.getUB()

    def isObjective(self):
        """
        Is result defined as objective

        Returns
        -------
        Is objective
        """

        return self.javaObj.isObjective()

    def getObjective(self):
        """
        Get objective-type or result

        Returns
        -------
        Objective-type
        """

        return SymObjectiveType._getPythonType(self.javaObj.getObjective())

    def setObjective(self, objectiveType):
        """
        Get objective-type or result

        Parameters
        ----------
        objectiveType : SymObjectiveType
            Objective-type
        """

        return self.javaObj.setObjective(SymObjectiveType._getJavaType(objectiveType))

    def isConstraint(self):
        """
        Is result defined as contraint (lower and upper boundary)

        Returns
        -------
        Is constraint
        """

        return self.javaObj.isConstraint()

    def setBounds(self, lb, ub):
        """
        Set the boundaries of result (constraint)

        Parameters
        ----------
        lb : float
            Lower boundary
        ub : float
            Upper boundary
        """

        self.javaObj.setBounds(lb, ub)


class SymObjectiveType(Enum):
    """
    Objective-types
    """
    NONE = 1
    MIN = 2
    MAX = 3

    @staticmethod
    def _getPythonType(javaType):
        """
        get python-type from java-type for objective

        Parameters
        ----------
        javaType : enum
            Java-tpye

        Returns
        -------
        Python-tpye
        """
        if javaType == JObjectiveType.NONE:
            pythonType = SymObjectiveType.NONE
        elif javaType == JObjectiveType.MIN:
            pythonType = SymObjectiveType.MIN
        elif javaType == JObjectiveType.MAX:
            pythonType = SymObjectiveType.MAX
        else:
            raise ValueError('Undefined objective-type')
        return pythonType

    @staticmethod
    def _getJavaType(pythonType):
        """
        get java-type from python-type for objective

        Parameters
        ----------
        pythonType : enum
            Python-tpye

        Returns
        -------
        Java-tpye
        """
        if pythonType == SymObjectiveType.NONE:
            javaType = JObjectiveType.NONE
        elif pythonType == SymObjectiveType.MIN:
            javaType = JObjectiveType.MIN
        elif pythonType == SymObjectiveType.MAX:
            javaType = JObjectiveType.MAX
        else:
            raise ValueError('Undefined objective-type')
        return javaType


class SymOptimization:
    """
    Optimization
    """

    def __init__(self, javaObj):
        self.javaObj = javaObj

    # methods
    def runProject(self):
        """
        Run project (add new individual by algorithm)
        """

        self.javaObj.runProject()

    def finishProject(self):
        """
        Finish project (add no new individual by algorithm, finisch existing and new from buffer)
        """

        self.javaObj.finishProject()

    def stopProject(self):
        """
        Stop project
        """

        self.javaObj.stopProject()

    def getProjectStatus(self):
        """
        Get project status

        Returns
        -------
        String with Status
        """

        return self.javaObj.getProjectStatus()

    def shutdown(self):
        """
        Shutdown optimizer (Stop optimizing and save)
        """

        self.javaObj.shutdown()

    def getProblem(self):
        """
        Get problem of optimization

        Returns
        -------
        SymProblem
        """

        return SymProblem(javaObj=self.javaObj.getProblem())

    def getEventLog(self):
        """
        Get Eventlog of optimization

        Returns
        -------
        String with EventLog
        """

        return self.javaObj.getEventLog().toString()

    def clearLog(self):
        """
        Clear Eventlog of optimization
        """

        self.javaObj.clearLog()

    def add2Buffer(self, individuals):
        """
        Add individuals to project-buffer

        Parameters
        ----------
        individuals : list[SymIndividual]
            List of SymIndividual
        """

        inds = JArray.newInstance(JIndividual, len(individuals))
        for k in range(0, len(individuals)):
            inds[k] = individuals[k].javaObj

        self.javaObj.add2Buffer(inds)

    def clearProjectBuffer(self):
        """
        Clear project buffer
        """

        self.javaObj.clearProjectBuffer()

    def initProjectBuffer(self, number):
        """
        Initilize project buffer

        Parameters
        ----------
        number : int
            Number of individuals
        """

        self.javaObj.initProjectBuffer(number)

    def isInProgress(self, individual):
        """
        Is individual with variables in progrss (cumputing or bufffer)

        Parameters
        ----------
        individual : SymIndividual
            Individual with variables

        Returns
        -------
        Is in progress
        """

        return self.javaObj.isInProgress(individual.javaObj)


class SymXLSWrite:

    @staticmethod
    def writeXLSReport(problem: SymProblem, file: str, ind: list = None):
        """
        Write pareto-front to excel

        Parameters
        ----------
        problem : SymProblem
        file : str
        ind : list[SymIndividual]
            list of SymIndividuals

        Returns
        -------
        xls_file : str
        """
        jfile = JFile(file)
        if ind is None:
            xls_file = JXLSWrite.writeXLSReport(problem.javaObj, jfile)

        elif isinstance(ind, list) or isinstance(ind, tuple):
            jind = JArray.newInstance(JIndividual, len(ind))
            for k in range(0, len(ind)):
                jind[k] = ind[k].javaObj

            xls_file = JXLSWrite.writeXLSReport(problem.javaObj, jfile, ind)
        else:
            raise TypeError('Wrong type for individuals: Expected type list, got {} instead'.format(type(ind)))
        return xls_file

    @classmethod
    def exportMultipleXLSReports(cls, prj_file: str, xls_dir: str, archive_names: tuple = None,
                                 problem_names: tuple = None):
        """
        Export pareto-fronts of multiple problems to excel

        This method is no wrapper but an addition and has no java implementation!

        Examples
        --------

        file = 'x:\\Optimierung.opt.mop'
        xls_dir = 'x:\\Optimierung'
        SymXLSWrite.exportMultipleXLSReports(file, xls_dir, problem_names=('p0',))

        Parameters
        ----------
        prj_file : str
            project file
        xls_dir : str
            xls output directory
        archive_names : tuple[str]
            Archive names
        problem_names : tuple[str]
            Problem names
        """

        if archive_names is None:
            '''use all existing archive names of this project'''
            tmp_archive_names = SymArchive.listArchives(prj_file)
        else:
            tmp_archive_names = archive_names

        os.makedirs(xls_dir)

        for a_name in tmp_archive_names:
            '''loop over archive names'''
            try:

                archive = SymArchive.load(prj_file, a_name)

                if problem_names is None:
                    '''use all existing problem names of this archive'''
                    tmp_problem_names = archive.listProblems()
                else:
                    tmp_problem_names = problem_names

                for p_name in tmp_problem_names:
                    '''loop over problem names'''
                    try:
                        xls_file = os.path.join(xls_dir, 'pareto_{}_{}.xlsx'.format(a_name, p_name))

                        problem = archive.readProblem(p_name)
                        cls.writeXLSReport(problem, xls_file)
                        print('Saved file:', xls_file)
                    except Exception as exp:
                        '''Exception is caught if the archive / problem name does not exist'''
                        print('Archive {}  Problem {} Exception:{}'.format(a_name, p_name, str(exp)))

            except Exception as exa:
                '''Exception is caught if the archive name does not exist'''
                print('Archive {} Exception:{}'.format(a_name, str(exa)))

    @staticmethod
    def plotComparison(xls_dir: str, x_labels: list, y_labels: list, x_limit: list = None, y_limit: list = None,
                       size: list = None, use_subfolders=False, colors: list = None):
        """
        Plot comparison of problems from excel export

        This method is no wrapper but an addition and has no java implementation!

        Parameters
        ----------

        xls_dir : str
            Directory with pareto-front excel export files
        x_labels : list[str]
            list x-axis labels to plot
        y_labels : list[str]
            list y-axis labels to plot
        x_limit : list[list]
            list x-axis-limits to plot
        y_limit : list[list]
            list y-axis-limits to plot
        size : list[int]
            Size of the subplot grid [rows, cols]
        use_subfolders : bool
            Search for files in subfolders
        colors: list
            List of RGB color tuples


        Examples
        --------

        xls_dir = 'x:\\Optimierung'
        y_limit = [(1600, 1800) for _ in range(3)]
        x_labels = ['Rate gesamt (MIN)', 'Auslastung AF', 'Auslastung IF']
        y_labels = ['Kraft F1 gesamt', 'Kraft F1 gesamt', 'Kraft F1 gesamt']

        plt = SymXLSWrite.plotComparison(xls_dir, x_labels,_y_labels, y_limit=y_limit, size=[2, 2])
        plt.show()

        Returns
        -------
        SymSubPlot

        """
        '''find all matching xlsx files in the folder'''
        if use_subfolders:
            xls_files = glob.glob(os.path.join(xls_dir, '*', 'pareto_*.xlsx'), recursive=True)
        else:
            xls_files = glob.glob(os.path.join(xls_dir, 'pareto_*.xlsx'), recursive=False)
        xls_files.sort()

        n_plots = len(x_labels)
        if x_limit is None:
            x_limit = [None for _ in range(n_plots)]
        if y_limit is None:
            y_limit = [None for _ in range(n_plots)]

        sym_subplot = SymSubPlot(n_plots, 1)
        for index in range(n_plots):
            sym_subplot[index, 0] = SymPlot2Scatter()

        for file_index, xls_file in enumerate(xls_files):
            df = pd.read_excel(xls_file)
            pareto_size = df[x_labels[0]].size

            '''deduce a label name from the filename'''
            label = os.path.basename(xls_file)
            label = label.replace('pareto_', '')
            label = label.replace('.xlsx', '')
            label = '{} ({:d})'.format(label, pareto_size)

            for index in range(n_plots):
                xlim = x_limit[index]
                ylim = y_limit[index]
                x_label = x_labels[index]
                y_label = y_labels[index]

                sym_subplot[index, 0].addSignal(label, list(df[x_label]), list(df[y_label]))

                if colors is not None:
                    style = sym_subplot[index, 0].getSignal(file_index).getStyle()
                    '''use color_order only if plot has multiple signals'''
                    style.setColor(JColor(*colors[file_index]))
                    style.setMarker(JMarker.CIRLE)

                sym_subplot[index, 0].setXLabel(x_label)
                sym_subplot[index, 0].setYLabel(y_label)

                if xlim is not None:
                    sym_subplot[index, 0].setXLimit(*xlim)
                if ylim is not None:
                    sym_subplot[index, 0].setYLimit(*ylim)

                sym_subplot[index, 0].setLegend(True)

        if size is None:
            sym_subplot2 = sym_subplot
        else:
            '''try to resize the plot-grid if size is given'''
            sym_subplot2 = SymSubPlot(size[0], size[1])
            index = 0
            for row in range(size[0]):
                for col in range(size[1]):
                    try:
                        sym_subplot2[row, col] = sym_subplot[index, 0]
                        index += 1
                    except Exception as ex:
                        print(ex)

        return sym_subplot2

    @staticmethod
    def plotHistogramm(xls_dir: str, hist_labels: list, size: tuple = None, use_subfolders: bool = False,
                       colors: list = None):
        """
        Plot variable histogram of problems from excel export

        This method is no wrapper but an addition and has no java implementation!

        Parameters
        ----------
        xls_dir : str
            Directory with pareto-front excel export files
        hist_labels : list[str]
            list y-axis labels to plot
        size : tuple[int]
            Size of the subplot grid [rows, cols]
        use_subfolders : bool
            Search for files in subfolders
        colors: list
            List of RGB color tuples


        Examples
        --------

        xls_dir = 'x:\\Optimierung'
        hist_labels = ['AF Drahtqualität', 'AF Draht-Profil', 'AF Strahlen', 'AF Draht d']

        plt = SymXLSWrite.plotComparison(xls_dir, hist_labels, size=[2, 2])
        plt.show()

        Returns
        -------
        SymSubPlot

        """

        '''find all matching xlsx files in the folder'''
        if use_subfolders:
            xls_files = glob.glob(os.path.join(xls_dir, '*', 'pareto_*.xlsx'), recursive=True)
        else:
            xls_files = glob.glob(os.path.join(xls_dir, 'pareto_*.xlsx'), recursive=False)
        xls_files.sort()

        n_plots = len(hist_labels)

        sym_subplot = SymSubPlot(n_plots, 1)
        for index in range(n_plots):
            sym_subplot[index, 0] = SymPlot2Hist()

        for file_index, xls_file in enumerate(xls_files):
            df = pd.read_excel(xls_file)
            pareto_size = df[hist_labels[0]].size

            '''deduce a label name from the filename'''
            label = os.path.basename(xls_file)
            label = label.replace('pareto_', '')
            label = label.replace('.xlsx', '')
            label = '{} ({:d})'.format(label, pareto_size)

            for index in range(n_plots):
                y_label = hist_labels[index]

                sym_subplot[index, 0].addSignal(label, list(df[y_label]))
                if colors is not None:
                    style = sym_subplot[index, 0].getSignal(file_index).getStyle()
                    '''use color_order only if plot has multiple signals'''
                    style.setColor(JColor(*colors[file_index]))
                sym_subplot[index, 0].setXLabel(y_label)
                sym_subplot[index, 0].setYLabel('Number of values')

                sym_subplot[index, 0].setLegend(True)

        if size is None:
            sym_subplot2 = sym_subplot
        else:
            '''try to resize the plot-grid if size is given'''
            sym_subplot2 = SymSubPlot(size[0], size[1])
            index = 0
            for row in range(size[0]):
                for col in range(size[1]):
                    try:
                        sym_subplot2[row, col] = sym_subplot[index, 0]
                        index += 1
                    except Exception as ex:
                        print(ex)

        return sym_subplot2
