/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9828 $
 * $Date: 2020-05-14 14:11:09 +0200 (Thu, 14 May 2020) $
 */
package jobmanager.server.local;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.LinkedList;

import at.lcm.bu21.magopt.utils.MagOptConfig;
import jobmanager.JobStatus;
import jobmanager.JobUniverse;
import jobmanager.server.Job;
import jobmanager.server.JobAdapter;
import magopt.utils.Execute;

/**
 * Adapter for local execution
 * 
 * @author Werner Koppelsttter
 */
public class LocalAdapter extends JobAdapter {

	/** Client for local execution */
	static LocalClient client;

	/** Handle for execution */
	private Execute exec = null;

	/** Start time of */
	private long start;

	/**
	 * Add job to local execution queue
	 * 
	 * @param job
	 *            Job to execute
	 */
	LocalAdapter(Job job) {
		this.job = job;
		this.job.setStatus(JobStatus.SUBMITTED);
	}

	/**
	 * Update object status
	 */
	void update() {

		// run job
		if (this.job.getStatus() == JobStatus.SUBMITTED) {
			this.runJob();
		}

		// update status
		if (this.job.getStatus() == JobStatus.RUNNING) {
			this.job.setRuntime((int) ((System.currentTimeMillis() - this.start) / 1000));
			this.job.setHost("localhost");
			switch (this.exec.getState()) {
			case FINISHED:
				if (this.exec.exitValue() == 0) {
					this.job.setStatus(JobStatus.COMPLETE);
				} else {
					this.job.addErrorLog("Error on local host\t" + this.exec.exitValue());
					this.job.setStatus(JobStatus.ERROR);
				}
				break;
			case TIMEOUT:
				this.job.addErrorLog("Timeout on local host");
				this.job.setStatus(JobStatus.TIMEOUT);
				break;
			case KILLED:
				this.job.setStatus(JobStatus.STOP);
				break;
			default:
				break;
			}
		}

		// remove from local-queue
		switch (this.job.getStatus()) {
		case COMPLETE:
		case TIMEOUT:
		case UNDEFINED:
		case REMOVED:
		case ERROR:
		case STOP:
		case RESTART:
			this.stop();
			break;
		default:
			break;
		}

	}

	/**
	 * Run job as local process
	 */
	private void runJob() {
		boolean jobDebug = false;
		String val;
		LinkedList<String> cmd = new LinkedList<String>();
		String config;
		String[] vmArgs;
		BufferedWriter out;
		File path;
		String command;

		val = MagOptConfig.getConfig("JobDebug");
		if (val != null) {
			jobDebug = Boolean.parseBoolean(val);
		}

		// TODO Linux
		// windows: cmd /c start /belownormal /b /wait ...
		// linux: nice -n 10 ...
		if (System.getProperty("os.name").equalsIgnoreCase("Linux")) {
			cmd.add("nice");
			cmd.add("-n");
			cmd.add("10");
		} else {
			cmd.add("cmd");
			cmd.add("/c");
			cmd.add("start");
			cmd.add("/belownormal");
			cmd.add("/b");
			cmd.add("/wait");
			cmd.add("Local Job"); // name of window
		}
		command = this.job.getCommand();
		// add java-path
		if (command.toLowerCase().startsWith("java")) {
			cmd.add(new File(new File(System.getProperty("java.home"), "bin"), command).getPath());
		} else {
			cmd.add(command);
		}
		if ((this.job.getUniverse() == JobUniverse.JAVA) || command.toLowerCase().startsWith("java")) {
			// add java argments
			config = MagOptConfig.getConfig("JavaArgsLocalJob");
			if (config != null) {
				vmArgs = config.split(" ");
				for (String str : vmArgs) {
					if (!str.isEmpty()) {
						cmd.add(str);
					}
				}
			}
			cmd.add("-cp");
			cmd.add(this.job.getLibraryFiles(File.pathSeparator, true));
		}

		for (String str : this.job.getArguments()) {
			cmd.add(str);
		}

		if (jobDebug) {
			try {

//				ads.append("'Out':'__job_Out__.txt', ");
//				ads.append("'Err':'__job_Err__.txt', ");
//				ads.append("'UserLog':'__job_Log__.txt', ");

				out = new BufferedWriter(new FileWriter(new File(this.job.getJobDirectory(), "__job_settings__.txt")));
//				out.write("user: " + System.getProperty("user.name") + "\n");
				out.write("Universe: " + this.job.getUniverse() + "\n");
//				out.write("Requirement: " + this.job.getRequirement() + "\n");
//				out.write("DockerImage: " + this.job.getDockerImage() + "\n");
				out.write("Command: " + this.job.getCommand() + "\n");
				out.write("Cmd: " + cmd.toString() + "\n");
				out.write("Iwd: " + this.job.getJobDirectory().getPath() + "\n");
				out.write("TransferInput: " + this.job.getTransferFiles(",") + "\n");
				out.write("JarFiles: " + this.job.getLibraryFiles(",", true) + "\n");
//				out.write("Rank: " + "JavaMFlops*(1-0.75*SlotID/TotalSlots)" + "\n");
//				out.write("NTDomain: " + new NTSystem().getDomain() + "\n");
				out.write("JobIdentifier: " + this.job.getIdentifier() + "\n");

				out.close();
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}

		// run Job
		this.exec = new Execute(cmd.toArray(new String[0]));
		this.exec.setDirectory(this.job.getJobDirectory());
		this.exec.setTimeout(this.job.getTimeout() * 1000);
		this.exec.setDiscardStreams(true);
		this.exec.start();
		this.job.setStatus(JobStatus.RUNNING);
		this.start = System.currentTimeMillis();
	}

	/**
	 * Stop from local execution queue
	 */
	private void stop() {
		if (this.exec != null) {
			this.exec.kill();
		}
		LocalAdapter.client.removeJob(this.job);
		this.job.clearAdapter();
	}

	@Override
	public String getID() {
		String str;
		str = "Local: ";
		if (this.exec == null) {
			str += 0;
		} else {
			str += this.exec.getPID();
		}

		return str;
	}
}
